'use strict';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

/**
 * Adapted from React: https://github.com/facebook/react/blob/master/packages/shared/formatProdErrorMessage.js
 *
 * Do not require this module directly! Use normal throw error calls. These messages will be replaced with error codes
 * during build.
 * @param {number} code
 */
function formatProdErrorMessage(code) {
  return "Minified Redux error #" + code + "; visit https://redux.js.org/Errors?code=" + code + " for the full message or " + 'use the non-minified dev environment for full errors. ';
}

// Inlined version of the `symbol-observable` polyfill
var $$observable = (function () {
  return typeof Symbol === 'function' && Symbol.observable || '@@observable';
})();

/**
 * These are private action types reserved by Redux.
 * For any unknown actions, you must return the current state.
 * If the current state is undefined, you must return the initial state.
 * Do not reference these action types directly in your code.
 */
var randomString = function randomString() {
  return Math.random().toString(36).substring(7).split('').join('.');
};

var ActionTypes = {
  INIT: "@@redux/INIT" + randomString(),
  REPLACE: "@@redux/REPLACE" + randomString(),
  PROBE_UNKNOWN_ACTION: function PROBE_UNKNOWN_ACTION() {
    return "@@redux/PROBE_UNKNOWN_ACTION" + randomString();
  }
};

/**
 * @param {any} obj The object to inspect.
 * @returns {boolean} True if the argument appears to be a plain object.
 */
function isPlainObject(obj) {
  if (typeof obj !== 'object' || obj === null) return false;
  var proto = obj;

  while (Object.getPrototypeOf(proto) !== null) {
    proto = Object.getPrototypeOf(proto);
  }

  return Object.getPrototypeOf(obj) === proto;
}

/**
 * @deprecated
 *
 * **We recommend using the `configureStore` method
 * of the `@reduxjs/toolkit` package**, which replaces `createStore`.
 *
 * Redux Toolkit is our recommended approach for writing Redux logic today,
 * including store setup, reducers, data fetching, and more.
 *
 * **For more details, please read this Redux docs page:**
 * **https://redux.js.org/introduction/why-rtk-is-redux-today**
 *
 * `configureStore` from Redux Toolkit is an improved version of `createStore` that
 * simplifies setup and helps avoid common bugs.
 *
 * You should not be using the `redux` core package by itself today, except for learning purposes.
 * The `createStore` method from the core `redux` package will not be removed, but we encourage
 * all users to migrate to using Redux Toolkit for all Redux code.
 *
 * If you want to use `createStore` without this visual deprecation warning, use
 * the `legacy_createStore` import instead:
 *
 * `import { legacy_createStore as createStore} from 'redux'`
 *
 */

function createStore(reducer, preloadedState, enhancer) {
  var _ref2;

  if (typeof preloadedState === 'function' && typeof enhancer === 'function' || typeof enhancer === 'function' && typeof arguments[3] === 'function') {
    throw new Error(formatProdErrorMessage(0) );
  }

  if (typeof preloadedState === 'function' && typeof enhancer === 'undefined') {
    enhancer = preloadedState;
    preloadedState = undefined;
  }

  if (typeof enhancer !== 'undefined') {
    if (typeof enhancer !== 'function') {
      throw new Error(formatProdErrorMessage(1) );
    }

    return enhancer(createStore)(reducer, preloadedState);
  }

  if (typeof reducer !== 'function') {
    throw new Error(formatProdErrorMessage(2) );
  }

  var currentReducer = reducer;
  var currentState = preloadedState;
  var currentListeners = [];
  var nextListeners = currentListeners;
  var isDispatching = false;
  /**
   * This makes a shallow copy of currentListeners so we can use
   * nextListeners as a temporary list while dispatching.
   *
   * This prevents any bugs around consumers calling
   * subscribe/unsubscribe in the middle of a dispatch.
   */

  function ensureCanMutateNextListeners() {
    if (nextListeners === currentListeners) {
      nextListeners = currentListeners.slice();
    }
  }
  /**
   * Reads the state tree managed by the store.
   *
   * @returns {any} The current state tree of your application.
   */


  function getState() {
    if (isDispatching) {
      throw new Error(formatProdErrorMessage(3) );
    }

    return currentState;
  }
  /**
   * Adds a change listener. It will be called any time an action is dispatched,
   * and some part of the state tree may potentially have changed. You may then
   * call `getState()` to read the current state tree inside the callback.
   *
   * You may call `dispatch()` from a change listener, with the following
   * caveats:
   *
   * 1. The subscriptions are snapshotted just before every `dispatch()` call.
   * If you subscribe or unsubscribe while the listeners are being invoked, this
   * will not have any effect on the `dispatch()` that is currently in progress.
   * However, the next `dispatch()` call, whether nested or not, will use a more
   * recent snapshot of the subscription list.
   *
   * 2. The listener should not expect to see all state changes, as the state
   * might have been updated multiple times during a nested `dispatch()` before
   * the listener is called. It is, however, guaranteed that all subscribers
   * registered before the `dispatch()` started will be called with the latest
   * state by the time it exits.
   *
   * @param {Function} listener A callback to be invoked on every dispatch.
   * @returns {Function} A function to remove this change listener.
   */


  function subscribe(listener) {
    if (typeof listener !== 'function') {
      throw new Error(formatProdErrorMessage(4) );
    }

    if (isDispatching) {
      throw new Error(formatProdErrorMessage(5) );
    }

    var isSubscribed = true;
    ensureCanMutateNextListeners();
    nextListeners.push(listener);
    return function unsubscribe() {
      if (!isSubscribed) {
        return;
      }

      if (isDispatching) {
        throw new Error(formatProdErrorMessage(6) );
      }

      isSubscribed = false;
      ensureCanMutateNextListeners();
      var index = nextListeners.indexOf(listener);
      nextListeners.splice(index, 1);
      currentListeners = null;
    };
  }
  /**
   * Dispatches an action. It is the only way to trigger a state change.
   *
   * The `reducer` function, used to create the store, will be called with the
   * current state tree and the given `action`. Its return value will
   * be considered the **next** state of the tree, and the change listeners
   * will be notified.
   *
   * The base implementation only supports plain object actions. If you want to
   * dispatch a Promise, an Observable, a thunk, or something else, you need to
   * wrap your store creating function into the corresponding middleware. For
   * example, see the documentation for the `redux-thunk` package. Even the
   * middleware will eventually dispatch plain object actions using this method.
   *
   * @param {Object} action A plain object representing “what changed”. It is
   * a good idea to keep actions serializable so you can record and replay user
   * sessions, or use the time travelling `redux-devtools`. An action must have
   * a `type` property which may not be `undefined`. It is a good idea to use
   * string constants for action types.
   *
   * @returns {Object} For convenience, the same action object you dispatched.
   *
   * Note that, if you use a custom middleware, it may wrap `dispatch()` to
   * return something else (for example, a Promise you can await).
   */


  function dispatch(action) {
    if (!isPlainObject(action)) {
      throw new Error(formatProdErrorMessage(7) );
    }

    if (typeof action.type === 'undefined') {
      throw new Error(formatProdErrorMessage(8) );
    }

    if (isDispatching) {
      throw new Error(formatProdErrorMessage(9) );
    }

    try {
      isDispatching = true;
      currentState = currentReducer(currentState, action);
    } finally {
      isDispatching = false;
    }

    var listeners = currentListeners = nextListeners;

    for (var i = 0; i < listeners.length; i++) {
      var listener = listeners[i];
      listener();
    }

    return action;
  }
  /**
   * Replaces the reducer currently used by the store to calculate the state.
   *
   * You might need this if your app implements code splitting and you want to
   * load some of the reducers dynamically. You might also need this if you
   * implement a hot reloading mechanism for Redux.
   *
   * @param {Function} nextReducer The reducer for the store to use instead.
   * @returns {void}
   */


  function replaceReducer(nextReducer) {
    if (typeof nextReducer !== 'function') {
      throw new Error(formatProdErrorMessage(10) );
    }

    currentReducer = nextReducer; // This action has a similiar effect to ActionTypes.INIT.
    // Any reducers that existed in both the new and old rootReducer
    // will receive the previous state. This effectively populates
    // the new state tree with any relevant data from the old one.

    dispatch({
      type: ActionTypes.REPLACE
    });
  }
  /**
   * Interoperability point for observable/reactive libraries.
   * @returns {observable} A minimal observable of state changes.
   * For more information, see the observable proposal:
   * https://github.com/tc39/proposal-observable
   */


  function observable() {
    var _ref;

    var outerSubscribe = subscribe;
    return _ref = {
      /**
       * The minimal observable subscription method.
       * @param {Object} observer Any object that can be used as an observer.
       * The observer object should have a `next` method.
       * @returns {subscription} An object with an `unsubscribe` method that can
       * be used to unsubscribe the observable from the store, and prevent further
       * emission of values from the observable.
       */
      subscribe: function subscribe(observer) {
        if (typeof observer !== 'object' || observer === null) {
          throw new Error(formatProdErrorMessage(11) );
        }

        function observeState() {
          if (observer.next) {
            observer.next(getState());
          }
        }

        observeState();
        var unsubscribe = outerSubscribe(observeState);
        return {
          unsubscribe: unsubscribe
        };
      }
    }, _ref[$$observable] = function () {
      return this;
    }, _ref;
  } // When a store is created, an "INIT" action is dispatched so that every
  // reducer returns their initial state. This effectively populates
  // the initial state tree.


  dispatch({
    type: ActionTypes.INIT
  });
  return _ref2 = {
    dispatch: dispatch,
    subscribe: subscribe,
    getState: getState,
    replaceReducer: replaceReducer
  }, _ref2[$$observable] = observable, _ref2;
}

function assertReducerShape(reducers) {
  Object.keys(reducers).forEach(function (key) {
    var reducer = reducers[key];
    var initialState = reducer(undefined, {
      type: ActionTypes.INIT
    });

    if (typeof initialState === 'undefined') {
      throw new Error(formatProdErrorMessage(12) );
    }

    if (typeof reducer(undefined, {
      type: ActionTypes.PROBE_UNKNOWN_ACTION()
    }) === 'undefined') {
      throw new Error(formatProdErrorMessage(13) );
    }
  });
}
/**
 * Turns an object whose values are different reducer functions, into a single
 * reducer function. It will call every child reducer, and gather their results
 * into a single state object, whose keys correspond to the keys of the passed
 * reducer functions.
 *
 * @param {Object} reducers An object whose values correspond to different
 * reducer functions that need to be combined into one. One handy way to obtain
 * it is to use ES6 `import * as reducers` syntax. The reducers may never return
 * undefined for any action. Instead, they should return their initial state
 * if the state passed to them was undefined, and the current state for any
 * unrecognized action.
 *
 * @returns {Function} A reducer function that invokes every reducer inside the
 * passed object, and builds a state object with the same shape.
 */


function combineReducers(reducers) {
  var reducerKeys = Object.keys(reducers);
  var finalReducers = {};

  for (var i = 0; i < reducerKeys.length; i++) {
    var key = reducerKeys[i];

    if (typeof reducers[key] === 'function') {
      finalReducers[key] = reducers[key];
    }
  }

  var finalReducerKeys = Object.keys(finalReducers); // This is used to make sure we don't warn about the same

  var shapeAssertionError;

  try {
    assertReducerShape(finalReducers);
  } catch (e) {
    shapeAssertionError = e;
  }

  return function combination(state, action) {
    if (state === void 0) {
      state = {};
    }

    if (shapeAssertionError) {
      throw shapeAssertionError;
    }

    var hasChanged = false;
    var nextState = {};

    for (var _i = 0; _i < finalReducerKeys.length; _i++) {
      var _key = finalReducerKeys[_i];
      var reducer = finalReducers[_key];
      var previousStateForKey = state[_key];
      var nextStateForKey = reducer(previousStateForKey, action);

      if (typeof nextStateForKey === 'undefined') {
        throw new Error(formatProdErrorMessage(14) );
      }

      nextState[_key] = nextStateForKey;
      hasChanged = hasChanged || nextStateForKey !== previousStateForKey;
    }

    hasChanged = hasChanged || finalReducerKeys.length !== Object.keys(state).length;
    return hasChanged ? nextState : state;
  };
}

/**
 * Composes single-argument functions from right to left. The rightmost
 * function can take multiple arguments as it provides the signature for
 * the resulting composite function.
 *
 * @param {...Function} funcs The functions to compose.
 * @returns {Function} A function obtained by composing the argument functions
 * from right to left. For example, compose(f, g, h) is identical to doing
 * (...args) => f(g(h(...args))).
 */
function compose() {
  for (var _len = arguments.length, funcs = new Array(_len), _key = 0; _key < _len; _key++) {
    funcs[_key] = arguments[_key];
  }

  if (funcs.length === 0) {
    return function (arg) {
      return arg;
    };
  }

  if (funcs.length === 1) {
    return funcs[0];
  }

  return funcs.reduce(function (a, b) {
    return function () {
      return a(b.apply(void 0, arguments));
    };
  });
}

/**
 * Creates a store enhancer that applies middleware to the dispatch method
 * of the Redux store. This is handy for a variety of tasks, such as expressing
 * asynchronous actions in a concise manner, or logging every action payload.
 *
 * See `redux-thunk` package as an example of the Redux middleware.
 *
 * Because middleware is potentially asynchronous, this should be the first
 * store enhancer in the composition chain.
 *
 * Note that each middleware will be given the `dispatch` and `getState` functions
 * as named arguments.
 *
 * @param {...Function} middlewares The middleware chain to be applied.
 * @returns {Function} A store enhancer applying the middleware.
 */

function applyMiddleware() {
  for (var _len = arguments.length, middlewares = new Array(_len), _key = 0; _key < _len; _key++) {
    middlewares[_key] = arguments[_key];
  }

  return function (createStore) {
    return function () {
      var store = createStore.apply(void 0, arguments);

      var _dispatch = function dispatch() {
        throw new Error(formatProdErrorMessage(15) );
      };

      var middlewareAPI = {
        getState: store.getState,
        dispatch: function dispatch() {
          return _dispatch.apply(void 0, arguments);
        }
      };
      var chain = middlewares.map(function (middleware) {
        return middleware(middlewareAPI);
      });
      _dispatch = compose.apply(void 0, chain)(store.dispatch);
      return _objectSpread2(_objectSpread2({}, store), {}, {
        dispatch: _dispatch
      });
    };
  };
}

function oe(e){for(var t=arguments.length,r=Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];throw Error("[Immer] minified error nr: "+e+(r.length?" "+r.map(function(i){return "'"+i+"'"}).join(","):"")+". Find the full error at: https://bit.ly/3cXEKWf")}function Re(e){return !!e&&!!e[z]}function Ce(e){var t;return !!e&&(function(r){if(!r||typeof r!="object")return !1;var n=Object.getPrototypeOf(r);if(n===null)return !0;var o=Object.hasOwnProperty.call(n,"constructor")&&n.constructor;return o===Object||typeof o=="function"&&Function.toString.call(o)===gb}(e)||Array.isArray(e)||!!e[Gs]||!!(!((t=e.constructor)===null||t===void 0)&&t[Gs])||Bo(e)||Go(e))}function Je(e,t,r){r===void 0&&(r=!1),Ot(e)===0?(r?Object.keys:Ct)(e).forEach(function(n){r&&typeof n=="symbol"||t(n,e[n],e);}):e.forEach(function(n,o){return t(o,n,e)});}function Ot(e){var t=e[z];return t?t.i>3?t.i-4:t.i:Array.isArray(e)?1:Bo(e)?2:Go(e)?3:0}function St(e,t){return Ot(e)===2?e.has(t):Object.prototype.hasOwnProperty.call(e,t)}function ib(e,t){return Ot(e)===2?e.get(t):e[t]}function Ls(e,t,r){var n=Ot(e);n===2?e.set(t,r):n===3?e.add(r):e[t]=r;}function Ns(e,t){return e===t?e!==0||1/e==1/t:e!=e&&t!=t}function Bo(e){return pb&&e instanceof Map}function Go(e){return fb&&e instanceof Set}function Me(e){return e.o||e.t}function zo(e){if(Array.isArray(e))return Array.prototype.slice.call(e);var t=zs(e);delete t[z];for(var r=Ct(t),n=0;n<r.length;n++){var o=r[n],a=t[o];a.writable===!1&&(a.writable=!0,a.configurable=!0),(a.get||a.set)&&(t[o]={configurable:!0,writable:!0,enumerable:a.enumerable,value:e[o]});}return Object.create(Object.getPrototypeOf(e),t)}function Wo(e,t){return t===void 0&&(t=!1),Ho(e)||Re(e)||!Ce(e)||(Ot(e)>1&&(e.set=e.add=e.clear=e.delete=sb),Object.freeze(e),t&&Je(e,function(r,n){return Wo(n,!0)},!0)),e}function sb(){oe(2);}function Ho(e){return e==null||typeof e!="object"||Object.isFrozen(e)}function we(e){var t=na[e];return t||oe(18,e),t}function cb(e,t){na[e]||(na[e]=t);}function Zo(){return jt}function Ko(e,t){t&&(we("Patches"),e.u=[],e.s=[],e.v=t);}function dn(e){Yo(e),e.p.forEach(ub),e.p=null;}function Yo(e){e===jt&&(jt=e.l);}function Rs(e){return jt={p:[],l:jt,h:e,m:!0,_:0}}function ub(e){var t=e[z];t.i===0||t.i===1?t.j():t.O=!0;}function Jo(e,t){t._=t.p.length;var r=t.p[0],n=e!==void 0&&e!==r;return t.h.g||we("ES5").S(t,e,n),n?(r[z].P&&(dn(t),oe(4)),Ce(e)&&(e=pn(t,e),t.l||fn(t,e)),t.u&&we("Patches").M(r[z].t,e,t.u,t.s)):e=pn(t,r,[]),dn(t),t.u&&t.v(t.u,t.s),e!==Bs?e:void 0}function pn(e,t,r){if(Ho(t))return t;var n=t[z];if(!n)return Je(t,function(s,c){return Ms(e,n,t,s,c,r)},!0),t;if(n.A!==e)return t;if(!n.P)return fn(e,n.t,!0),n.t;if(!n.I){n.I=!0,n.A._--;var o=n.i===4||n.i===5?n.o=zo(n.k):n.o,a=o,i=!1;n.i===3&&(a=new Set(o),o.clear(),i=!0),Je(a,function(s,c){return Ms(e,n,o,s,c,r,i)}),fn(e,o,!1),r&&e.u&&we("Patches").N(n,r,e.u,e.s);}return n.o}function Ms(e,t,r,n,o,a,i){if(Re(o)){var s=pn(e,o,a&&t&&t.i!==3&&!St(t.R,n)?a.concat(n):void 0);if(Ls(r,n,s),!Re(s))return;e.m=!1;}else i&&r.add(o);if(Ce(o)&&!Ho(o)){if(!e.h.D&&e._<1)return;pn(e,o),t&&t.A.l||fn(e,o);}}function fn(e,t,r){r===void 0&&(r=!1),!e.l&&e.h.D&&e.m&&Wo(t,r);}function Xo(e,t){var r=e[z];return (r?Me(r):e)[t]}function Vs(e,t){if(t in e)for(var r=Object.getPrototypeOf(e);r;){var n=Object.getOwnPropertyDescriptor(r,t);if(n)return n;r=Object.getPrototypeOf(r);}}function Ve(e){e.P||(e.P=!0,e.l&&Ve(e.l));}function ea(e){e.o||(e.o=zo(e.t));}function ta(e,t,r){var n=Bo(t)?we("MapSet").F(t,r):Go(t)?we("MapSet").T(t,r):e.g?function(o,a){var i=Array.isArray(o),s={i:i?1:0,A:a?a.A:Zo(),P:!1,I:!1,R:{},l:a,t:o,k:null,o:null,j:null,C:!1},c=s,u=mr;i&&(c=[s],u=br);var l=Proxy.revocable(c,u),d=l.revoke,f=l.proxy;return s.k=f,s.j=d,f}(t,r):we("ES5").J(t,r);return (r?r.A:Zo()).p.push(n),n}function lb(e){return Re(e)||oe(22,e),function t(r){if(!Ce(r))return r;var n,o=r[z],a=Ot(r);if(o){if(!o.P&&(o.i<4||!we("ES5").K(o)))return o.t;o.I=!0,n=Fs(r,a),o.I=!1;}else n=Fs(r,a);return Je(n,function(i,s){o&&ib(o.t,i)===s||Ls(n,i,t(s));}),a===3?new Set(n):n}(e)}function Fs(e,t){switch(t){case 2:return new Map(e);case 3:return Array.from(e)}return zo(e)}function db(){function e(i,s){var c=a[i];return c?c.enumerable=s:a[i]=c={configurable:!0,enumerable:s,get:function(){var u=this[z];return mr.get(u,i)},set:function(u){var l=this[z];mr.set(l,i,u);}},c}function t(i){for(var s=i.length-1;s>=0;s--){var c=i[s][z];if(!c.P)switch(c.i){case 5:n(c)&&Ve(c);break;case 4:r(c)&&Ve(c);}}}function r(i){for(var s=i.t,c=i.k,u=Ct(c),l=u.length-1;l>=0;l--){var d=u[l];if(d!==z){var f=s[d];if(f===void 0&&!St(s,d))return !0;var g=c[d],v=g&&g[z];if(v?v.t!==f:!Ns(g,f))return !0}}var h=!!s[z];return u.length!==Ct(s).length+(h?0:1)}function n(i){var s=i.k;if(s.length!==i.t.length)return !0;var c=Object.getOwnPropertyDescriptor(s,s.length-1);if(c&&!c.get)return !0;for(var u=0;u<s.length;u++)if(!s.hasOwnProperty(u))return !0;return !1}var a={};cb("ES5",{J:function(i,s){var c=Array.isArray(i),u=function(d,f){if(d){for(var g=Array(f.length),v=0;v<f.length;v++)Object.defineProperty(g,""+v,e(v,!0));return g}var h=zs(f);delete h[z];for(var b=Ct(h),m=0;m<b.length;m++){var _=b[m];h[_]=e(_,d||!!h[_].enumerable);}return Object.create(Object.getPrototypeOf(f),h)}(c,i),l={i:c?5:4,A:s?s.A:Zo(),P:!1,I:!1,R:{},l:s,t:i,k:u,o:null,O:!1,C:!1};return Object.defineProperty(u,z,{value:l,writable:!0}),u},S:function(i,s,c){c?Re(s)&&s[z].A===i&&t(i.p):(i.u&&function u(l){if(l&&typeof l=="object"){var d=l[z];if(d){var f=d.t,g=d.k,v=d.R,h=d.i;if(h===4)Je(g,function(w){w!==z&&(f[w]!==void 0||St(f,w)?v[w]||u(g[w]):(v[w]=!0,Ve(d)));}),Je(f,function(w){g[w]!==void 0||St(g,w)||(v[w]=!1,Ve(d));});else if(h===5){if(n(d)&&(Ve(d),v.length=!0),g.length<f.length)for(var b=g.length;b<f.length;b++)v[b]=!1;else for(var m=f.length;m<g.length;m++)v[m]=!0;for(var _=Math.min(g.length,f.length),A=0;A<_;A++)g.hasOwnProperty(A)||(v[A]=!0),v[A]===void 0&&u(g[A]);}}}}(i.p[0]),t(i.p));},K:function(i){return i.i===4?r(i):n(i)}});}var Qs,jt,ra=typeof Symbol!="undefined"&&typeof Symbol("x")=="symbol",pb=typeof Map!="undefined",fb=typeof Set!="undefined",Us=typeof Proxy!="undefined"&&Proxy.revocable!==void 0&&typeof Reflect!="undefined",Bs=ra?Symbol.for("immer-nothing"):((Qs={})["immer-nothing"]=!0,Qs),Gs=ra?Symbol.for("immer-draftable"):"__$immer_draftable",z=ra?Symbol.for("immer-state"):"__$immer_state",gb=""+Object.prototype.constructor,Ct=typeof Reflect!="undefined"&&Reflect.ownKeys?Reflect.ownKeys:Object.getOwnPropertySymbols!==void 0?function(e){return Object.getOwnPropertyNames(e).concat(Object.getOwnPropertySymbols(e))}:Object.getOwnPropertyNames,zs=Object.getOwnPropertyDescriptors||function(e){var t={};return Ct(e).forEach(function(r){t[r]=Object.getOwnPropertyDescriptor(e,r);}),t},na={},mr={get:function(e,t){if(t===z)return e;var r=Me(e);if(!St(r,t))return function(o,a,i){var s,c=Vs(a,i);return c?"value"in c?c.value:(s=c.get)===null||s===void 0?void 0:s.call(o.k):void 0}(e,r,t);var n=r[t];return e.I||!Ce(n)?n:n===Xo(e.t,t)?(ea(e),e.o[t]=ta(e.A.h,n,e)):n},has:function(e,t){return t in Me(e)},ownKeys:function(e){return Reflect.ownKeys(Me(e))},set:function(e,t,r){var n=Vs(Me(e),t);if(n!=null&&n.set)return n.set.call(e.k,r),!0;if(!e.P){var o=Xo(Me(e),t),a=o==null?void 0:o[z];if(a&&a.t===r)return e.o[t]=r,e.R[t]=!1,!0;if(Ns(r,o)&&(r!==void 0||St(e.t,t)))return !0;ea(e),Ve(e);}return e.o[t]===r&&(r!==void 0||t in e.o)||Number.isNaN(r)&&Number.isNaN(e.o[t])||(e.o[t]=r,e.R[t]=!0),!0},deleteProperty:function(e,t){return Xo(e.t,t)!==void 0||t in e.t?(e.R[t]=!1,ea(e),Ve(e)):delete e.R[t],e.o&&delete e.o[t],!0},getOwnPropertyDescriptor:function(e,t){var r=Me(e),n=Reflect.getOwnPropertyDescriptor(r,t);return n&&{writable:!0,configurable:e.i!==1||t!=="length",enumerable:n.enumerable,value:r[t]}},defineProperty:function(){oe(11);},getPrototypeOf:function(e){return Object.getPrototypeOf(e.t)},setPrototypeOf:function(){oe(12);}},br={};Je(mr,function(e,t){br[e]=function(){return arguments[0]=arguments[0][0],t.apply(this,arguments)};}),br.deleteProperty=function(e,t){return br.set.call(this,e,t,void 0)},br.set=function(e,t,r){return mr.set.call(this,e[0],t,r,e[0])};var hb=function(){function e(r){var n=this;this.g=Us,this.D=!0,this.produce=function(o,a,i){if(typeof o=="function"&&typeof a!="function"){var s=a;a=o;var c=n;return function(h){var b=this;h===void 0&&(h=s);for(var m=arguments.length,_=Array(m>1?m-1:0),A=1;A<m;A++)_[A-1]=arguments[A];return c.produce(h,function(w){var S;return (S=a).call.apply(S,[b,w].concat(_))})}}var u;if(typeof a!="function"&&oe(6),i!==void 0&&typeof i!="function"&&oe(7),Ce(o)){var l=Rs(n),d=ta(n,o,void 0),f=!0;try{u=a(d),f=!1;}finally{f?dn(l):Yo(l);}return typeof Promise!="undefined"&&u instanceof Promise?u.then(function(h){return Ko(l,i),Jo(h,l)},function(h){throw dn(l),h}):(Ko(l,i),Jo(u,l))}if(!o||typeof o!="object"){if((u=a(o))===void 0&&(u=o),u===Bs&&(u=void 0),n.D&&Wo(u,!0),i){var g=[],v=[];we("Patches").M(o,u,g,v),i(g,v);}return u}oe(21,o);},this.produceWithPatches=function(o,a){if(typeof o=="function")return function(u){for(var l=arguments.length,d=Array(l>1?l-1:0),f=1;f<l;f++)d[f-1]=arguments[f];return n.produceWithPatches(u,function(g){return o.apply(void 0,[g].concat(d))})};var i,s,c=n.produce(o,a,function(u,l){i=u,s=l;});return typeof Promise!="undefined"&&c instanceof Promise?c.then(function(u){return [u,i,s]}):[c,i,s]},typeof(r==null?void 0:r.useProxies)=="boolean"&&this.setUseProxies(r.useProxies),typeof(r==null?void 0:r.autoFreeze)=="boolean"&&this.setAutoFreeze(r.autoFreeze);}var t=e.prototype;return t.createDraft=function(r){Ce(r)||oe(8),Re(r)&&(r=lb(r));var n=Rs(this),o=ta(this,r,void 0);return o[z].C=!0,Yo(n),o},t.finishDraft=function(r,n){var o=r&&r[z];var a=o.A;return Ko(a,n),Jo(void 0,a)},t.setAutoFreeze=function(r){this.D=r;},t.setUseProxies=function(r){r&&!Us&&oe(20),this.g=r;},t.applyPatches=function(r,n){var o;for(o=n.length-1;o>=0;o--){var a=n[o];if(a.path.length===0&&a.op==="replace"){r=a.value;break}}o>-1&&(n=n.slice(o+1));var i=we("Patches").$;return Re(r)?i(r,n):this.produce(r,function(s){return i(s,n)})},e}(),ge=new hb,Ws=ge.produce;ge.produceWithPatches.bind(ge),ge.setAutoFreeze.bind(ge),ge.setUseProxies.bind(ge),ge.applyPatches.bind(ge),ge.createDraft.bind(ge),ge.finishDraft.bind(ge);var vn="NOT_FOUND";function mb(e){var t;return {get:function(n){return t&&e(t.key,n)?t.value:vn},put:function(n,o){t={key:n,value:o};},getEntries:function(){return t?[t]:[]},clear:function(){t=void 0;}}}function bb(e,t){var r=[];function n(s){var c=r.findIndex(function(l){return t(s,l.key)});if(c>-1){var u=r[c];return c>0&&(r.splice(c,1),r.unshift(u)),u.value}return vn}function o(s,c){n(s)===vn&&(r.unshift({key:s,value:c}),r.length>e&&r.pop());}function a(){return r}function i(){r=[];}return {get:n,put:o,getEntries:a,clear:i}}var yb=function(t,r){return t===r};function _b(e){return function(r,n){if(r===null||n===null||r.length!==n.length)return !1;for(var o=r.length,a=0;a<o;a++)if(!e(r[a],n[a]))return !1;return !0}}function $b(e,t){var r=typeof t=="object"?t:{equalityCheck:t},n=r.equalityCheck,o=n===void 0?yb:n,a=r.maxSize,i=a===void 0?1:a,s=r.resultEqualityCheck,c=_b(o),u=i===1?mb(c):bb(i,c);function l(){var d=u.get(arguments);if(d===vn){if(d=e.apply(null,arguments),s){var f=u.getEntries(),g=f.find(function(v){return s(v.value,d)});g&&(d=g.value);}u.put(arguments,d);}return d}return l.clearCache=function(){return u.clear()},l}function wb(e){var t=Array.isArray(e[0])?e[0]:e;if(!t.every(function(n){return typeof n=="function"})){var r=t.map(function(n){return typeof n=="function"?"function "+(n.name||"unnamed")+"()":typeof n}).join(", ");throw new Error("createSelector expects all input-selectors to be functions, but received the following types: ["+r+"]")}return t}function Ab(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];var o=function(){for(var i=arguments.length,s=new Array(i),c=0;c<i;c++)s[c]=arguments[c];var u=0,l,d={memoizeOptions:void 0},f=s.pop();if(typeof f=="object"&&(d=f,f=s.pop()),typeof f!="function")throw new Error("createSelector expects an output function after the inputs, but received: ["+typeof f+"]");var g=d,v=g.memoizeOptions,h=v===void 0?r:v,b=Array.isArray(h)?h:[h],m=wb(s),_=e.apply(void 0,[function(){return u++,f.apply(null,arguments)}].concat(b)),A=e(function(){for(var S=[],K=m.length,re=0;re<K;re++)S.push(m[re].apply(null,arguments));return l=_.apply(null,S),l});return Object.assign(A,{resultFunc:f,memoizedResultFunc:_,dependencies:m,lastResult:function(){return l},recomputations:function(){return u},resetRecomputations:function(){return u=0}}),A};return o}var p=Ab($b);function Hs(e){var t=function(n){var o=n.dispatch,a=n.getState;return function(i){return function(s){return typeof s=="function"?s(o,a,e):i(s)}}};return t}var Zs=Hs();Zs.withExtraArgument=Hs;var Ks=Zs,Ob=function(){var e=function(t,r){return e=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(n,o){n.__proto__=o;}||function(n,o){for(var a in o)Object.prototype.hasOwnProperty.call(o,a)&&(n[a]=o[a]);},e(t,r)};return function(t,r){if(typeof r!="function"&&r!==null)throw new TypeError("Class extends value "+String(r)+" is not a constructor or null");e(t,r);function n(){this.constructor=t;}t.prototype=r===null?Object.create(r):(n.prototype=r.prototype,new n);}}(),yr=function(e,t){var r={label:0,sent:function(){if(a[0]&1)throw a[1];return a[1]},trys:[],ops:[]},n,o,a,i;return i={next:s(0),throw:s(1),return:s(2)},typeof Symbol=="function"&&(i[Symbol.iterator]=function(){return this}),i;function s(u){return function(l){return c([u,l])}}function c(u){if(n)throw new TypeError("Generator is already executing.");for(;r;)try{if(n=1,o&&(a=u[0]&2?o.return:u[0]?o.throw||((a=o.return)&&a.call(o),0):o.next)&&!(a=a.call(o,u[1])).done)return a;switch(o=0,a&&(u=[u[0]&2,a.value]),u[0]){case 0:case 1:a=u;break;case 4:return r.label++,{value:u[1],done:!1};case 5:r.label++,o=u[1],u=[0];continue;case 7:u=r.ops.pop(),r.trys.pop();continue;default:if(a=r.trys,!(a=a.length>0&&a[a.length-1])&&(u[0]===6||u[0]===2)){r=0;continue}if(u[0]===3&&(!a||u[1]>a[0]&&u[1]<a[3])){r.label=u[1];break}if(u[0]===6&&r.label<a[1]){r.label=a[1],a=u;break}if(a&&r.label<a[2]){r.label=a[2],r.ops.push(u);break}a[2]&&r.ops.pop(),r.trys.pop();continue}u=t.call(e,r);}catch(l){u=[6,l],o=0;}finally{n=a=0;}if(u[0]&5)throw u[1];return {value:u[0]?u[1]:void 0,done:!0}}},_r=function(e,t){for(var r=0,n=t.length,o=e.length;r<n;r++,o++)e[o]=t[r];return e},Sb=Object.defineProperty,jb=Object.defineProperties,Cb=Object.getOwnPropertyDescriptors,Ys=Object.getOwnPropertySymbols,Pb=Object.prototype.hasOwnProperty,Eb=Object.prototype.propertyIsEnumerable,Js=function(e,t,r){return t in e?Sb(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r},Fe=function(e,t){for(var r in t||(t={}))Pb.call(t,r)&&Js(e,r,t[r]);if(Ys)for(var n=0,o=Ys(t);n<o.length;n++){var r=o[n];Eb.call(t,r)&&Js(e,r,t[r]);}return e},oa=function(e,t){return jb(e,Cb(t))},$r=function(e,t,r){return new Promise(function(n,o){var a=function(c){try{s(r.next(c));}catch(u){o(u);}},i=function(c){try{s(r.throw(c));}catch(u){o(u);}},s=function(c){return c.done?n(c.value):Promise.resolve(c.value).then(a,i)};s((r=r.apply(e,t)).next());})},Ib=typeof window!="undefined"&&window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__?window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__:function(){if(arguments.length!==0)return typeof arguments[0]=="object"?compose:compose.apply(null,arguments)};function Xs(e){if(typeof e!="object"||e===null)return !1;var t=Object.getPrototypeOf(e);if(t===null)return !0;for(var r=t;Object.getPrototypeOf(r)!==null;)r=Object.getPrototypeOf(r);return t===r}var Tb=function(e){Ob(t,e);function t(){for(var r=[],n=0;n<arguments.length;n++)r[n]=arguments[n];var o=e.apply(this,r)||this;return Object.setPrototypeOf(o,t.prototype),o}return Object.defineProperty(t,Symbol.species,{get:function(){return t},enumerable:!1,configurable:!0}),t.prototype.concat=function(){for(var r=[],n=0;n<arguments.length;n++)r[n]=arguments[n];return e.prototype.concat.apply(this,r)},t.prototype.prepend=function(){for(var r=[],n=0;n<arguments.length;n++)r[n]=arguments[n];return r.length===1&&Array.isArray(r[0])?new(t.bind.apply(t,_r([void 0],r[0].concat(this)))):new(t.bind.apply(t,_r([void 0],r.concat(this))))},t}(Array);function aa(e){return Ce(e)?Ws(e,function(){}):e}function sa(e){return typeof e=="boolean"}function Mb(){return function(t){return Vb(t)}}function Vb(e){e===void 0&&(e={});var t=e.thunk,r=t===void 0?!0:t,s=new Tb;if(r&&(sa(r)?s.push(Ks):s.push(Ks.withExtraArgument(r.extraArgument))),"production"!=="production");return s}var ca="production"==="production";function Fb(e){var t=Mb(),r=e||{},n=r.reducer,o=n===void 0?void 0:n,a=r.middleware,i=a===void 0?t():a,s=r.devTools,c=s===void 0?!0:s,u=r.preloadedState,l=u===void 0?void 0:u,d=r.enhancers,f=d===void 0?void 0:d,g;if(typeof o=="function")g=o;else if(Xs(o))g=combineReducers(o);else throw new Error('"reducer" is a required argument, and must be a function or an object of functions that can be passed to combineReducers');var v=i;if(typeof v=="function"&&(v=v(t),!ca))throw new Error("when using a middleware builder function, an array of middleware must be returned");var h=applyMiddleware.apply(void 0,v),b=compose;c&&(b=Ib(Fe({trace:!ca},typeof c=="object"&&c)));var m=[h];Array.isArray(f)?m=_r([h],f):typeof f=="function"&&(m=f(m));var _=b.apply(void 0,m);return createStore(g,l,_)}function J(e,t){function r(){for(var n=[],o=0;o<arguments.length;o++)n[o]=arguments[o];if(t){var a=t.apply(void 0,n);if(!a)throw new Error("prepareAction did not return an object");return Fe(Fe({type:e,payload:a.payload},"meta"in a&&{meta:a.meta}),"error"in a&&{error:a.error})}return {type:e,payload:n[0]}}return r.toString=function(){return ""+e},r.type=e,r.match=function(n){return n.type===e},r}function sc(e){var t={},r=[],n,o={addCase:function(a,i){var s=typeof a=="string"?a:a.type;if(s in t)throw new Error("addCase cannot be called with two reducers for the same action type");return t[s]=i,o},addMatcher:function(a,i){return r.push({matcher:a,reducer:i}),o},addDefaultCase:function(a){return n=a,o}};return e(o),[t,r,n]}function Qb(e){return typeof e=="function"}function Ub(e,t,r,n){r===void 0&&(r=[]);var o=typeof t=="function"?sc(t):[t,r,n],a=o[0],i=o[1],s=o[2],c;if(Qb(e))c=function(){return aa(e())};else {var u=aa(e);c=function(){return u};}function l(d,f){d===void 0&&(d=c());var g=_r([a[f.type]],i.filter(function(v){var h=v.matcher;return h(f)}).map(function(v){var h=v.reducer;return h}));return g.filter(function(v){return !!v}).length===0&&(g=[s]),g.reduce(function(v,h){if(h)if(Re(v)){var b=v,m=h(b,f);return m===void 0?v:m}else {if(Ce(v))return Ws(v,function(_){return h(_,f)});var m=h(v,f);if(m===void 0){if(v===null)return v;throw Error("A case reducer on a non-draftable value must not return undefined")}return m}return v},d)}return l.getInitialState=c,l}function Bb(e,t){return e+"/"+t}function P(e){var t=e.name;if(!t)throw new Error("`name` is a required option for createSlice");typeof process!="undefined"&&"production"==="development"&&e.initialState===void 0&&console.error("You must provide an `initialState` value that is not `undefined`. You may have misspelled `initialState`");var r=typeof e.initialState=="function"?e.initialState:aa(e.initialState),n=e.reducers||{},o=Object.keys(n),a={},i={},s={};o.forEach(function(l){var d=n[l],f=Bb(t,l),g,v;"reducer"in d?(g=d.reducer,v=d.prepare):g=d,a[l]=g,i[f]=g,s[l]=v?J(f,v):J(f);});function c(){var l=typeof e.extraReducers=="function"?sc(e.extraReducers):[e.extraReducers],d=l[0],f=d===void 0?{}:d,g=l[1],v=g===void 0?[]:g,h=l[2],b=h===void 0?void 0:h,m=Fe(Fe({},f),i);return Ub(r,function(_){for(var A in m)_.addCase(A,m[A]);for(var w=0,S=v;w<S.length;w++){var K=S[w];_.addMatcher(K.matcher,K.reducer);}b&&_.addDefaultCase(b);})}var u;return {name:t,reducer:function(l,d){return u||(u=c()),u(l,d)},actions:s,caseReducers:a,getInitialState:function(){return u||(u=c()),u.getInitialState()}}}var Gb="ModuleSymbhasOwnPr-0123456789ABCDEFGHNRVfgctiUvz_KqYTJkLxpZXIjQW",lc=function(e){e===void 0&&(e=21);for(var t="",r=e;r--;)t+=Gb[Math.random()*64|0];return t},zb=["name","message","stack","code"],ua=function(){function e(t,r){this.payload=t,this.meta=r;}return e}(),dc=function(){function e(t,r){this.payload=t,this.meta=r;}return e}(),Wb=function(e){if(typeof e=="object"&&e!==null){for(var t={},r=0,n=zb;r<n.length;r++){var o=n[r];typeof e[o]=="string"&&(t[o]=e[o]);}return t}return {message:String(e)}},L=function(){function e(t,r,n){var o=J(t+"/fulfilled",function(l,d,f,g){return {payload:l,meta:oa(Fe({},g||{}),{arg:f,requestId:d,requestStatus:"fulfilled"})}}),a=J(t+"/pending",function(l,d,f){return {payload:void 0,meta:oa(Fe({},f||{}),{arg:d,requestId:l,requestStatus:"pending"})}}),i=J(t+"/rejected",function(l,d,f,g,v){return {payload:g,error:(n&&n.serializeError||Wb)(l||"Rejected"),meta:oa(Fe({},v||{}),{arg:f,requestId:d,rejectedWithValue:!!g,requestStatus:"rejected",aborted:(l==null?void 0:l.name)==="AbortError",condition:(l==null?void 0:l.name)==="ConditionError"})}}),c=typeof AbortController!="undefined"?AbortController:function(){function l(){this.signal={aborted:!1,addEventListener:function(){},dispatchEvent:function(){return !1},onabort:function(){},removeEventListener:function(){},reason:void 0,throwIfAborted:function(){}};}return l.prototype.abort=function(){},l}();function u(l){return function(d,f,g){var v=n!=null&&n.idGenerator?n.idGenerator(l):lc(),h=new c,b;function m(A){b=A,h.abort();}var _=function(){return $r(this,null,function(){var A,w,S,K,re,ne,fe;return yr(this,function(ve){switch(ve.label){case 0:return ve.trys.push([0,4,,5]),K=(A=n==null?void 0:n.condition)==null?void 0:A.call(n,l,{getState:f,extra:g}),Zb(K)?[4,K]:[3,2];case 1:K=ve.sent(),ve.label=2;case 2:if(K===!1||h.signal.aborted)throw {name:"ConditionError",message:"Aborted due to condition callback returning false."};return re=new Promise(function(ue,Ne){return h.signal.addEventListener("abort",function(){return Ne({name:"AbortError",message:b||"Aborted"})})}),d(a(v,l,(w=n==null?void 0:n.getPendingMeta)==null?void 0:w.call(n,{requestId:v,arg:l},{getState:f,extra:g}))),[4,Promise.race([re,Promise.resolve(r(l,{dispatch:d,getState:f,extra:g,requestId:v,signal:h.signal,abort:m,rejectWithValue:function(ue,Ne){return new ua(ue,Ne)},fulfillWithValue:function(ue,Ne){return new dc(ue,Ne)}})).then(function(ue){if(ue instanceof ua)throw ue;return ue instanceof dc?o(ue.payload,v,l,ue.meta):o(ue,v,l)})])];case 3:return S=ve.sent(),[3,5];case 4:return ne=ve.sent(),S=ne instanceof ua?i(null,v,l,ne.payload,ne.meta):i(ne,v,l),[3,5];case 5:return fe=n&&!n.dispatchConditionRejection&&i.match(S)&&S.meta.condition,fe||d(S),[2,S]}})})}();return Object.assign(_,{abort:m,requestId:v,arg:l,unwrap:function(){return _.then(Hb)}})}}return Object.assign(u,{pending:a,rejected:i,fulfilled:o,typePrefix:t})}return e.withTypes=function(){return e},e}();function Hb(e){if(e.meta&&e.meta.rejectedWithValue)throw e.payload;if(e.error)throw e.error;return e.payload}function Zb(e){return e!==null&&typeof e=="object"&&typeof e.then=="function"}var Kb=function(e){return e&&typeof e.match=="function"},Yb=function(e,t){return Kb(e)?e.match(t):e(t)};function $(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return function(r){return e.some(function(n){return Yb(n,r)})}}var la=function(e,t){if(typeof e!="function")throw new TypeError(t+" is not a function")},da=function(){},pc=function(e,t){return t===void 0&&(t=da),e.catch(t),e},fc=function(e,t){return e.addEventListener("abort",t,{once:!0}),function(){return e.removeEventListener("abort",t)}},Pt=function(e,t){var r=e.signal;r.aborted||("reason"in r||Object.defineProperty(r,"reason",{enumerable:!0,value:t,configurable:!0,writable:!0}),e.abort(t));},Jb="task",vc="listener",gc="completed",pa="cancelled",Xb="task-"+pa,ey="task-"+gc,hc=vc+"-"+pa,ty=vc+"-"+gc,gn=function(){function e(t){this.code=t,this.name="TaskAbortError",this.message=Jb+" "+pa+" (reason: "+t+")";}return e}(),Et=function(e){if(e.aborted)throw new gn(e.reason)};function mc(e,t){var r=da;return new Promise(function(n,o){var a=function(){return o(new gn(e.reason))};if(e.aborted){a();return}r=fc(e,a),t.finally(function(){return r()}).then(n,o);}).finally(function(){r=da;})}var ry=function(e,t){return $r(void 0,null,function(){var r,n;return yr(this,function(o){switch(o.label){case 0:return o.trys.push([0,3,4,5]),[4,Promise.resolve()];case 1:return o.sent(),[4,e()];case 2:return r=o.sent(),[2,{status:"ok",value:r}];case 3:return n=o.sent(),[2,{status:n instanceof gn?"cancelled":"rejected",error:n}];case 4:return t==null||t(),[7];case 5:return [2]}})})},hn=function(e){return function(t){return pc(mc(e,t).then(function(r){return Et(e),r}))}},bc=function(e){var t=hn(e);return function(r){return t(new Promise(function(n){return setTimeout(n,r)}))}},ny=Object.assign,yc={},wr="listenerMiddleware",oy=function(e){var t=function(r){return fc(e,function(){return Pt(r,e.reason)})};return function(r){la(r,"taskExecutor");var n=new AbortController;t(n);var o=ry(function(){return $r(void 0,null,function(){var a;return yr(this,function(i){switch(i.label){case 0:return Et(e),Et(n.signal),[4,r({pause:hn(n.signal),delay:bc(n.signal),signal:n.signal})];case 1:return a=i.sent(),Et(n.signal),[2,a]}})})},function(){return Pt(n,ey)});return {result:hn(e)(o),cancel:function(){Pt(n,Xb);}}}},ay=function(e,t){var r=function(n,o){return $r(void 0,null,function(){var a,i,s,c;return yr(this,function(u){switch(u.label){case 0:Et(t),a=function(){},i=new Promise(function(l,d){var f=e({predicate:n,effect:function(g,v){v.unsubscribe(),l([g,v.getState(),v.getOriginalState()]);}});a=function(){f(),d();};}),s=[i],o!=null&&s.push(new Promise(function(l){return setTimeout(l,o,null)})),u.label=1;case 1:return u.trys.push([1,,3,4]),[4,mc(t,Promise.race(s))];case 2:return c=u.sent(),Et(t),[2,c];case 3:return a(),[7];case 4:return [2]}})})};return function(n,o){return pc(r(n,o))}},_c=function(e){var t=e.type,r=e.actionCreator,n=e.matcher,o=e.predicate,a=e.effect;if(t)o=J(t).match;else if(r)t=r.type,o=r.match;else if(n)o=n;else if(!o)throw new Error("Creating or removing a listener requires one of the known fields for matching an action");return la(a,"options.listener"),{predicate:o,type:t,effect:a}},iy=function(e){var t=_c(e),r=t.type,n=t.predicate,o=t.effect,a=lc(),i={id:a,effect:o,type:r,predicate:n,pending:new Set,unsubscribe:function(){throw new Error("Unsubscribe not initialized")}};return i},fa=function(e){e.pending.forEach(function(t){Pt(t,hc);});},sy=function(e){return function(){e.forEach(fa),e.clear();}},$c=function(e,t,r){try{e(t,r);}catch(n){setTimeout(function(){throw n},0);}},cy=J(wr+"/add"),uy=J(wr+"/removeAll"),ly=J(wr+"/remove"),dy=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];console.error.apply(console,_r([wr+"/error"],e));};function py(e){var t=this;e===void 0&&(e={});var r=new Map,n=e.extra,o=e.onError,a=o===void 0?dy:o;la(a,"onError");var i=function(g){return g.unsubscribe=function(){return r.delete(g.id)},r.set(g.id,g),function(v){g.unsubscribe(),v!=null&&v.cancelActive&&fa(g);}},s=function(g){for(var v=0,h=Array.from(r.values());v<h.length;v++){var b=h[v];if(g(b))return b}},c=function(g){var v=s(function(h){return h.effect===g.effect});return v||(v=iy(g)),i(v)},u=function(g){var v=_c(g),h=v.type,b=v.effect,m=v.predicate,_=s(function(A){var w=typeof h=="string"?A.type===h:A.predicate===m;return w&&A.effect===b});return _&&(_.unsubscribe(),g.cancelActive&&fa(_)),!!_},l=function(g,v,h,b){return $r(t,null,function(){var m,_,A;return yr(this,function(w){switch(w.label){case 0:m=new AbortController,_=ay(c,m.signal),w.label=1;case 1:return w.trys.push([1,3,4,5]),g.pending.add(m),[4,Promise.resolve(g.effect(v,ny({},h,{getOriginalState:b,condition:function(S,K){return _(S,K).then(Boolean)},take:_,delay:bc(m.signal),pause:hn(m.signal),extra:n,signal:m.signal,fork:oy(m.signal),unsubscribe:g.unsubscribe,subscribe:function(){r.set(g.id,g);},cancelActiveListeners:function(){g.pending.forEach(function(S,K,re){S!==m&&(Pt(S,hc),re.delete(S));});}})))];case 2:return w.sent(),[3,5];case 3:return A=w.sent(),A instanceof gn||$c(a,A,{raisedBy:"effect"}),[3,5];case 4:return Pt(m,ty),g.pending.delete(m),[7];case 5:return [2]}})})},d=sy(r),f=function(g){return function(v){return function(h){if(cy.match(h))return c(h.payload);if(uy.match(h)){d();return}if(ly.match(h))return u(h.payload);var b=g.getState(),m=function(){if(b===yc)throw new Error(wr+": getOriginalState can only be called synchronously");return b},_;try{if(_=v(h),r.size>0)for(var A=g.getState(),w=Array.from(r.values()),S=0,K=w;S<K.length;S++){var re=K[S],ne=!1;try{ne=re.predicate(h,A,b);}catch(fe){ne=!1,$c(a,fe,{raisedBy:"predicate"});}ne&&l(re,h,g,m);}}finally{b=yc;}return _}}};return {middleware:f,startListening:c,stopListening:u,clearListeners:d}}typeof queueMicrotask=="function"&&queueMicrotask.bind(typeof window!="undefined"?window:typeof global!="undefined"?global:globalThis),db();var fy=(e=>(e.LIGHT="Light",e.COLOR="Color",e))(fy||{}),vy=(e=>(e.CONFIRMED="CONFIRMED",e.UNCONFIRMED_BUT_PLAUSIBLE="UNCONFIRMED_BUT_PLAUSIBLE",e.UNCONFIRMED_BUT_SUSPICIOUS="UNCONFIRMED_BUT_SUSPICIOUS",e))(vy||{}),Xe=(e=>(e.ROUTE="route",e.STREET_NUMBER="street_number",e.STREET_ADDRESS="street_address",e.SUBPREMISE="subpremise",e.LOCALITY="locality",e.STATE="administrative_area_level_1",e.POSTAL_CODE="postal_code",e.POSTAL_CODE_SUFFIX="postal_code_suffix",e))(Xe||{}),Ac=(e=>(e.STREET_1="street_1",e.STREET_2="street_2",e.CITY="city",e.STATE="state",e.ZIPCODE="zipcode",e))(Ac||{}),gy=(e=>(e.ApplicationForms="application_forms",e.PartnerPhoneCalls="partner_phone_calls",e.FeatureCheck="features",e.Questions="questions",e.Rules="rules",e.RuleExcludedChoices="rule_excluded_choices",e.SearchableQuestions="searchable_questions",e.SendToExchange="send_to_exchange",e.TakeOver="take_over",e))(gy||{}),It=(e=>(e.NotConnected="NotConnected",e.Connected="Connected",e.Disconnected="Disconnected",e.Connecting="Connecting",e.Disconnecting="Disconnecting",e))(It||{}),hy=(e=>(e.Create="create",e.Index="index",e.Show="show",e.Update="update",e))(hy||{}),Oc=(e=>(e.Development="development",e.Alpha="alpha",e.Beta="beta",e.Production="prod",e))(Oc||{}),my=(e=>(e.Accepted="Accepted",e.Expired="Expired",e.Not_Created="Not created",e.Queued="Queued to send",e.Sent="Sent",e))(my||{}),by=(e=>(e.INVITE="invite",e.RESET="reset",e))(by||{}),Tt=(e=>(e.token="token",e.cableToken="cable_token",e.user="user",e.state="state",e.expiresAt="expiresAt",e.refreshAt="refreshAt",e.impersonated="impersonated",e.terminalTheme="terminalTheme",e))(Tt||{}),q=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Configured="Configured",e.Error="Error",e))(q||{}),qt=(e=>(e.Healthy="Healthy",e.Error="Error",e))(qt||{}),yy=(e=>(e.OAuth="OAuth",e.Login="Login",e))(yy||{}),_y=(e=>(e.storageKey="BP_SDK_ANALYTICS_VARS",e.additionalQuoteOnline="onlinebuy",e.additionalQuoteOffline="nobuy",e))(_y||{}),$y=(e=>(e.activeIdUpdate="activeIdUpdate",e.activeQuestionSetChange="activeQuestionSetChange",e.addItem="addItem",e.answerStateChanged="answerStateChanged",e.bpAnalyticsTrack="bpAnalyticsTrack",e.closePanel="closePanel",e.deleteItem="deleteItem",e.deletePool="deletePool",e.epFieldChanged="epFieldChanged",e.errorsUpdate="errorsUpdate",e.fakeValueUpdate="fakeValueUpdate",e.filterUpdate="filterUpdate",e.formLoaded="formLoaded",e.nudgeAnswer="nudgeAnswer",e.openPanel="openPanel",e.placesUpdate="placesUpdate",e.questionSetAdded="questionSetAdded",e.questionSetCompletionChanged="questionSetCompletionChanged",e.quoteFailure="quoteFailure",e.quotesChanged="quotesChanged",e.quoteSuccess="quoteSuccess",e.removeActiveAnswer="removeActiveAnswer",e.saveAnswer="saveAnswer",e.saveVehicle="saveVehicle",e.sdkError="sdkError",e.tabChanged="tabChanged",e.updateActiveAnswer="updateActiveAnswer",e.userAuthenticationChanged="userAuthenticationChanged",e.valueUpdate="valueUpdate",e.vinUpdate="vinUpdate",e.ymmcUpdate="ymmcUpdate",e))($y||{}),wy=(e=>(e.vin="vin",e.ymm="ymm",e.trailers="Trailers",e))(wy||{}),Ay=(e=>(e.trailers="trailers",e))(Ay||{}),Oy=(e=>(e.Configuration="configuration",e.Make="make",e.Model="model",e.Year="year",e))(Oy||{}),Sy=(e=>(e.Address="address",e.Accordion="accordion",e.AccumulatePercentage="accumulate_percentage",e.Vehicle="vehicle",e.Driver="driver",e.Stepper="stepper",e.Step="step",e.Coverage="coverage",e.AutoCollapsedAccordion="auto_collapsed_accordion",e.ValidatedAddress="validated_address",e))(Sy||{}),C=(e=>(e.Address="address",e.Anchor="anchor",e.CheckBox="checkbox",e.CoverageType="coverage_type",e.Currency="currency",e.Date="date",e.DeletePool="delete_pool",e.Dialog="dialog",e.DriversLicense="drivers_license",e.DropDown="dropdown",e.Email="email",e.FederalEmployeeIdentification="fein",e.JSON="json",e.Label="label",e.Masked="masked",e.MonthYear="month_year",e.Number="number",e.Object="object",e.Percentage="percentage",e.Phone="phone",e.Radio="radio",e.Select="select",e.SocialSecurityNumber="ssn",e.Step="step",e.Tenant="tenant",e.Text="text",e.Vehicle="vehicle",e.Url="url",e.VIN="vin",e.Year="year",e.ZipCode="zipcode",e))(C||{}),mn=(e=>(e.Accumulator="accumulator",e.AddressJSON="address_json",e.BusinessClass="business_class",e.City="city",e.CommercialAutoACV="ca_acv",e.CommercialAutoCategory="ca_category",e.CommercialAutoCount="ca_count",e.CommercialAutoDrivetrain="ca_drivetrain",e.CommercialAutoFlow="ca_flow",e.CommercialAutoGCW="ca_gcw",e.CommercialAutoGVW="ca_gvw",e.CommercialAutoJson="ca_json",e.CommercialAutoMake="ca_make",e.CommercialAutoModel="ca_model",e.CommercialAutoMSRP="ca_msrp",e.CommercialAutoSubModel="ca_submodel",e.CommercialAutoTractor="ca_tractor",e.CommercialAutoVIN="ca_vin",e.CommercialAutoYear="ca_year",e.County="county",e.DriversLicenseState="drivers_license_state",e.Indent="indent",e.Increment="increment",e.JornayaLeadID="jornaya_lead_id",e.LinesOfBusiness="lines_of_business",e.LinesOfBusinessAdditional="lines_of_business_additional",e.MotorVehicleReportComplete="mvr_complete",e.MotorVehicleReportState="mvr_state",e.MotorVehicleReportCondition="mvr_report_condition",e.State="state",e.Street1="street_1",e.Street2="street_2",e.ZipCode="zipcode",e))(mn||{}),jy=(e=>(e.CoverageLimit="coverage limit",e))(jy||{}),Cy=(e=>(e.MaximumDate="maxDate",e.MaximumValue="max",e.MinimumDate="minDate",e.MinimumValue="min",e.Pattern="pattern",e.Required="required",e))(Cy||{}),Py=(e=>(e.Acord="acord",e.Acord125="acord_125",e.Acord126="acord_126",e.Acord130="acord_130",e))(Py||{}),va=(e=>(e.BP_QUOTABLE="bp_quotable",e.OFFLINE="offline",e.ONLINE="online",e))(va||{}),bn=(e=>(e.Pending="pending",e.Queued="queued",e.Executing="executing",e.Completed="completed",e.Failed="failed",e.Aborted="aborted",e))(bn||{}),Ey=(e=>(e.ClueReport="ClueReport",e.MotorVehicleReport="MotorVehicleReport",e.RequestPrefill="RequestPrefill",e.DriverScore="DriverScore",e.JornayaAudit="JornayaAudit",e.DotNumber="DotNumber",e))(Ey||{}),xt=(e=>(e.Incomplete="incomplete",e.InProgress="in_progress",e.Completed="completed",e.Quoted="quoted",e))(xt||{}),Iy=(e=>(e.not_permitted_to_bind="not_permitted_to_bind",e.sent_to_wholesaler="sent_to_wholesaler",e))(Iy||{}),Ty=(e=>(e.dialog="dialog",e.wholesale_workflow="wholesale_workflow",e))(Ty||{}),ga=(e=>(e.NON_ADMITTED="non_admitted",e.WHOLESALER="wholesaler",e.INSTANT_ENROLLMENT="instant_enrollment",e.ADDITIONAL_MARKET="additional_market",e.OPT_IN_QUOTE_FLOW="opt_in_quote_flow",e))(ga||{});var qy=(e=>(e.Sub="sub",e.Direct="direct",e))(qy||{}),et=(e=>(e.Answer="answer",e.Application="application_form",e.QuoteRequest="quote_request",e))(et||{}),Sc=(e=>(e.info="info",e.warning="warning",e.error="error",e))(Sc||{}),yn=(e=>(e.QuestionSet="question_set",e.Question="question",e))(yn||{}),xy=(e=>(e.ReadyToBind="ready_to_bind",e))(xy||{}),ha=(e=>(e.unsent="unsent",e.in_progress="in_progress",e.sent="sent",e.completed="completed",e.failed="failed",e.timed_out="timed_out",e.ineligible="ineligible",e.referral="referral",e.bound="bound",e))(ha||{}),x=(e=>(e.AddQuestionSet="application_forms_question_set",e.Answer="answer",e.Appetite="appetite",e.ApplicationForm="application_form",e.ApplicationFormPercentCompleted="application_form_percent_completed",e.ApplicationFormProcess="application_form_process",e.Batch="batch",e.CarrierProducts="eligible_carriers_products",e.Message="message",e.Notification="notification",e.QuoteRequest="quote_request",e.RealTimeEligibility="real_time_eligibility",e.RemoveQuestionSet="destroy_application_forms_question_set",e))(x||{}),ky=(e=>(e.AcordForm="acord_form_pdf",e.AgentQuoteStart="agent_quote_start",e.ClientQuoteStart="client_quote_start",e.Storefront="storefront",e))(ky||{}),y=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Error="Error",e))(y||{});const Dy={email:"",expiresAt:0,firstName:"",lastName:"",name:"",options:{},permissions:[],refreshCount:0,tenantLogoUrl:"",tenantName:"",tenantState:"",tenantTheme:"",userId:"",terminalTheme:void 0,token:"",user:void 0},Ly={},Ny={bcsChoiceLimit:7,driverLimit:20,enableAutocomplete:!0,enableDetailedRelevanceChips:!1,enableQuestionSetNavigation:!0,enableTrailerSupport:!0,enableUserApplicationGuard:!1,enableWebSockets:!0,quoteStatusFilter:[ha.completed],renderOfflineQuotes:!1,vehicleLimit:20,renderQuotesAsync:!1,renderQuotesWithAppState:[xt.Completed,xt.Quoted],hideNestedQuestionGroupLabels:!1,showParentQuestionSetsInNavigation:!1,enableAddressValidation:!1,shouldQuoteCardRenderSupplierLogos:!1},Ry={clientId:"",cableToken:"",options:Ny,env:Oc.Production,debug:!1,googleApiToken:"",uri:"",wsBaseUri:"",theming:!1},My={},Vy={appetite:[],buckets:[],cloned:!1,completed_at:new Date(0),coverage_start_date:null,copy:!1,created_at:new Date(0),created_by:{id:"",first_name:"",last_name:""},current_bucket_id:null,eligible_carriers_products:[],features:[],finished_quote_requests_count:0,global_origin_tenant_name:null,id:"",inactive:!1,locked:!1,messages:[],owner_id:"",owner:{id:"",first_name:"",last_name:""},owner_is_guest:!1,parent_id:null,percent_completed:0,quote_requests_count:0,real_time_eligibility:{application_form_id:"",carriers:[]},redirect_url:"",reference_id:"",referred_by:null,state:xt.Incomplete,storefront_active_enrolled_product_codes:null,taken_over:!1,tenant:{id:"",name:""},tenant_id:"",updated_at:new Date(0),user_group_id:"",sent_to_wholesaler_at:null,received_by_wholesaler_at:null,source:"",source_metadata:null,wholesaler_preferred_quotes:null},Fy={meta:{form:{percent:0,forwardProgressPercent:0}}},Qy={list:[],active:"",activeAnswerId:"",idOfNextActiveQuestionSetNotInState:void 0,completedQuestionSetAnswerHasChanged:!1,parentQuestionSets:[]},Uy={},By={},ma={isAvailable:!1,errorCount:0},Gy={configurations:{},makes:{},models:{},selection:{},vin:{},years:{}},jc={list:[],isLoaded:!1,status:q.NotLoaded},Cc={filteredOnlineQuotes:[],filteredOfflineQuotes:[]};var kt=(e=>(e.FIX="fix",e.ACCEPT="accept",e.CONFIRM="confirm",e.TYPO="typo",e))(kt||{}),ie=(e=>(e.REQUIRED="required",e.READY_TO_VALIDATE="ready to validate",e.VALIDATED="validated",e.PROCEED_NEXT="proceed next",e.COMPLETED="completed",e.VALIDATION_ERROR="error",e))(ie||{}),zy=(e=>(e.EDIT="return to edit address",e.AS_ENTERED="continue with original address",e.CONFIRM="continue with confirm suggested address",e.MISSING_FIX="continue with missing input fix",e))(zy||{});const Pc={verdict:null,responseId:"",address:null,stagedAnswers:[],validationStatus:"required",attempt:0,originalAddress:""},Wy={shouldOverrideValidationTrigger:!1,validations:{}};var Hy=Object.defineProperty,Ec=Object.getOwnPropertySymbols,Zy=Object.prototype.hasOwnProperty,Ky=Object.prototype.propertyIsEnumerable,Ic=(e,t,r)=>t in e?Hy(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,_n=(e,t)=>{for(var r in t||(t={}))Zy.call(t,r)&&Ic(e,r,t[r]);if(Ec)for(var r of Ec(t))Ky.call(t,r)&&Ic(e,r,t[r]);return e};_n({},Gy),_n({},jc),_n({},Cc),_n({},Pc);const Yy={answers:{queue:[],outbound:{}}};var Jy=Object.defineProperty,Tc=Object.getOwnPropertySymbols,Xy=Object.prototype.hasOwnProperty,e_=Object.prototype.propertyIsEnumerable,qc=(e,t,r)=>t in e?Jy(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,tt=(e,t)=>{for(var r in t||(t={}))Xy.call(t,r)&&qc(e,r,t[r]);if(Tc)for(var r of Tc(t))e_.call(t,r)&&qc(e,r,t[r]);return e};const t_={configurations:{},makes:{},models:{},selection:{},vins:{},years:{}},Dt={state:y.NotLoaded,map:{}},$n={state:y.NotLoaded},xc={form:y.NotLoaded},r_={answers:tt({},Dt),application:tt({},xc),bindQuotes:{},choices:{},carriersProducts:tt({},$n),questionPools:tt({},Dt),questionSets:tt({},Dt),quotes:tt({},$n),refilters:{},vehicles:tt({},t_)};var n_=Object.defineProperty,kc=Object.getOwnPropertySymbols,o_=Object.prototype.hasOwnProperty,a_=Object.prototype.propertyIsEnumerable,Dc=(e,t,r)=>t in e?n_(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Lc=(e,t)=>{for(var r in t||(t={}))o_.call(t,r)&&Dc(e,r,t[r]);if(kc)for(var r of kc(t))a_.call(t,r)&&Dc(e,r,t[r]);return e};const Nc={auth:q.NotLoaded,config:q.NotLoaded,health:qt.Healthy,state:q.NotLoaded,user:q.NotLoaded,socket:It.NotConnected,socketId:""};Lc({},r_),Lc({},Nc);var i_=(e=>(e.QuestionSetSaved="QuestionSetSaved",e.AddressValidation_ValidatedOnFirstAttempt="AddressValidOnFirstAttempt",e.AddressValidation_Verdict="AddressValidationVerdict",e.AddressValidation_CTAClicked="AddressValidationCTAClicked",e.AddressValidation_ValidationError="AddressValidationValidationError",e))(i_||{});let D=class extends Error{constructor(t,r){var n;super(t),this.name="ReduxAsyncError";const o=[this.message,r==null?void 0:r.message,(n=r==null?void 0:r.error)==null?void 0:n.message].filter(Boolean);if(r!=null&&r.parsedBody){const i=r;this.response=i,i.parsedBody instanceof Array?o.push(...i.parsedBody):i.parsedBody.errors instanceof Array&&o.push(...i.parsedBody.errors);}this.message=o.join(", ");const a=new.target.prototype;Object.setPrototypeOf?Object.setPrototypeOf(this,a):this.__proto__=a;}};var wn={logger:typeof console!="undefined"?console:void 0,WebSocket:typeof WebSocket!="undefined"?WebSocket:void 0},W={log(...e){this.enabled&&(e.push(Date.now()),wn.logger.log("[ActionCable]",...e));}};const Ar=()=>new Date().getTime(),An=e=>(Ar()-e)/1e3;let ba=class{constructor(t){this.visibilityDidChange=this.visibilityDidChange.bind(this),this.connection=t,this.reconnectAttempts=0;}start(){this.isRunning()||(this.startedAt=Ar(),delete this.stoppedAt,this.startPolling(),addEventListener("visibilitychange",this.visibilityDidChange),W.log(`ConnectionMonitor started. stale threshold = ${this.constructor.staleThreshold} s`));}stop(){this.isRunning()&&(this.stoppedAt=Ar(),this.stopPolling(),removeEventListener("visibilitychange",this.visibilityDidChange),W.log("ConnectionMonitor stopped"));}isRunning(){return this.startedAt&&!this.stoppedAt}recordPing(){this.pingedAt=Ar();}recordConnect(){this.reconnectAttempts=0,this.recordPing(),delete this.disconnectedAt,W.log("ConnectionMonitor recorded connect");}recordDisconnect(){this.disconnectedAt=Ar(),W.log("ConnectionMonitor recorded disconnect");}startPolling(){this.stopPolling(),this.poll();}stopPolling(){clearTimeout(this.pollTimeout);}poll(){this.pollTimeout=setTimeout(()=>{this.reconnectIfStale(),this.poll();},this.getPollInterval());}getPollInterval(){const{staleThreshold:t,reconnectionBackoffRate:r}=this.constructor,n=Math.pow(1+r,Math.min(this.reconnectAttempts,10)),o=(this.reconnectAttempts===0?1:r)*Math.random();return t*1e3*n*(1+o)}reconnectIfStale(){this.connectionIsStale()&&(W.log(`ConnectionMonitor detected stale connection. reconnectAttempts = ${this.reconnectAttempts}, time stale = ${An(this.refreshedAt)} s, stale threshold = ${this.constructor.staleThreshold} s`),this.reconnectAttempts++,this.disconnectedRecently()?W.log(`ConnectionMonitor skipping reopening recent disconnect. time disconnected = ${An(this.disconnectedAt)} s`):(W.log("ConnectionMonitor reopening"),this.connection.reopen()));}get refreshedAt(){return this.pingedAt?this.pingedAt:this.startedAt}connectionIsStale(){return An(this.refreshedAt)>this.constructor.staleThreshold}disconnectedRecently(){return this.disconnectedAt&&An(this.disconnectedAt)<this.constructor.staleThreshold}visibilityDidChange(){document.visibilityState==="visible"&&setTimeout(()=>{(this.connectionIsStale()||!this.connection.isOpen())&&(W.log(`ConnectionMonitor reopening stale connection on visibilitychange. visibilityState = ${document.visibilityState}`),this.connection.reopen());},200);}};ba.staleThreshold=6,ba.reconnectionBackoffRate=.15;var s_={message_types:{welcome:"welcome",disconnect:"disconnect",ping:"ping",confirmation:"confirm_subscription",rejection:"reject_subscription"},disconnect_reasons:{unauthorized:"unauthorized",invalid_request:"invalid_request",server_restart:"server_restart",remote:"remote"},default_mount_path:"/cable",protocols:["actioncable-v1-json","actioncable-unsupported"]};const{message_types:Or,protocols:ya}=s_,c_=ya.slice(0,ya.length-1),Rc=[].indexOf;let _a=class{constructor(t){this.open=this.open.bind(this),this.consumer=t,this.subscriptions=this.consumer.subscriptions,this.monitor=new ba(this),this.disconnected=!0;}send(t){return this.isOpen()?(this.webSocket.send(JSON.stringify(t)),!0):!1}open(){if(this.isActive())return W.log(`Attempted to open WebSocket, but existing socket is ${this.getState()}`),!1;{const t=[...ya,...this.consumer.subprotocols||[]];return W.log(`Opening WebSocket, current state is ${this.getState()}, subprotocols: ${t}`),this.webSocket&&this.uninstallEventHandlers(),this.webSocket=new wn.WebSocket(this.consumer.url,t),this.installEventHandlers(),this.monitor.start(),!0}}close({allowReconnect:t}={allowReconnect:!0}){if(t||this.monitor.stop(),this.isOpen())return this.webSocket.close()}reopen(){if(W.log(`Reopening WebSocket, current state is ${this.getState()}`),this.isActive())try{return this.close()}catch(t){W.log("Failed to reopen WebSocket",t);}finally{W.log(`Reopening WebSocket in ${this.constructor.reopenDelay}ms`),setTimeout(this.open,this.constructor.reopenDelay);}else return this.open()}getProtocol(){if(this.webSocket)return this.webSocket.protocol}isOpen(){return this.isState("open")}isActive(){return this.isState("open","connecting")}triedToReconnect(){return this.monitor.reconnectAttempts>0}isProtocolSupported(){return Rc.call(c_,this.getProtocol())>=0}isState(...t){return Rc.call(t,this.getState())>=0}getState(){if(this.webSocket){for(let t in wn.WebSocket)if(wn.WebSocket[t]===this.webSocket.readyState)return t.toLowerCase()}return null}installEventHandlers(){for(let t in this.events){const r=this.events[t].bind(this);this.webSocket[`on${t}`]=r;}}uninstallEventHandlers(){for(let t in this.events)this.webSocket[`on${t}`]=function(){};}};_a.reopenDelay=500,_a.prototype.events={message(e){if(!this.isProtocolSupported())return;const{identifier:t,message:r,reason:n,reconnect:o,type:a}=JSON.parse(e.data);switch(a){case Or.welcome:return this.triedToReconnect()&&(this.reconnectAttempted=!0),this.monitor.recordConnect(),this.subscriptions.reload();case Or.disconnect:return W.log(`Disconnecting. Reason: ${n}`),this.close({allowReconnect:o});case Or.ping:return this.monitor.recordPing();case Or.confirmation:return this.subscriptions.confirmSubscription(t),this.reconnectAttempted?(this.reconnectAttempted=!1,this.subscriptions.notify(t,"connected",{reconnected:!0})):this.subscriptions.notify(t,"connected",{reconnected:!1});case Or.rejection:return this.subscriptions.reject(t);default:return this.subscriptions.notify(t,"received",r)}},open(){if(W.log(`WebSocket onopen event, using '${this.getProtocol()}' subprotocol`),this.disconnected=!1,!this.isProtocolSupported())return W.log("Protocol is unsupported. Stopping monitor and disconnecting."),this.close({allowReconnect:!1})},close(e){if(W.log("WebSocket onclose event"),!this.disconnected)return this.disconnected=!0,this.monitor.recordDisconnect(),this.subscriptions.notifyAll("disconnected",{willAttemptReconnect:this.monitor.isRunning()})},error(){W.log("WebSocket onerror event");}};const u_=function(e,t){if(t!=null)for(let r in t){const n=t[r];e[r]=n;}return e};let l_=class{constructor(t,r={},n){this.consumer=t,this.identifier=JSON.stringify(r),u_(this,n);}perform(t,r={}){return r.action=t,this.send(r)}send(t){return this.consumer.send({command:"message",identifier:this.identifier,data:JSON.stringify(t)})}unsubscribe(){return this.consumer.subscriptions.remove(this)}},d_=class{constructor(t){this.subscriptions=t,this.pendingSubscriptions=[];}guarantee(t){this.pendingSubscriptions.indexOf(t)==-1?(W.log(`SubscriptionGuarantor guaranteeing ${t.identifier}`),this.pendingSubscriptions.push(t)):W.log(`SubscriptionGuarantor already guaranteeing ${t.identifier}`),this.startGuaranteeing();}forget(t){W.log(`SubscriptionGuarantor forgetting ${t.identifier}`),this.pendingSubscriptions=this.pendingSubscriptions.filter(r=>r!==t);}startGuaranteeing(){this.stopGuaranteeing(),this.retrySubscribing();}stopGuaranteeing(){clearTimeout(this.retryTimeout);}retrySubscribing(){this.retryTimeout=setTimeout(()=>{this.subscriptions&&typeof this.subscriptions.subscribe=="function"&&this.pendingSubscriptions.map(t=>{W.log(`SubscriptionGuarantor resubscribing ${t.identifier}`),this.subscriptions.subscribe(t);});},500);}},p_=class{constructor(t){this.consumer=t,this.guarantor=new d_(this),this.subscriptions=[];}create(t,r){const n=t,o=typeof n=="object"?n:{channel:n},a=new l_(this.consumer,o,r);return this.add(a)}add(t){return this.subscriptions.push(t),this.consumer.ensureActiveConnection(),this.notify(t,"initialized"),this.subscribe(t),t}remove(t){return this.forget(t),this.findAll(t.identifier).length||this.sendCommand(t,"unsubscribe"),t}reject(t){return this.findAll(t).map(r=>(this.forget(r),this.notify(r,"rejected"),r))}forget(t){return this.guarantor.forget(t),this.subscriptions=this.subscriptions.filter(r=>r!==t),t}findAll(t){return this.subscriptions.filter(r=>r.identifier===t)}reload(){return this.subscriptions.map(t=>this.subscribe(t))}notifyAll(t,...r){return this.subscriptions.map(n=>this.notify(n,t,...r))}notify(t,r,...n){let o;return typeof t=="string"?o=this.findAll(t):o=[t],o.map(a=>typeof a[r]=="function"?a[r](...n):void 0)}subscribe(t){this.sendCommand(t,"subscribe")&&this.guarantor.guarantee(t);}confirmSubscription(t){W.log(`Subscription confirmed ${t}`),this.findAll(t).map(r=>this.guarantor.forget(r));}sendCommand(t,r){const{identifier:n}=t;return this.consumer.send({command:r,identifier:n})}},f_=class{constructor(t){this._url=t,this.subscriptions=new p_(this),this.connection=new _a(this),this.subprotocols=[];}get url(){return v_(this._url)}send(t){return this.connection.send(t)}connect(){return this.connection.open()}disconnect(){return this.connection.close({allowReconnect:!1})}ensureActiveConnection(){if(!this.connection.isActive())return this.connection.open()}addSubProtocol(t){this.subprotocols=[...this.subprotocols,t];}};function v_(e){if(typeof e=="function"&&(e=e()),e&&!/^wss?:/i.test(e)){const t=document.createElement("a");return t.href=e,t.href=t.href,t.protocol=t.protocol.replace("http","ws"),t.href}else return e}let g_=class Xm extends Error{constructor(t){super(t),this.name="MissingConfigError",Object.setPrototypeOf(this,Xm.prototype);}},h_=class eb extends Error{constructor(t){super(t),this.name="MissingTokenError",Object.setPrototypeOf(this,eb.prototype);}},m_=class tb extends Error{constructor(t){super(t),this.name="MissingWebsocketUriError",Object.setPrototypeOf(this,tb.prototype);}};var b_=Object.defineProperty,Mc=Object.getOwnPropertySymbols,y_=Object.prototype.hasOwnProperty,__=Object.prototype.propertyIsEnumerable,Vc=(e,t,r)=>t in e?b_(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Fc=(e,t)=>{for(var r in t||(t={}))y_.call(t,r)&&Vc(e,r,t[r]);if(Mc)for(var r of Mc(t))__.call(t,r)&&Vc(e,r,t[r]);return e};const $a=()=>{};let $_=class{constructor(t){this.timeout=6e4,this._socketConnected=!1,this.handleConnected=r=>(this._socketConnected||(this._socketConnected=!0),r),this.handleDisconnected=(r,n)=>(this._socketConnected&&(this._socketConnected=!1,setTimeout(()=>{n&&n(),this.socketTimeOut();},this.timeout)),r),this._config=t;}get id(){var t,r;return (r=(t=this._config)==null?void 0:t.id)!=null?r:""}start(){return this.validate(),this.connect()}validate(){if(!this._config)throw new g_("config is required");if(!this._config.token)throw new h_("config.token is required");if(!this._config.wsBaseUri)throw new m_("config.wsBaseUri is required")}connect(){let t=this.cable;return t&&t!==this.cable?(this.cable=t,this.cable):(t=new f_(`${this._config.wsBaseUri}?token=${this._config.token}`),this.cable=t,t.ensureActiveConnection(),this.cable)}getHandlers(t){var r,n,o;const a={received:(r=t==null?void 0:t.received)!=null?r:$a,connected:(n=t==null?void 0:t.connected)!=null?n:$a,disconnected:(o=t==null?void 0:t.disconnected)!=null?o:$a},i=this.handleDisconnected(a.disconnected),s=this.handleConnected(a.connected);return {received:a.received,connected:s,disconnected:i}}subscribe(t,r,n){const o=this.cable;if(!o)return;r?r=Fc({token:this._config.token},r):r={token:this._config.token};const a=Fc({channel:t},r),i=this.getHandlers(n);return o.subscriptions.create(a,i)}disconnect(t=!0){return this.cleanup(this.cable,t)}cleanup(t,r){t&&t.disconnect(),r&&(this.cable=void 0);}socketTimeOut(){this._socketConnected||this.cleanup(this.cable,!1);}};const wa=J("answers/Push"),w_=J("answersQueue/Check"),On=J("queues/QueueAnswer"),Qc=J("auth/NotifyAuthFailed"),Uc=J("quoteResources/Timeout",()=>({payload:{updatedAt:new Date}})),Bc=J("reset/ResetApplication"),Sr=J("reset/ResetStore"),Gc=J("selectVehicle/Reset"),Sn=J("websocket/ConnectionEvent"),Aa=J("websocket/SetActiveSocketId"),Ae=J("websocket/ReceiveBatchWebsocket");var A_=(e=>(e.LIGHT="Light",e.COLOR="Color",e))(A_||{}),O_=(e=>(e.CONFIRMED="CONFIRMED",e.UNCONFIRMED_BUT_PLAUSIBLE="UNCONFIRMED_BUT_PLAUSIBLE",e.UNCONFIRMED_BUT_SUSPICIOUS="UNCONFIRMED_BUT_SUSPICIOUS",e))(O_||{}),S_=(e=>(e.ROUTE="route",e.STREET_NUMBER="street_number",e.STREET_ADDRESS="street_address",e.SUBPREMISE="subpremise",e.LOCALITY="locality",e.STATE="administrative_area_level_1",e.POSTAL_CODE="postal_code",e.POSTAL_CODE_SUFFIX="postal_code_suffix",e))(S_||{}),j_=(e=>(e.STREET_1="street_1",e.STREET_2="street_2",e.CITY="city",e.STATE="state",e.ZIPCODE="zipcode",e))(j_||{}),C_=(e=>(e.ApplicationForms="application_forms",e.PartnerPhoneCalls="partner_phone_calls",e.FeatureCheck="features",e.Questions="questions",e.Rules="rules",e.RuleExcludedChoices="rule_excluded_choices",e.SearchableQuestions="searchable_questions",e.SendToExchange="send_to_exchange",e.TakeOver="take_over",e))(C_||{}),zc=(e=>(e.NotConnected="NotConnected",e.Connected="Connected",e.Disconnected="Disconnected",e.Connecting="Connecting",e.Disconnecting="Disconnecting",e))(zc||{}),P_=(e=>(e.Create="create",e.Index="index",e.Show="show",e.Update="update",e))(P_||{}),jn=(e=>(e.Development="development",e.Alpha="alpha",e.Beta="beta",e.Production="prod",e))(jn||{}),E_=(e=>(e.Accepted="Accepted",e.Expired="Expired",e.Not_Created="Not created",e.Queued="Queued to send",e.Sent="Sent",e))(E_||{}),I_=(e=>(e.INVITE="invite",e.RESET="reset",e))(I_||{}),T_=(e=>(e.token="token",e.cableToken="cable_token",e.user="user",e.state="state",e.expiresAt="expiresAt",e.refreshAt="refreshAt",e.impersonated="impersonated",e.terminalTheme="terminalTheme",e))(T_||{}),Lt=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Configured="Configured",e.Error="Error",e))(Lt||{}),Wc=(e=>(e.Healthy="Healthy",e.Error="Error",e))(Wc||{}),q_=(e=>(e.OAuth="OAuth",e.Login="Login",e))(q_||{}),x_=(e=>(e.storageKey="BP_SDK_ANALYTICS_VARS",e.additionalQuoteOnline="onlinebuy",e.additionalQuoteOffline="nobuy",e))(x_||{}),k_=(e=>(e.activeIdUpdate="activeIdUpdate",e.activeQuestionSetChange="activeQuestionSetChange",e.addItem="addItem",e.answerStateChanged="answerStateChanged",e.bpAnalyticsTrack="bpAnalyticsTrack",e.closePanel="closePanel",e.deleteItem="deleteItem",e.deletePool="deletePool",e.epFieldChanged="epFieldChanged",e.errorsUpdate="errorsUpdate",e.fakeValueUpdate="fakeValueUpdate",e.filterUpdate="filterUpdate",e.formLoaded="formLoaded",e.nudgeAnswer="nudgeAnswer",e.openPanel="openPanel",e.placesUpdate="placesUpdate",e.questionSetAdded="questionSetAdded",e.questionSetCompletionChanged="questionSetCompletionChanged",e.quoteFailure="quoteFailure",e.quotesChanged="quotesChanged",e.quoteSuccess="quoteSuccess",e.removeActiveAnswer="removeActiveAnswer",e.saveAnswer="saveAnswer",e.saveVehicle="saveVehicle",e.sdkError="sdkError",e.tabChanged="tabChanged",e.updateActiveAnswer="updateActiveAnswer",e.userAuthenticationChanged="userAuthenticationChanged",e.valueUpdate="valueUpdate",e.vinUpdate="vinUpdate",e.ymmcUpdate="ymmcUpdate",e))(k_||{}),D_=(e=>(e.vin="vin",e.ymm="ymm",e.trailers="Trailers",e))(D_||{}),L_=(e=>(e.trailers="trailers",e))(L_||{}),N_=(e=>(e.Configuration="configuration",e.Make="make",e.Model="model",e.Year="year",e))(N_||{}),R_=(e=>(e.Address="address",e.Accordion="accordion",e.AccumulatePercentage="accumulate_percentage",e.Vehicle="vehicle",e.Driver="driver",e.Stepper="stepper",e.Step="step",e.Coverage="coverage",e.AutoCollapsedAccordion="auto_collapsed_accordion",e.ValidatedAddress="validated_address",e))(R_||{}),j=(e=>(e.Address="address",e.Anchor="anchor",e.CheckBox="checkbox",e.CoverageType="coverage_type",e.Currency="currency",e.Date="date",e.DeletePool="delete_pool",e.Dialog="dialog",e.DriversLicense="drivers_license",e.DropDown="dropdown",e.Email="email",e.FederalEmployeeIdentification="fein",e.JSON="json",e.Label="label",e.Masked="masked",e.MonthYear="month_year",e.Number="number",e.Object="object",e.Percentage="percentage",e.Phone="phone",e.Radio="radio",e.Select="select",e.SocialSecurityNumber="ssn",e.Step="step",e.Tenant="tenant",e.Text="text",e.Vehicle="vehicle",e.Url="url",e.VIN="vin",e.Year="year",e.ZipCode="zipcode",e))(j||{}),Oa=(e=>(e.Accumulator="accumulator",e.AddressJSON="address_json",e.BusinessClass="business_class",e.City="city",e.CommercialAutoACV="ca_acv",e.CommercialAutoCategory="ca_category",e.CommercialAutoCount="ca_count",e.CommercialAutoDrivetrain="ca_drivetrain",e.CommercialAutoFlow="ca_flow",e.CommercialAutoGCW="ca_gcw",e.CommercialAutoGVW="ca_gvw",e.CommercialAutoJson="ca_json",e.CommercialAutoMake="ca_make",e.CommercialAutoModel="ca_model",e.CommercialAutoMSRP="ca_msrp",e.CommercialAutoSubModel="ca_submodel",e.CommercialAutoTractor="ca_tractor",e.CommercialAutoVIN="ca_vin",e.CommercialAutoYear="ca_year",e.County="county",e.DriversLicenseState="drivers_license_state",e.Indent="indent",e.Increment="increment",e.JornayaLeadID="jornaya_lead_id",e.LinesOfBusiness="lines_of_business",e.LinesOfBusinessAdditional="lines_of_business_additional",e.MotorVehicleReportComplete="mvr_complete",e.MotorVehicleReportState="mvr_state",e.MotorVehicleReportCondition="mvr_report_condition",e.State="state",e.Street1="street_1",e.Street2="street_2",e.ZipCode="zipcode",e))(Oa||{}),M_=(e=>(e.CoverageLimit="coverage limit",e))(M_||{}),V_=(e=>(e.MaximumDate="maxDate",e.MaximumValue="max",e.MinimumDate="minDate",e.MinimumValue="min",e.Pattern="pattern",e.Required="required",e))(V_||{}),F_=(e=>(e.Acord="acord",e.Acord125="acord_125",e.Acord126="acord_126",e.Acord130="acord_130",e))(F_||{}),Q_=(e=>(e.BP_QUOTABLE="bp_quotable",e.OFFLINE="offline",e.ONLINE="online",e))(Q_||{}),U_=(e=>(e.Pending="pending",e.Queued="queued",e.Executing="executing",e.Completed="completed",e.Failed="failed",e.Aborted="aborted",e))(U_||{}),B_=(e=>(e.ClueReport="ClueReport",e.MotorVehicleReport="MotorVehicleReport",e.RequestPrefill="RequestPrefill",e.DriverScore="DriverScore",e.JornayaAudit="JornayaAudit",e.DotNumber="DotNumber",e))(B_||{}),Cn=(e=>(e.Incomplete="incomplete",e.InProgress="in_progress",e.Completed="completed",e.Quoted="quoted",e))(Cn||{}),G_=(e=>(e.not_permitted_to_bind="not_permitted_to_bind",e.sent_to_wholesaler="sent_to_wholesaler",e))(G_||{}),z_=(e=>(e.dialog="dialog",e.wholesale_workflow="wholesale_workflow",e))(z_||{}),W_=(e=>(e.NON_ADMITTED="non_admitted",e.WHOLESALER="wholesaler",e.INSTANT_ENROLLMENT="instant_enrollment",e.ADDITIONAL_MARKET="additional_market",e.OPT_IN_QUOTE_FLOW="opt_in_quote_flow",e))(W_||{});var H_=(e=>(e.Sub="sub",e.Direct="direct",e))(H_||{}),Z_=(e=>(e.Answer="answer",e.Application="application_form",e.QuoteRequest="quote_request",e))(Z_||{}),K_=(e=>(e.info="info",e.warning="warning",e.error="error",e))(K_||{}),Y_=(e=>(e.QuestionSet="question_set",e.Question="question",e))(Y_||{}),J_=(e=>(e.ReadyToBind="ready_to_bind",e))(J_||{}),Hc=(e=>(e.unsent="unsent",e.in_progress="in_progress",e.sent="sent",e.completed="completed",e.failed="failed",e.timed_out="timed_out",e.ineligible="ineligible",e.referral="referral",e.bound="bound",e))(Hc||{}),X_=(e=>(e.AddQuestionSet="application_forms_question_set",e.Answer="answer",e.Appetite="appetite",e.ApplicationForm="application_form",e.ApplicationFormPercentCompleted="application_form_percent_completed",e.ApplicationFormProcess="application_form_process",e.Batch="batch",e.CarrierProducts="eligible_carriers_products",e.Message="message",e.Notification="notification",e.QuoteRequest="quote_request",e.RealTimeEligibility="real_time_eligibility",e.RemoveQuestionSet="destroy_application_forms_question_set",e))(X_||{}),e$=(e=>(e.AcordForm="acord_form_pdf",e.AgentQuoteStart="agent_quote_start",e.ClientQuoteStart="client_quote_start",e.Storefront="storefront",e))(e$||{}),Pn=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Error="Error",e))(Pn||{});const t$={configurations:{},makes:{},models:{},selection:{},vin:{},years:{}},r$={list:[],isLoaded:!1,status:Lt.NotLoaded},n$={filteredOnlineQuotes:[],filteredOfflineQuotes:[]};var o$=(e=>(e.FIX="fix",e.ACCEPT="accept",e.CONFIRM="confirm",e.TYPO="typo",e))(o$||{}),a$=(e=>(e.REQUIRED="required",e.READY_TO_VALIDATE="ready to validate",e.VALIDATED="validated",e.PROCEED_NEXT="proceed next",e.COMPLETED="completed",e.VALIDATION_ERROR="error",e))(a$||{}),i$=(e=>(e.EDIT="return to edit address",e.AS_ENTERED="continue with original address",e.CONFIRM="continue with confirm suggested address",e.MISSING_FIX="continue with missing input fix",e))(i$||{});const s$={verdict:null,responseId:"",address:null,stagedAnswers:[],validationStatus:"required",attempt:0,originalAddress:""};var c$=Object.defineProperty,Zc=Object.getOwnPropertySymbols,u$=Object.prototype.hasOwnProperty,l$=Object.prototype.propertyIsEnumerable,Kc=(e,t,r)=>t in e?c$(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,En=(e,t)=>{for(var r in t||(t={}))u$.call(t,r)&&Kc(e,r,t[r]);if(Zc)for(var r of Zc(t))l$.call(t,r)&&Kc(e,r,t[r]);return e};En({},t$),En({},r$),En({},n$),En({},s$);var d$=Object.defineProperty,Yc=Object.getOwnPropertySymbols,p$=Object.prototype.hasOwnProperty,f$=Object.prototype.propertyIsEnumerable,Jc=(e,t,r)=>t in e?d$(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,rt=(e,t)=>{for(var r in t||(t={}))p$.call(t,r)&&Jc(e,r,t[r]);if(Yc)for(var r of Yc(t))f$.call(t,r)&&Jc(e,r,t[r]);return e};const v$={configurations:{},makes:{},models:{},selection:{},vins:{},years:{}},Sa={state:Pn.NotLoaded,map:{}},Xc={state:Pn.NotLoaded},g$={form:Pn.NotLoaded},h$={answers:rt({},Sa),application:rt({},g$),bindQuotes:{},choices:{},carriersProducts:rt({},Xc),questionPools:rt({},Sa),questionSets:rt({},Sa),quotes:rt({},Xc),refilters:{},vehicles:rt({},v$)};var m$=Object.defineProperty,eu=Object.getOwnPropertySymbols,b$=Object.prototype.hasOwnProperty,y$=Object.prototype.propertyIsEnumerable,tu=(e,t,r)=>t in e?m$(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,ru=(e,t)=>{for(var r in t||(t={}))b$.call(t,r)&&tu(e,r,t[r]);if(eu)for(var r of eu(t))y$.call(t,r)&&tu(e,r,t[r]);return e};const _$={auth:Lt.NotLoaded,config:Lt.NotLoaded,health:Wc.Healthy,state:Lt.NotLoaded,user:Lt.NotLoaded,socket:zc.NotConnected,socketId:""};ru({},h$),ru({},_$);var $$=(e=>(e.QuestionSetSaved="QuestionSetSaved",e.AddressValidation_ValidatedOnFirstAttempt="AddressValidOnFirstAttempt",e.AddressValidation_Verdict="AddressValidationVerdict",e.AddressValidation_CTAClicked="AddressValidationCTAClicked",e.AddressValidation_ValidationError="AddressValidationValidationError",e))($$||{});function E(e){return e.uri?e.uri:e.env===jn.Production||e.env===void 0?"https://partner-engine.boldpenguin.com":e.env===jn.Development?"http://localhost:8080":`https://partner-engine-uat.${e.env}.boldpenguin.com`}var w$="11.0.1",A$=Object.defineProperty,O$=Object.defineProperties,S$=Object.getOwnPropertyDescriptors,nu=Object.getOwnPropertySymbols,j$=Object.prototype.hasOwnProperty,C$=Object.prototype.propertyIsEnumerable,ou=(e,t,r)=>t in e?A$(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,P$=(e,t)=>{for(var r in t||(t={}))j$.call(t,r)&&ou(e,r,t[r]);if(nu)for(var r of nu(t))C$.call(t,r)&&ou(e,r,t[r]);return e},E$=(e,t)=>O$(e,S$(t)),Y=(e=>(e.GET="GET",e.POST="POST",e.PUT="PUT",e.PATCH="PATCH",e.DELETE="DELETE",e))(Y||{});function I(e,t="GET",r){const n=new Headers({Accept:"application/json","Content-Type":"application/json","x-sdk-version":w$});e&&n.append("Authorization",`Bearer ${e}`);const o={credentials:"include",method:t,headers:n};return r&&(o.body=JSON.stringify(r)),o}function Qe(e){let t="";const r=Object.entries(e!=null?e:{}).filter(([,n])=>n);return r.length>0&&(t=`?${r.map(([n,o])=>{const a=encodeURIComponent(n),i=encodeURIComponent(o);return `${a}=${i}`}).join("&")}`),t}const T=(e,t,r)=>new Promise((n,o)=>{fetch(e,t).then(a=>{let i;return r!=null&&r.expectsEmptyResponse?i=x$:r!=null&&r.couldHaveEmptyResponse?i=T$:r!=null&&r.expectBlob?i=k$:i=q$,i(a,n,o)}).catch(a=>I$(a,o));});function I$(e,t){let r={error:e};e.error&&(r=e),t(r);}function au(e,t,r){const n=E$(P$({},t||{}),{error:e});r(n);}function T$(e,t,r){return e.ok?t(jr(e)):r(jr(e))}function q$(e,t,r){return e.json().then(n=>{const o=jr(e,n);(e.ok?t:r)(o);}).catch(n=>au(n,e,r))}function x$(e,t,r){const n=jr(e);(e.ok?t:r)(n);}function k$(e,t,r){return e.blob().then(n=>{const o=jr(e,n);(e.ok?t:r)(o);}).catch(n=>au(n,e,r))}function jr(e,t){return {status:e.status,redirected:e.redirected,url:e.url,metadata:{searchTotalItems:Number(e.headers.get("x-total"))},parsedBody:t}}let D$=class rb{static updateAnswer(t,r){const n=`${E(r)}/application_forms/${t.application_form_id}/answers/${t.id}`;return T(n,I(r.token,Y.PATCH,rb.transformAnswerData(t)))}static transformAnswerData(t){let r;return t.question.field_type===j.Select?r=null:t.question.field_type===j.CheckBox?r=t.value?"1":null:r=t.value,{answer:{choice_id:t.choice_id,choice_ids:t.choice_ids,value:r,previous_version:t.version,question_id:t.question.id}}}},nt=class{static getApplicationForm(t,r){const n=`${E(r)}/application_forms/${t}`;return T(n,I(r.token))}static getLatestApplicationForm(t,r){const n=`${E(r)}/application_forms/${t}/latest`;return T(n,I(r.token))}static getApplicationFormEitScenario(t,r){const n=`${E(r)}/application_forms/${t}/export/eit_scenario`;return T(n,I(r.token),{expectBlob:!0})}static createExchangeApplicationForm(t,r){const n=`${E(r)}/exchange_application_forms`;return T(n,I(r.token,Y.POST,t))}static createApplicationForm(t,r){const n=`${E(r)}/application_forms`;return T(n,I(r.token,Y.POST,t))}static cloneApplicationForm(t,r){const n=`${E(r)}/application_forms/${t}/clones`;return T(n,I(r.token,Y.POST))}static updateApplicationForm(t,r){const n=`${E(r)}/application_forms/${t.id}`;return T(n,I(r.token,Y.PATCH,t))}static deleteApplicationForm(t,r){const n=`${E(r)}/application_forms/${t.id}`;return T(n,I(r.token,Y.DELETE))}static getApplicationFormFromToken(t,r){const n=`${E(r)}/tokens/${t}`;return T(n,I(r.token,Y.GET))}static getTokenForAuthGuestFlow(t){const r=E(t);return T(`${r}/tenants/${t.tenantId}/application_forms/guest`,I(void 0,Y.POST,{source:t.applicationSource,source_metadata:t.applicationMetadata,application_form:t.applicationForm}))}static searchApplicationForms(t,{sortColumn:r="",sortOrder:n="asc",pageNumber:o=0,pageSize:a=10,answers:i=[],q:s=""}={}){const c=Qe({column:r,order:n,page:o.toString(),per_page:a.toString(),questions:this.serializeAnswers(i),q:s}),u=`${E(t)}/application_forms/${c}`;return T(u,I(t.token))}static getEligibilityResults(t,r){const n=`${E(r)}/application_forms/${t}/eligibility_results`;return T(n,I(r.token,Y.GET))}static updateApplicationToNextBucket(t,r,n){const o=`${E(n)}/application_forms/${t}/add_next_bucket`;return T(o,I(n.token,Y.POST,{product_codes:r}),{expectsEmptyResponse:!0})}static createCurrentTierSkips(t,r){const n=`${E(r)}/application_forms/${t.id}/current_tier_skips`;return T(n,I(r.token,Y.POST,t))}static serializeAnswers(t){const r={};for(const n of t)r[n.question.id]={value:n.value,choice_ids:n.choice_ids,choice:n.choice_id};return JSON.stringify(r)}static getCarrierPortalUrl(t,r,n){const o=`${E(n)}/application_forms/${t}/carriers/${r}/portal_url`;return T(o,I(n.token))}},ja=class{static getQuestionSet(t,r,n){const o=`${E(n)}/application_forms/${t.id}/application_forms_question_sets/${r.id}`;return T(o,I(n.token))}static updateQuestionSet(t,r){const n=`${E(r)}/application_forms/${t.application_form_id}/application_forms_question_sets/${t.id}`;return T(n,I(r.token,Y.PATCH,this.transformQuestionSetDataForUpdate(t)))}static createQuestionSet(t,r){const n=`${E(r)}/application_forms/${t.application_form_id}/application_forms_question_sets`;return T(n,I(r.token,Y.POST,this.transformQuestionSetDataForCreate(t)))}static transformQuestionSetDataForUpdate(t){return {application_forms_question_set:{is_completed:t.is_completed}}}static transformQuestionSetDataForCreate(t){return {application_forms_question_set:{question_set:{code:t.question_set.code}}}}};function iu(e){const t={};return Object.keys(e).forEach(r=>{e[r]&&(t[r]=e[r]);}),t}let su=class xs{static getChoices(t,r){const n=`${E(r)}/questions/${t}/choices`;return T(n,I(r.token))}static getChoice(t,r){const n=`${E(r)}/choices/${t}`;return T(n,I(r.token))}static addChoice(t,r){return T(`${E(r)}/choices`,I(r.token,Y.POST,xs.toChoiceObject(t)))}static updateChoice(t,r){const n=`${E(r)}/choices/${t.id}`;return T(n,I(r.token,Y.PATCH,xs.toChoiceObject(t)))}static searchChoices(t,r,n){const o=iu(r),a=`${E(n)}/questions/${t}/choices${Qe(o)}`;return T(a,I(n.token))}static searchChoicesWithNudges(t,r,n){const o=iu(r),a=`${E(n)}/questions/${t}/choices${Qe(o)}`;return T(a,I(n.token))}static toChoiceObject(t){return {choice:{value:t.value,alias:t.alias,question_id:t.question_id,order:t.order}}}},L$=class{static getFeatures(t){const r=`${E(t)}/features`;return T(r,I(t.token,Y.GET))}},cu=class{static addQuestionPool({application_id:t,config:r,question_pool_id:n,parent_answer_pool_id:o}){const a=`${E(r)}/application_forms/${t}/answer_pools`;return T(a,I(r.token,Y.POST,{question_pool_id:n,parent_answer_pool_id:o}))}static removeQuestionPool({application_id:t,config:r,answer_pool_id:n}){const o=`${E(r)}/application_forms/${t}/answer_pools/${n}`;return T(o,I(r.token,Y.DELETE))}},N$=class{static getApplicationFormCarriersProducts(t,r){const n=`${E(r)}/application_forms/${t}/carriers_products`;return T(n,I(r.token))}},uu=class nb{static getQuoteRequests(t,r){const n=`${E(r)}/application_forms/${t}/quote_requests`;return T(n,I(r.token))}static bindQuoteAction(t,r,n){const o=Qe({bind_action:t.value}),a=`${E(n)}/application_forms/${r.application_form_id}/quote_requests/${r.id}/bind/${o}`;return T(a,I(n.token),{couldHaveEmptyResponse:!0})}static bindQuote(t,r,n){const o=`${E(n)}/application_forms/${r}/quote_requests/${t}/bind`;return T(o,I(n.token))}static quoteDetails(t,r){const n=`${E(r)}/application_forms/${t.application_form_id}/quote_requests/${t.id}/details`;return T(n,I(r.token))}static updateQuote(t,r){const n=`${E(r)}/application_forms/${t.application_form_id}/quote_requests/${t.id}`;return T(n,I(r.token,Y.PATCH,nb.toQuoteRequestObject(t)))}static toQuoteRequestObject(t){return {quote_request:{focused:t.focused}}}},de=class{static getYears(t){const r=`${E(t)}/vehicles/years`;return T(r,I(t.token))}static getMakes(t,r,n){const o=`${E(r)}/vehicles/years/${t}/makes${Qe({type:n==null?void 0:n.type,name:n==null?void 0:n.name})}`;return T(o,I(r.token))}static getModels(t,r,n,o){const a=`${E(n)}/vehicles/years/${t}/makes/${r}/models${Qe({type:o})}`;return T(a,I(n.token))}static getConfigurations(t,r,n,o,a){const i=`${E(o)}/vehicles/years/${t}/makes/${r}/models/${n}/configurations${Qe({type:a})}`;return T(i,I(o.token))}static getVinDetails(t,r){const n=`${E(r)}/vehicles/vins/${t}`;return T(n,I(r.token))}static selectVehicle(t){var r,n,o;const{appId:a,questionGroupId:i,configurationId:s,flowType:c,config:u,answerPoolId:l,type:d,vehicle:f}=t,g={question_group_id:i,configuration_id:s,flow:c,answer_pool_id:l};l||delete g.answer_pool_id;const v=`${E(u)}/application_forms/${a}/vehicles${Qe({type:d,year:(r=f==null?void 0:f.yearId)!=null?r:void 0,make:(n=f==null?void 0:f.makeId)!=null?n:void 0,model:(o=f==null?void 0:f.modelId)!=null?o:void 0})}`;return T(v,I(u.token,Y.POST,g))}};var R$=(e=>(e.LIGHT="Light",e.COLOR="Color",e))(R$||{}),M$=(e=>(e.CONFIRMED="CONFIRMED",e.UNCONFIRMED_BUT_PLAUSIBLE="UNCONFIRMED_BUT_PLAUSIBLE",e.UNCONFIRMED_BUT_SUSPICIOUS="UNCONFIRMED_BUT_SUSPICIOUS",e))(M$||{}),Ue=(e=>(e.ROUTE="route",e.STREET_NUMBER="street_number",e.STREET_ADDRESS="street_address",e.SUBPREMISE="subpremise",e.LOCALITY="locality",e.STATE="administrative_area_level_1",e.POSTAL_CODE="postal_code",e.POSTAL_CODE_SUFFIX="postal_code_suffix",e))(Ue||{}),Nt=(e=>(e.STREET_1="street_1",e.STREET_2="street_2",e.CITY="city",e.STATE="state",e.ZIPCODE="zipcode",e))(Nt||{}),In=(e=>(e.ApplicationForms="application_forms",e.PartnerPhoneCalls="partner_phone_calls",e.FeatureCheck="features",e.Questions="questions",e.Rules="rules",e.RuleExcludedChoices="rule_excluded_choices",e.SearchableQuestions="searchable_questions",e.SendToExchange="send_to_exchange",e.TakeOver="take_over",e))(In||{}),lu=(e=>(e.NotConnected="NotConnected",e.Connected="Connected",e.Disconnected="Disconnected",e.Connecting="Connecting",e.Disconnecting="Disconnecting",e))(lu||{}),V$=(e=>(e.Create="create",e.Index="index",e.Show="show",e.Update="update",e))(V$||{}),du=(e=>(e.Development="development",e.Alpha="alpha",e.Beta="beta",e.Production="prod",e))(du||{}),F$=(e=>(e.Accepted="Accepted",e.Expired="Expired",e.Not_Created="Not created",e.Queued="Queued to send",e.Sent="Sent",e))(F$||{}),Q$=(e=>(e.INVITE="invite",e.RESET="reset",e))(Q$||{}),Rt=(e=>(e.token="token",e.cableToken="cable_token",e.user="user",e.state="state",e.expiresAt="expiresAt",e.refreshAt="refreshAt",e.impersonated="impersonated",e.terminalTheme="terminalTheme",e))(Rt||{}),Mt=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Configured="Configured",e.Error="Error",e))(Mt||{}),pu=(e=>(e.Healthy="Healthy",e.Error="Error",e))(pu||{}),U$=(e=>(e.OAuth="OAuth",e.Login="Login",e))(U$||{}),B$=(e=>(e.storageKey="BP_SDK_ANALYTICS_VARS",e.additionalQuoteOnline="onlinebuy",e.additionalQuoteOffline="nobuy",e))(B$||{}),G$=(e=>(e.activeIdUpdate="activeIdUpdate",e.activeQuestionSetChange="activeQuestionSetChange",e.addItem="addItem",e.answerStateChanged="answerStateChanged",e.bpAnalyticsTrack="bpAnalyticsTrack",e.closePanel="closePanel",e.deleteItem="deleteItem",e.deletePool="deletePool",e.epFieldChanged="epFieldChanged",e.errorsUpdate="errorsUpdate",e.fakeValueUpdate="fakeValueUpdate",e.filterUpdate="filterUpdate",e.formLoaded="formLoaded",e.nudgeAnswer="nudgeAnswer",e.openPanel="openPanel",e.placesUpdate="placesUpdate",e.questionSetAdded="questionSetAdded",e.questionSetCompletionChanged="questionSetCompletionChanged",e.quoteFailure="quoteFailure",e.quotesChanged="quotesChanged",e.quoteSuccess="quoteSuccess",e.removeActiveAnswer="removeActiveAnswer",e.saveAnswer="saveAnswer",e.saveVehicle="saveVehicle",e.sdkError="sdkError",e.tabChanged="tabChanged",e.updateActiveAnswer="updateActiveAnswer",e.userAuthenticationChanged="userAuthenticationChanged",e.valueUpdate="valueUpdate",e.vinUpdate="vinUpdate",e.ymmcUpdate="ymmcUpdate",e))(G$||{}),z$=(e=>(e.vin="vin",e.ymm="ymm",e.trailers="Trailers",e))(z$||{}),W$=(e=>(e.trailers="trailers",e))(W$||{}),H$=(e=>(e.Configuration="configuration",e.Make="make",e.Model="model",e.Year="year",e))(H$||{}),Ca=(e=>(e.Address="address",e.Accordion="accordion",e.AccumulatePercentage="accumulate_percentage",e.Vehicle="vehicle",e.Driver="driver",e.Stepper="stepper",e.Step="step",e.Coverage="coverage",e.AutoCollapsedAccordion="auto_collapsed_accordion",e.ValidatedAddress="validated_address",e))(Ca||{}),O=(e=>(e.Address="address",e.Anchor="anchor",e.CheckBox="checkbox",e.CoverageType="coverage_type",e.Currency="currency",e.Date="date",e.DeletePool="delete_pool",e.Dialog="dialog",e.DriversLicense="drivers_license",e.DropDown="dropdown",e.Email="email",e.FederalEmployeeIdentification="fein",e.JSON="json",e.Label="label",e.Masked="masked",e.MonthYear="month_year",e.Number="number",e.Object="object",e.Percentage="percentage",e.Phone="phone",e.Radio="radio",e.Select="select",e.SocialSecurityNumber="ssn",e.Step="step",e.Tenant="tenant",e.Text="text",e.Vehicle="vehicle",e.Url="url",e.VIN="vin",e.Year="year",e.ZipCode="zipcode",e))(O||{}),Vt=(e=>(e.Accumulator="accumulator",e.AddressJSON="address_json",e.BusinessClass="business_class",e.City="city",e.CommercialAutoACV="ca_acv",e.CommercialAutoCategory="ca_category",e.CommercialAutoCount="ca_count",e.CommercialAutoDrivetrain="ca_drivetrain",e.CommercialAutoFlow="ca_flow",e.CommercialAutoGCW="ca_gcw",e.CommercialAutoGVW="ca_gvw",e.CommercialAutoJson="ca_json",e.CommercialAutoMake="ca_make",e.CommercialAutoModel="ca_model",e.CommercialAutoMSRP="ca_msrp",e.CommercialAutoSubModel="ca_submodel",e.CommercialAutoTractor="ca_tractor",e.CommercialAutoVIN="ca_vin",e.CommercialAutoYear="ca_year",e.County="county",e.DriversLicenseState="drivers_license_state",e.Indent="indent",e.Increment="increment",e.JornayaLeadID="jornaya_lead_id",e.LinesOfBusiness="lines_of_business",e.LinesOfBusinessAdditional="lines_of_business_additional",e.MotorVehicleReportComplete="mvr_complete",e.MotorVehicleReportState="mvr_state",e.MotorVehicleReportCondition="mvr_report_condition",e.State="state",e.Street1="street_1",e.Street2="street_2",e.ZipCode="zipcode",e))(Vt||{}),Z$=(e=>(e.CoverageLimit="coverage limit",e))(Z$||{}),se=(e=>(e.MaximumDate="maxDate",e.MaximumValue="max",e.MinimumDate="minDate",e.MinimumValue="min",e.Pattern="pattern",e.Required="required",e))(se||{}),K$=(e=>(e.Acord="acord",e.Acord125="acord_125",e.Acord126="acord_126",e.Acord130="acord_130",e))(K$||{}),Y$=(e=>(e.BP_QUOTABLE="bp_quotable",e.OFFLINE="offline",e.ONLINE="online",e))(Y$||{}),J$=(e=>(e.Pending="pending",e.Queued="queued",e.Executing="executing",e.Completed="completed",e.Failed="failed",e.Aborted="aborted",e))(J$||{}),X$=(e=>(e.ClueReport="ClueReport",e.MotorVehicleReport="MotorVehicleReport",e.RequestPrefill="RequestPrefill",e.DriverScore="DriverScore",e.JornayaAudit="JornayaAudit",e.DotNumber="DotNumber",e))(X$||{}),Tn=(e=>(e.Incomplete="incomplete",e.InProgress="in_progress",e.Completed="completed",e.Quoted="quoted",e))(Tn||{}),ew=(e=>(e.not_permitted_to_bind="not_permitted_to_bind",e.sent_to_wholesaler="sent_to_wholesaler",e))(ew||{}),tw=(e=>(e.dialog="dialog",e.wholesale_workflow="wholesale_workflow",e))(tw||{}),rw=(e=>(e.NON_ADMITTED="non_admitted",e.WHOLESALER="wholesaler",e.INSTANT_ENROLLMENT="instant_enrollment",e.ADDITIONAL_MARKET="additional_market",e.OPT_IN_QUOTE_FLOW="opt_in_quote_flow",e))(rw||{});const nw=[O.Number,O.Currency,O.Percentage,O.Year];var ow=(e=>(e.Sub="sub",e.Direct="direct",e))(ow||{}),aw=(e=>(e.Answer="answer",e.Application="application_form",e.QuoteRequest="quote_request",e))(aw||{}),iw=(e=>(e.info="info",e.warning="warning",e.error="error",e))(iw||{}),sw=(e=>(e.QuestionSet="question_set",e.Question="question",e))(sw||{}),cw=(e=>(e.ReadyToBind="ready_to_bind",e))(cw||{}),fu=(e=>(e.unsent="unsent",e.in_progress="in_progress",e.sent="sent",e.completed="completed",e.failed="failed",e.timed_out="timed_out",e.ineligible="ineligible",e.referral="referral",e.bound="bound",e))(fu||{}),uw=(e=>(e.AddQuestionSet="application_forms_question_set",e.Answer="answer",e.Appetite="appetite",e.ApplicationForm="application_form",e.ApplicationFormPercentCompleted="application_form_percent_completed",e.ApplicationFormProcess="application_form_process",e.Batch="batch",e.CarrierProducts="eligible_carriers_products",e.Message="message",e.Notification="notification",e.QuoteRequest="quote_request",e.RealTimeEligibility="real_time_eligibility",e.RemoveQuestionSet="destroy_application_forms_question_set",e))(uw||{}),lw=(e=>(e.AcordForm="acord_form_pdf",e.AgentQuoteStart="agent_quote_start",e.ClientQuoteStart="client_quote_start",e.Storefront="storefront",e))(lw||{}),ye=(e=>(e.NotLoaded="NotLoaded",e.Loading="Loading",e.Loaded="Loaded",e.Error="Error",e))(ye||{});const dw={configurations:{},makes:{},models:{},selection:{},vin:{},years:{}},pw={list:[],isLoaded:!1,status:Mt.NotLoaded},fw={filteredOnlineQuotes:[],filteredOfflineQuotes:[]};var vw=(e=>(e.FIX="fix",e.ACCEPT="accept",e.CONFIRM="confirm",e.TYPO="typo",e))(vw||{}),gw=(e=>(e.REQUIRED="required",e.READY_TO_VALIDATE="ready to validate",e.VALIDATED="validated",e.PROCEED_NEXT="proceed next",e.COMPLETED="completed",e.VALIDATION_ERROR="error",e))(gw||{}),hw=(e=>(e.EDIT="return to edit address",e.AS_ENTERED="continue with original address",e.CONFIRM="continue with confirm suggested address",e.MISSING_FIX="continue with missing input fix",e))(hw||{});const mw={verdict:null,responseId:"",address:null,stagedAnswers:[],validationStatus:"required",attempt:0,originalAddress:""};var bw=Object.defineProperty,vu=Object.getOwnPropertySymbols,yw=Object.prototype.hasOwnProperty,_w=Object.prototype.propertyIsEnumerable,gu=(e,t,r)=>t in e?bw(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,qn=(e,t)=>{for(var r in t||(t={}))yw.call(t,r)&&gu(e,r,t[r]);if(vu)for(var r of vu(t))_w.call(t,r)&&gu(e,r,t[r]);return e};qn({},dw),qn({},pw),qn({},fw),qn({},mw);var $w=Object.defineProperty,hu=Object.getOwnPropertySymbols,ww=Object.prototype.hasOwnProperty,Aw=Object.prototype.propertyIsEnumerable,mu=(e,t,r)=>t in e?$w(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,ot=(e,t)=>{for(var r in t||(t={}))ww.call(t,r)&&mu(e,r,t[r]);if(hu)for(var r of hu(t))Aw.call(t,r)&&mu(e,r,t[r]);return e};const Ow={configurations:{},makes:{},models:{},selection:{},vins:{},years:{}},Pa={state:ye.NotLoaded,map:{}},bu={state:ye.NotLoaded},Sw={form:ye.NotLoaded},jw={answers:ot({},Pa),application:ot({},Sw),bindQuotes:{},choices:{},carriersProducts:ot({},bu),questionPools:ot({},Pa),questionSets:ot({},Pa),quotes:ot({},bu),refilters:{},vehicles:ot({},Ow)};var Cw=Object.defineProperty,yu=Object.getOwnPropertySymbols,Pw=Object.prototype.hasOwnProperty,Ew=Object.prototype.propertyIsEnumerable,_u=(e,t,r)=>t in e?Cw(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,$u=(e,t)=>{for(var r in t||(t={}))Pw.call(t,r)&&_u(e,r,t[r]);if(yu)for(var r of yu(t))Ew.call(t,r)&&_u(e,r,t[r]);return e};const Iw={auth:Mt.NotLoaded,config:Mt.NotLoaded,health:pu.Healthy,state:Mt.NotLoaded,user:Mt.NotLoaded,socket:lu.NotConnected,socketId:""};$u({},jw),$u({},Iw);var Tw=(e=>(e.QuestionSetSaved="QuestionSetSaved",e.AddressValidation_ValidatedOnFirstAttempt="AddressValidOnFirstAttempt",e.AddressValidation_Verdict="AddressValidationVerdict",e.AddressValidation_CTAClicked="AddressValidationCTAClicked",e.AddressValidation_ValidationError="AddressValidationValidationError",e))(Tw||{}),xn=typeof globalThis!="undefined"?globalThis:typeof window!="undefined"?window:typeof global!="undefined"?global:typeof self!="undefined"?self:{};function Cr(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}function qw(e,t){for(var r=-1,n=e==null?0:e.length,o=Array(n);++r<n;)o[r]=t(e[r],r,e);return o}var Ea=qw,xw=Array.isArray,he=xw,kw=typeof xn=="object"&&xn&&xn.Object===Object&&xn,wu=kw,Dw=wu,Lw=typeof self=="object"&&self&&self.Object===Object&&self,Nw=Dw||Lw||Function("return this")(),Pe=Nw,Rw=Pe,Mw=Rw.Symbol,Pr=Mw,Au=Pr,Ou=Object.prototype,Vw=Ou.hasOwnProperty,Fw=Ou.toString,Er=Au?Au.toStringTag:void 0;function Qw(e){var t=Vw.call(e,Er),r=e[Er];try{e[Er]=void 0;var n=!0;}catch(a){}var o=Fw.call(e);return n&&(t?e[Er]=r:delete e[Er]),o}var Uw=Qw,Bw=Object.prototype,Gw=Bw.toString;function zw(e){return Gw.call(e)}var Ww=zw,Su=Pr,Hw=Uw,Zw=Ww,Kw="[object Null]",Yw="[object Undefined]",ju=Su?Su.toStringTag:void 0;function Jw(e){return e==null?e===void 0?Yw:Kw:ju&&ju in Object(e)?Hw(e):Zw(e)}var Ir=Jw;function Xw(e){return e!=null&&typeof e=="object"}var Ft=Xw,eA=Ir,tA=Ft,rA="[object Symbol]";function nA(e){return typeof e=="symbol"||tA(e)&&eA(e)==rA}var kn=nA,oA=he,aA=kn,iA=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,sA=/^\w*$/;function cA(e,t){if(oA(e))return !1;var r=typeof e;return r=="number"||r=="symbol"||r=="boolean"||e==null||aA(e)?!0:sA.test(e)||!iA.test(e)||t!=null&&e in Object(t)}var Ia=cA;function uA(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}var Ta=uA,lA=Ir,dA=Ta,pA="[object AsyncFunction]",fA="[object Function]",vA="[object GeneratorFunction]",gA="[object Proxy]";function hA(e){if(!dA(e))return !1;var t=lA(e);return t==fA||t==vA||t==pA||t==gA}var Cu=hA,mA=Pe,bA=mA["__core-js_shared__"],yA=bA,qa=yA,Pu=function(){var e=/[^.]+$/.exec(qa&&qa.keys&&qa.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function _A(e){return !!Pu&&Pu in e}var $A=_A,wA=Function.prototype,AA=wA.toString;function OA(e){if(e!=null){try{return AA.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var Eu=OA,SA=Cu,jA=$A,CA=Ta,PA=Eu,EA=/[\\^$.*+?()[\]{}|]/g,IA=/^\[object .+?Constructor\]$/,TA=Function.prototype,qA=Object.prototype,xA=TA.toString,kA=qA.hasOwnProperty,DA=RegExp("^"+xA.call(kA).replace(EA,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function LA(e){if(!CA(e)||jA(e))return !1;var t=SA(e)?DA:IA;return t.test(PA(e))}var NA=LA;function RA(e,t){return e==null?void 0:e[t]}var MA=RA,VA=NA,FA=MA;function QA(e,t){var r=FA(e,t);return VA(r)?r:void 0}var at=QA,UA=at,BA=UA(Object,"create"),Dn=BA,Iu=Dn;function GA(){this.__data__=Iu?Iu(null):{},this.size=0;}var zA=GA;function WA(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var HA=WA,ZA=Dn,KA="__lodash_hash_undefined__",YA=Object.prototype,JA=YA.hasOwnProperty;function XA(e){var t=this.__data__;if(ZA){var r=t[e];return r===KA?void 0:r}return JA.call(t,e)?t[e]:void 0}var e0=XA,t0=Dn,r0=Object.prototype,n0=r0.hasOwnProperty;function o0(e){var t=this.__data__;return t0?t[e]!==void 0:n0.call(t,e)}var a0=o0,i0=Dn,s0="__lodash_hash_undefined__";function c0(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=i0&&t===void 0?s0:t,this}var u0=c0,l0=zA,d0=HA,p0=e0,f0=a0,v0=u0;function Qt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Qt.prototype.clear=l0,Qt.prototype.delete=d0,Qt.prototype.get=p0,Qt.prototype.has=f0,Qt.prototype.set=v0;var g0=Qt;function h0(){this.__data__=[],this.size=0;}var m0=h0;function b0(e,t){return e===t||e!==e&&t!==t}var Tu=b0,y0=Tu;function _0(e,t){for(var r=e.length;r--;)if(y0(e[r][0],t))return r;return -1}var Ln=_0,$0=Ln,w0=Array.prototype,A0=w0.splice;function O0(e){var t=this.__data__,r=$0(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():A0.call(t,r,1),--this.size,!0}var S0=O0,j0=Ln;function C0(e){var t=this.__data__,r=j0(t,e);return r<0?void 0:t[r][1]}var P0=C0,E0=Ln;function I0(e){return E0(this.__data__,e)>-1}var T0=I0,q0=Ln;function x0(e,t){var r=this.__data__,n=q0(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}var k0=x0,D0=m0,L0=S0,N0=P0,R0=T0,M0=k0;function Ut(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Ut.prototype.clear=D0,Ut.prototype.delete=L0,Ut.prototype.get=N0,Ut.prototype.has=R0,Ut.prototype.set=M0;var Nn=Ut,V0=at,F0=Pe,Q0=V0(F0,"Map"),xa=Q0,qu=g0,U0=Nn,B0=xa;function G0(){this.size=0,this.__data__={hash:new qu,map:new(B0||U0),string:new qu};}var z0=G0;function W0(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}var H0=W0,Z0=H0;function K0(e,t){var r=e.__data__;return Z0(t)?r[typeof t=="string"?"string":"hash"]:r.map}var Rn=K0,Y0=Rn;function J0(e){var t=Y0(this,e).delete(e);return this.size-=t?1:0,t}var X0=J0,eO=Rn;function tO(e){return eO(this,e).get(e)}var rO=tO,nO=Rn;function oO(e){return nO(this,e).has(e)}var aO=oO,iO=Rn;function sO(e,t){var r=iO(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}var cO=sO,uO=z0,lO=X0,dO=rO,pO=aO,fO=cO;function Bt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Bt.prototype.clear=uO,Bt.prototype.delete=lO,Bt.prototype.get=dO,Bt.prototype.has=pO,Bt.prototype.set=fO;var ka=Bt,xu=ka,vO="Expected a function";function Da(e,t){if(typeof e!="function"||t!=null&&typeof t!="function")throw new TypeError(vO);var r=function(){var n=arguments,o=t?t.apply(this,n):n[0],a=r.cache;if(a.has(o))return a.get(o);var i=e.apply(this,n);return r.cache=a.set(o,i)||a,i};return r.cache=new(Da.Cache||xu),r}Da.Cache=xu;var gO=Da,hO=gO,mO=500;function bO(e){var t=hO(e,function(n){return r.size===mO&&r.clear(),n}),r=t.cache;return t}var yO=bO,_O=yO,$O=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,wO=/\\(\\)?/g,AO=_O(function(e){var t=[];return e.charCodeAt(0)===46&&t.push(""),e.replace($O,function(r,n,o,a){t.push(o?a.replace(wO,"$1"):n||r);}),t}),OO=AO,ku=Pr,SO=Ea,jO=he,CO=kn,PO=1/0,Du=ku?ku.prototype:void 0,Lu=Du?Du.toString:void 0;function Nu(e){if(typeof e=="string")return e;if(jO(e))return SO(e,Nu)+"";if(CO(e))return Lu?Lu.call(e):"";var t=e+"";return t=="0"&&1/e==-PO?"-0":t}var EO=Nu,IO=EO;function TO(e){return e==null?"":IO(e)}var qO=TO,xO=he,kO=Ia,DO=OO,LO=qO;function NO(e,t){return xO(e)?e:kO(e,t)?[e]:DO(LO(e))}var Ru=NO,RO=kn,MO=1/0;function VO(e){if(typeof e=="string"||RO(e))return e;var t=e+"";return t=="0"&&1/e==-MO?"-0":t}var Mn=VO,FO=Ru,QO=Mn;function UO(e,t){t=FO(t,e);for(var r=0,n=t.length;e!=null&&r<n;)e=e[QO(t[r++])];return r&&r==n?e:void 0}var La=UO,BO=Nn;function GO(){this.__data__=new BO,this.size=0;}var zO=GO;function WO(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}var HO=WO;function ZO(e){return this.__data__.get(e)}var KO=ZO;function YO(e){return this.__data__.has(e)}var JO=YO,XO=Nn,eS=xa,tS=ka,rS=200;function nS(e,t){var r=this.__data__;if(r instanceof XO){var n=r.__data__;if(!eS||n.length<rS-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new tS(n);}return r.set(e,t),this.size=r.size,this}var oS=nS,aS=Nn,iS=zO,sS=HO,cS=KO,uS=JO,lS=oS;function Gt(e){var t=this.__data__=new aS(e);this.size=t.size;}Gt.prototype.clear=iS,Gt.prototype.delete=sS,Gt.prototype.get=cS,Gt.prototype.has=uS,Gt.prototype.set=lS;var Mu=Gt,dS="__lodash_hash_undefined__";function pS(e){return this.__data__.set(e,dS),this}var fS=pS;function vS(e){return this.__data__.has(e)}var gS=vS,hS=ka,mS=fS,bS=gS;function Vn(e){var t=-1,r=e==null?0:e.length;for(this.__data__=new hS;++t<r;)this.add(e[t]);}Vn.prototype.add=Vn.prototype.push=mS,Vn.prototype.has=bS;var Na=Vn;function yS(e,t){for(var r=-1,n=e==null?0:e.length;++r<n;)if(t(e[r],r,e))return !0;return !1}var _S=yS;function $S(e,t){return e.has(t)}var Ra=$S,wS=Na,AS=_S,OS=Ra,SS=1,jS=2;function CS(e,t,r,n,o,a){var i=r&SS,s=e.length,c=t.length;if(s!=c&&!(i&&c>s))return !1;var u=a.get(e),l=a.get(t);if(u&&l)return u==t&&l==e;var d=-1,f=!0,g=r&jS?new wS:void 0;for(a.set(e,t),a.set(t,e);++d<s;){var v=e[d],h=t[d];if(n)var b=i?n(h,v,d,t,e,a):n(v,h,d,e,t,a);if(b!==void 0){if(b)continue;f=!1;break}if(g){if(!AS(t,function(m,_){if(!OS(g,_)&&(v===m||o(v,m,r,n,a)))return g.push(_)})){f=!1;break}}else if(!(v===h||o(v,h,r,n,a))){f=!1;break}}return a.delete(e),a.delete(t),f}var Vu=CS,PS=Pe,ES=PS.Uint8Array,IS=ES;function TS(e){var t=-1,r=Array(e.size);return e.forEach(function(n,o){r[++t]=[o,n];}),r}var qS=TS;function xS(e){var t=-1,r=Array(e.size);return e.forEach(function(n){r[++t]=n;}),r}var Ma=xS,Fu=Pr,Qu=IS,kS=Tu,DS=Vu,LS=qS,NS=Ma,RS=1,MS=2,VS="[object Boolean]",FS="[object Date]",QS="[object Error]",US="[object Map]",BS="[object Number]",GS="[object RegExp]",zS="[object Set]",WS="[object String]",HS="[object Symbol]",ZS="[object ArrayBuffer]",KS="[object DataView]",Uu=Fu?Fu.prototype:void 0,Va=Uu?Uu.valueOf:void 0;function YS(e,t,r,n,o,a,i){switch(r){case KS:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return !1;e=e.buffer,t=t.buffer;case ZS:return !(e.byteLength!=t.byteLength||!a(new Qu(e),new Qu(t)));case VS:case FS:case BS:return kS(+e,+t);case QS:return e.name==t.name&&e.message==t.message;case GS:case WS:return e==t+"";case US:var s=LS;case zS:var c=n&RS;if(s||(s=NS),e.size!=t.size&&!c)return !1;var u=i.get(e);if(u)return u==t;n|=MS,i.set(e,t);var l=DS(s(e),s(t),n,o,a,i);return i.delete(e),l;case HS:if(Va)return Va.call(e)==Va.call(t)}return !1}var JS=YS;function XS(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var Bu=XS,e1=Bu,t1=he;function r1(e,t,r){var n=t(e);return t1(e)?n:e1(n,r(e))}var n1=r1;function o1(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}var Gu=o1;function a1(){return []}var i1=a1,s1=Gu,c1=i1,u1=Object.prototype,l1=u1.propertyIsEnumerable,zu=Object.getOwnPropertySymbols,d1=zu?function(e){return e==null?[]:(e=Object(e),s1(zu(e),function(t){return l1.call(e,t)}))}:c1,p1=d1;function f1(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var v1=f1,g1=Ir,h1=Ft,m1="[object Arguments]";function b1(e){return h1(e)&&g1(e)==m1}var y1=b1,Wu=y1,_1=Ft,Hu=Object.prototype,$1=Hu.hasOwnProperty,w1=Hu.propertyIsEnumerable,A1=Wu(function(){return arguments}())?Wu:function(e){return _1(e)&&$1.call(e,"callee")&&!w1.call(e,"callee")},Fa=A1,Fn={exports:{}};function O1(){return !1}var S1=O1;(function(e,t){var r=Pe,n=S1,o=t&&!t.nodeType&&t,a=o&&!0&&e&&!e.nodeType&&e,i=a&&a.exports===o,s=i?r.Buffer:void 0,c=s?s.isBuffer:void 0,u=c||n;e.exports=u;})(Fn,Fn.exports);var Zu=Fn.exports,j1=9007199254740991,C1=/^(?:0|[1-9]\d*)$/;function P1(e,t){var r=typeof e;return t=t==null?j1:t,!!t&&(r=="number"||r!="symbol"&&C1.test(e))&&e>-1&&e%1==0&&e<t}var Ku=P1,E1=9007199254740991;function I1(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=E1}var Qa=I1,T1=Ir,q1=Qa,x1=Ft,k1="[object Arguments]",D1="[object Array]",L1="[object Boolean]",N1="[object Date]",R1="[object Error]",M1="[object Function]",V1="[object Map]",F1="[object Number]",Q1="[object Object]",U1="[object RegExp]",B1="[object Set]",G1="[object String]",z1="[object WeakMap]",W1="[object ArrayBuffer]",H1="[object DataView]",Z1="[object Float32Array]",K1="[object Float64Array]",Y1="[object Int8Array]",J1="[object Int16Array]",X1="[object Int32Array]",ej="[object Uint8Array]",tj="[object Uint8ClampedArray]",rj="[object Uint16Array]",nj="[object Uint32Array]",Q={};Q[Z1]=Q[K1]=Q[Y1]=Q[J1]=Q[X1]=Q[ej]=Q[tj]=Q[rj]=Q[nj]=!0,Q[k1]=Q[D1]=Q[W1]=Q[L1]=Q[H1]=Q[N1]=Q[R1]=Q[M1]=Q[V1]=Q[F1]=Q[Q1]=Q[U1]=Q[B1]=Q[G1]=Q[z1]=!1;function oj(e){return x1(e)&&q1(e.length)&&!!Q[T1(e)]}var aj=oj;function ij(e){return function(t){return e(t)}}var Ua=ij,Qn={exports:{}};(function(e,t){var r=wu,n=t&&!t.nodeType&&t,o=n&&!0&&e&&!e.nodeType&&e,a=o&&o.exports===n,i=a&&r.process,s=function(){try{var c=o&&o.require&&o.require("util").types;return c||i&&i.binding&&i.binding("util")}catch(u){}}();e.exports=s;})(Qn,Qn.exports);var sj=Qn.exports,cj=aj,uj=Ua,Yu=sj,Ju=Yu&&Yu.isTypedArray,lj=Ju?uj(Ju):cj,Xu=lj,dj=v1,pj=Fa,fj=he,vj=Zu,gj=Ku,hj=Xu,mj=Object.prototype,bj=mj.hasOwnProperty;function yj(e,t){var r=fj(e),n=!r&&pj(e),o=!r&&!n&&vj(e),a=!r&&!n&&!o&&hj(e),i=r||n||o||a,s=i?dj(e.length,String):[],c=s.length;for(var u in e)(t||bj.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||gj(u,c)))&&s.push(u);return s}var _j=yj,$j=Object.prototype;function wj(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||$j;return e===r}var Aj=wj;function Oj(e,t){return function(r){return e(t(r))}}var Sj=Oj,jj=Sj,Cj=jj(Object.keys,Object),Pj=Cj,Ej=Aj,Ij=Pj,Tj=Object.prototype,qj=Tj.hasOwnProperty;function xj(e){if(!Ej(e))return Ij(e);var t=[];for(var r in Object(e))qj.call(e,r)&&r!="constructor"&&t.push(r);return t}var kj=xj,Dj=Cu,Lj=Qa;function Nj(e){return e!=null&&Lj(e.length)&&!Dj(e)}var Un=Nj,Rj=_j,Mj=kj,Vj=Un;function Fj(e){return Vj(e)?Rj(e):Mj(e)}var Ba=Fj,Qj=n1,Uj=p1,Bj=Ba;function Gj(e){return Qj(e,Bj,Uj)}var zj=Gj,el=zj,Wj=1,Hj=Object.prototype,Zj=Hj.hasOwnProperty;function Kj(e,t,r,n,o,a){var i=r&Wj,s=el(e),c=s.length,u=el(t),l=u.length;if(c!=l&&!i)return !1;for(var d=c;d--;){var f=s[d];if(!(i?f in t:Zj.call(t,f)))return !1}var g=a.get(e),v=a.get(t);if(g&&v)return g==t&&v==e;var h=!0;a.set(e,t),a.set(t,e);for(var b=i;++d<c;){f=s[d];var m=e[f],_=t[f];if(n)var A=i?n(_,m,f,t,e,a):n(m,_,f,e,t,a);if(!(A===void 0?m===_||o(m,_,r,n,a):A)){h=!1;break}b||(b=f=="constructor");}if(h&&!b){var w=e.constructor,S=t.constructor;w!=S&&"constructor"in e&&"constructor"in t&&!(typeof w=="function"&&w instanceof w&&typeof S=="function"&&S instanceof S)&&(h=!1);}return a.delete(e),a.delete(t),h}var Yj=Kj,Jj=at,Xj=Pe,eC=Jj(Xj,"DataView"),tC=eC,rC=at,nC=Pe,oC=rC(nC,"Promise"),aC=oC,iC=at,sC=Pe,cC=iC(sC,"Set"),tl=cC,uC=at,lC=Pe,dC=uC(lC,"WeakMap"),pC=dC,Ga=tC,za=xa,Wa=aC,Ha=tl,Za=pC,rl=Ir,zt=Eu,nl="[object Map]",fC="[object Object]",ol="[object Promise]",al="[object Set]",il="[object WeakMap]",sl="[object DataView]",vC=zt(Ga),gC=zt(za),hC=zt(Wa),mC=zt(Ha),bC=zt(Za),it=rl;(Ga&&it(new Ga(new ArrayBuffer(1)))!=sl||za&&it(new za)!=nl||Wa&&it(Wa.resolve())!=ol||Ha&&it(new Ha)!=al||Za&&it(new Za)!=il)&&(it=function(e){var t=rl(e),r=t==fC?e.constructor:void 0,n=r?zt(r):"";if(n)switch(n){case vC:return sl;case gC:return nl;case hC:return ol;case mC:return al;case bC:return il}return t});var yC=it,Ka=Mu,_C=Vu,$C=JS,wC=Yj,cl=yC,ul=he,ll=Zu,AC=Xu,OC=1,dl="[object Arguments]",pl="[object Array]",Bn="[object Object]",SC=Object.prototype,fl=SC.hasOwnProperty;function jC(e,t,r,n,o,a){var i=ul(e),s=ul(t),c=i?pl:cl(e),u=s?pl:cl(t);c=c==dl?Bn:c,u=u==dl?Bn:u;var l=c==Bn,d=u==Bn,f=c==u;if(f&&ll(e)){if(!ll(t))return !1;i=!0,l=!1;}if(f&&!l)return a||(a=new Ka),i||AC(e)?_C(e,t,r,n,o,a):$C(e,t,c,r,n,o,a);if(!(r&OC)){var g=l&&fl.call(e,"__wrapped__"),v=d&&fl.call(t,"__wrapped__");if(g||v){var h=g?e.value():e,b=v?t.value():t;return a||(a=new Ka),o(h,b,r,n,a)}}return f?(a||(a=new Ka),wC(e,t,r,n,o,a)):!1}var CC=jC,PC=CC,vl=Ft;function gl(e,t,r,n,o){return e===t?!0:e==null||t==null||!vl(e)&&!vl(t)?e!==e&&t!==t:PC(e,t,r,n,gl,o)}var Ya=gl,EC=Mu,IC=Ya,TC=1,qC=2;function xC(e,t,r,n){var o=r.length,a=o,i=!n;if(e==null)return !a;for(e=Object(e);o--;){var s=r[o];if(i&&s[2]?s[1]!==e[s[0]]:!(s[0]in e))return !1}for(;++o<a;){s=r[o];var c=s[0],u=e[c],l=s[1];if(i&&s[2]){if(u===void 0&&!(c in e))return !1}else {var d=new EC;if(n)var f=n(u,l,c,e,t,d);if(!(f===void 0?IC(l,u,TC|qC,n,d):f))return !1}}return !0}var kC=xC,DC=Ta;function LC(e){return e===e&&!DC(e)}var hl=LC,NC=hl,RC=Ba;function MC(e){for(var t=RC(e),r=t.length;r--;){var n=t[r],o=e[n];t[r]=[n,o,NC(o)];}return t}var VC=MC;function FC(e,t){return function(r){return r==null?!1:r[e]===t&&(t!==void 0||e in Object(r))}}var ml=FC,QC=kC,UC=VC,BC=ml;function GC(e){var t=UC(e);return t.length==1&&t[0][2]?BC(t[0][0],t[0][1]):function(r){return r===e||QC(r,e,t)}}var zC=GC,WC=La;function HC(e,t,r){var n=e==null?void 0:WC(e,t);return n===void 0?r:n}var ZC=HC;function KC(e,t){return e!=null&&t in Object(e)}var YC=KC,JC=Ru,XC=Fa,eP=he,tP=Ku,rP=Qa,nP=Mn;function oP(e,t,r){t=JC(t,e);for(var n=-1,o=t.length,a=!1;++n<o;){var i=nP(t[n]);if(!(a=e!=null&&r(e,i)))break;e=e[i];}return a||++n!=o?a:(o=e==null?0:e.length,!!o&&rP(o)&&tP(i,o)&&(eP(e)||XC(e)))}var aP=oP,iP=YC,sP=aP;function cP(e,t){return e!=null&&sP(e,t,iP)}var uP=cP,lP=Ya,dP=ZC,pP=uP,fP=Ia,vP=hl,gP=ml,hP=Mn,mP=1,bP=2;function yP(e,t){return fP(e)&&vP(t)?gP(hP(e),t):function(r){var n=dP(r,e);return n===void 0&&n===t?pP(r,e):lP(t,n,mP|bP)}}var _P=yP;function $P(e){return e}var Gn=$P;function wP(e){return function(t){return t==null?void 0:t[e]}}var AP=wP,OP=La;function SP(e){return function(t){return OP(t,e)}}var jP=SP,CP=AP,PP=jP,EP=Ia,IP=Mn;function TP(e){return EP(e)?CP(IP(e)):PP(e)}var qP=TP,xP=zC,kP=_P,DP=Gn,LP=he,NP=qP;function RP(e){return typeof e=="function"?e:e==null?DP:typeof e=="object"?LP(e)?kP(e[0],e[1]):xP(e):NP(e)}var bl=RP;function MP(e){return function(t,r,n){for(var o=-1,a=Object(t),i=n(t),s=i.length;s--;){var c=i[e?s:++o];if(r(a[c],c,a)===!1)break}return t}}var VP=MP,FP=VP,QP=FP(),UP=QP,BP=UP,GP=Ba;function zP(e,t){return e&&BP(e,t,GP)}var WP=zP,HP=Un;function ZP(e,t){return function(r,n){if(r==null)return r;if(!HP(r))return e(r,n);for(var o=r.length,a=t?o:-1,i=Object(r);(t?a--:++a<o)&&n(i[a],a,i)!==!1;);return r}}var KP=ZP,YP=WP,JP=KP,XP=JP(YP),yl=XP,eE=yl,tE=Un;function rE(e,t){var r=-1,n=tE(e)?Array(e.length):[];return eE(e,function(o,a,i){n[++r]=t(o,a,i);}),n}var nE=rE;function oE(e,t){var r=e.length;for(e.sort(t);r--;)e[r]=e[r].value;return e}var aE=oE,_l=kn;function iE(e,t){if(e!==t){var r=e!==void 0,n=e===null,o=e===e,a=_l(e),i=t!==void 0,s=t===null,c=t===t,u=_l(t);if(!s&&!u&&!a&&e>t||a&&i&&c&&!s&&!u||n&&i&&c||!r&&c||!o)return 1;if(!n&&!a&&!u&&e<t||u&&r&&o&&!n&&!a||s&&r&&o||!i&&o||!c)return -1}return 0}var sE=iE,cE=sE;function uE(e,t,r){for(var n=-1,o=e.criteria,a=t.criteria,i=o.length,s=r.length;++n<i;){var c=cE(o[n],a[n]);if(c){if(n>=s)return c;var u=r[n];return c*(u=="desc"?-1:1)}}return e.index-t.index}var lE=uE,Ja=Ea,dE=La,pE=bl,fE=nE,vE=aE,gE=Ua,hE=lE,mE=Gn,bE=he;function yE(e,t,r){t.length?t=Ja(t,function(a){return bE(a)?function(i){return dE(i,a.length===1?a[0]:a)}:a}):t=[mE];var n=-1;t=Ja(t,gE(pE));var o=fE(e,function(a,i,s){var c=Ja(t,function(u){return u(a)});return {criteria:c,index:++n,value:a}});return vE(o,function(a,i){return hE(a,i,r)})}var _E=yE,$E=_E,$l=he;function wE(e,t,r,n){return e==null?[]:($l(t)||(t=t==null?[]:[t]),r=n?void 0:r,$l(r)||(r=r==null?[]:[r]),$E(e,t,r))}var AE=wE,OE=Cr(AE),wl=typeof global=="object"&&global&&global.Object===Object&&global,SE=typeof self=="object"&&self&&self.Object===Object&&self,Ee=wl||SE||Function("return this")(),Be=Ee.Symbol,Al=Object.prototype,jE=Al.hasOwnProperty,CE=Al.toString,Tr=Be?Be.toStringTag:void 0;function PE(e){var t=jE.call(e,Tr),r=e[Tr];try{e[Tr]=void 0;var n=!0;}catch(a){}var o=CE.call(e);return n&&(t?e[Tr]=r:delete e[Tr]),o}var EE=Object.prototype,IE=EE.toString;function TE(e){return IE.call(e)}var qE="[object Null]",xE="[object Undefined]",Ol=Be?Be.toStringTag:void 0;function qr(e){return e==null?e===void 0?xE:qE:Ol&&Ol in Object(e)?PE(e):TE(e)}function Wt(e){return e!=null&&typeof e=="object"}function kE(e,t){for(var r=-1,n=e==null?0:e.length,o=Array(n);++r<n;)o[r]=t(e[r],r,e);return o}var xr=Array.isArray;function Sl(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}function jl(e){return e}var DE="[object AsyncFunction]",LE="[object Function]",NE="[object GeneratorFunction]",RE="[object Proxy]";function Cl(e){if(!Sl(e))return !1;var t=qr(e);return t==LE||t==NE||t==DE||t==RE}var Xa=Ee["__core-js_shared__"],Pl=function(){var e=/[^.]+$/.exec(Xa&&Xa.keys&&Xa.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function ME(e){return !!Pl&&Pl in e}var VE=Function.prototype,FE=VE.toString;function st(e){if(e!=null){try{return FE.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var QE=/[\\^$.*+?()[\]{}|]/g,UE=/^\[object .+?Constructor\]$/,BE=Function.prototype,GE=Object.prototype,zE=BE.toString,WE=GE.hasOwnProperty,HE=RegExp("^"+zE.call(WE).replace(QE,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function ZE(e){if(!Sl(e)||ME(e))return !1;var t=Cl(e)?HE:UE;return t.test(st(e))}function KE(e,t){return e==null?void 0:e[t]}function ct(e,t){var r=KE(e,t);return ZE(r)?r:void 0}var ei=ct(Ee,"WeakMap");function YE(e,t,r){switch(r.length){case 0:return e.call(t);case 1:return e.call(t,r[0]);case 2:return e.call(t,r[0],r[1]);case 3:return e.call(t,r[0],r[1],r[2])}return e.apply(t,r)}function JE(){}var XE=800,eI=16,tI=Date.now;function rI(e){var t=0,r=0;return function(){var n=tI(),o=eI-(n-r);if(r=n,o>0){if(++t>=XE)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}function nI(e){return function(){return e}}var El=function(){try{var e=ct(Object,"defineProperty");return e({},"",{}),e}catch(t){}}(),oI=El?function(e,t){return El(e,"toString",{configurable:!0,enumerable:!1,value:nI(t),writable:!0})}:jl,aI=rI(oI);function iI(e,t,r,n){for(var o=e.length,a=r+(n?1:-1);n?a--:++a<o;)if(t(e[a],a,e))return a;return -1}function sI(e){return e!==e}function cI(e,t,r){for(var n=r-1,o=e.length;++n<o;)if(e[n]===t)return n;return -1}function uI(e,t,r){return t===t?cI(e,t,r):iI(e,sI,r)}function Il(e,t){var r=e==null?0:e.length;return !!r&&uI(e,t,0)>-1}var lI=9007199254740991,dI=/^(?:0|[1-9]\d*)$/;function pI(e,t){var r=typeof e;return t=t==null?lI:t,!!t&&(r=="number"||r!="symbol"&&dI.test(e))&&e>-1&&e%1==0&&e<t}function Tl(e,t){return e===t||e!==e&&t!==t}var ql=Math.max;function fI(e,t,r){return t=ql(t===void 0?e.length-1:t,0),function(){for(var n=arguments,o=-1,a=ql(n.length-t,0),i=Array(a);++o<a;)i[o]=n[t+o];o=-1;for(var s=Array(t+1);++o<t;)s[o]=n[o];return s[t]=r(i),YE(e,this,s)}}function vI(e,t){return aI(fI(e,t,jl),e+"")}var gI=9007199254740991;function xl(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=gI}function kl(e){return e!=null&&xl(e.length)&&!Cl(e)}var hI=Object.prototype;function mI(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||hI;return e===r}function bI(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var yI="[object Arguments]";function Dl(e){return Wt(e)&&qr(e)==yI}var Ll=Object.prototype,_I=Ll.hasOwnProperty,$I=Ll.propertyIsEnumerable,Nl=Dl(function(){return arguments}())?Dl:function(e){return Wt(e)&&_I.call(e,"callee")&&!$I.call(e,"callee")};function wI(){return !1}var Rl=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Ml=Rl&&typeof module=="object"&&module&&!module.nodeType&&module,AI=Ml&&Ml.exports===Rl,Vl=AI?Ee.Buffer:void 0,OI=Vl?Vl.isBuffer:void 0,ti=OI||wI,SI="[object Arguments]",jI="[object Array]",CI="[object Boolean]",PI="[object Date]",EI="[object Error]",II="[object Function]",TI="[object Map]",qI="[object Number]",xI="[object Object]",kI="[object RegExp]",DI="[object Set]",LI="[object String]",NI="[object WeakMap]",RI="[object ArrayBuffer]",MI="[object DataView]",VI="[object Float32Array]",FI="[object Float64Array]",QI="[object Int8Array]",UI="[object Int16Array]",BI="[object Int32Array]",GI="[object Uint8Array]",zI="[object Uint8ClampedArray]",WI="[object Uint16Array]",HI="[object Uint32Array]",U={};U[VI]=U[FI]=U[QI]=U[UI]=U[BI]=U[GI]=U[zI]=U[WI]=U[HI]=!0,U[SI]=U[jI]=U[RI]=U[CI]=U[MI]=U[PI]=U[EI]=U[II]=U[TI]=U[qI]=U[xI]=U[kI]=U[DI]=U[LI]=U[NI]=!1;function ZI(e){return Wt(e)&&xl(e.length)&&!!U[qr(e)]}function Fl(e){return function(t){return e(t)}}var Ql=typeof exports=="object"&&exports&&!exports.nodeType&&exports,kr=Ql&&typeof module=="object"&&module&&!module.nodeType&&module,KI=kr&&kr.exports===Ql,ri=KI&&wl.process,Ul=function(){try{var e=kr&&kr.require&&kr.require("util").types;return e||ri&&ri.binding&&ri.binding("util")}catch(t){}}(),Bl=Ul&&Ul.isTypedArray,Gl=Bl?Fl(Bl):ZI,YI=Object.prototype,JI=YI.hasOwnProperty;function XI(e,t){var r=xr(e),n=!r&&Nl(e),o=!r&&!n&&ti(e),a=!r&&!n&&!o&&Gl(e),i=r||n||o||a,s=i?bI(e.length,String):[],c=s.length;for(var u in e)(t||JI.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||pI(u,c)))&&s.push(u);return s}function eT(e,t){return function(r){return e(t(r))}}var tT=eT(Object.keys,Object),rT=Object.prototype,nT=rT.hasOwnProperty;function oT(e){if(!mI(e))return tT(e);var t=[];for(var r in Object(e))nT.call(e,r)&&r!="constructor"&&t.push(r);return t}function aT(e){return kl(e)?XI(e):oT(e)}var Dr=ct(Object,"create");function iT(){this.__data__=Dr?Dr(null):{},this.size=0;}function sT(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var cT="__lodash_hash_undefined__",uT=Object.prototype,lT=uT.hasOwnProperty;function dT(e){var t=this.__data__;if(Dr){var r=t[e];return r===cT?void 0:r}return lT.call(t,e)?t[e]:void 0}var pT=Object.prototype,fT=pT.hasOwnProperty;function vT(e){var t=this.__data__;return Dr?t[e]!==void 0:fT.call(t,e)}var gT="__lodash_hash_undefined__";function hT(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=Dr&&t===void 0?gT:t,this}function ut(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}ut.prototype.clear=iT,ut.prototype.delete=sT,ut.prototype.get=dT,ut.prototype.has=vT,ut.prototype.set=hT;function mT(){this.__data__=[],this.size=0;}function zn(e,t){for(var r=e.length;r--;)if(Tl(e[r][0],t))return r;return -1}var bT=Array.prototype,yT=bT.splice;function _T(e){var t=this.__data__,r=zn(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():yT.call(t,r,1),--this.size,!0}function $T(e){var t=this.__data__,r=zn(t,e);return r<0?void 0:t[r][1]}function wT(e){return zn(this.__data__,e)>-1}function AT(e,t){var r=this.__data__,n=zn(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}function Ie(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}Ie.prototype.clear=mT,Ie.prototype.delete=_T,Ie.prototype.get=$T,Ie.prototype.has=wT,Ie.prototype.set=AT;var Lr=ct(Ee,"Map");function OT(){this.size=0,this.__data__={hash:new ut,map:new(Lr||Ie),string:new ut};}function ST(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}function Wn(e,t){var r=e.__data__;return ST(t)?r[typeof t=="string"?"string":"hash"]:r.map}function jT(e){var t=Wn(this,e).delete(e);return this.size-=t?1:0,t}function CT(e){return Wn(this,e).get(e)}function PT(e){return Wn(this,e).has(e)}function ET(e,t){var r=Wn(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}function lt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}lt.prototype.clear=OT,lt.prototype.delete=jT,lt.prototype.get=CT,lt.prototype.has=PT,lt.prototype.set=ET;function zl(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var Wl=Be?Be.isConcatSpreadable:void 0;function IT(e){return xr(e)||Nl(e)||!!(Wl&&e&&e[Wl])}function Hl(e,t,r,n,o){var a=-1,i=e.length;for(r||(r=IT),o||(o=[]);++a<i;){var s=e[a];t>0&&r(s)?t>1?Hl(s,t-1,r,n,o):zl(o,s):n||(o[o.length]=s);}return o}function TT(){this.__data__=new Ie,this.size=0;}function qT(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}function xT(e){return this.__data__.get(e)}function kT(e){return this.__data__.has(e)}var DT=200;function LT(e,t){var r=this.__data__;if(r instanceof Ie){var n=r.__data__;if(!Lr||n.length<DT-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new lt(n);}return r.set(e,t),this.size=r.size,this}function Ge(e){var t=this.__data__=new Ie(e);this.size=t.size;}Ge.prototype.clear=TT,Ge.prototype.delete=qT,Ge.prototype.get=xT,Ge.prototype.has=kT,Ge.prototype.set=LT;function Zl(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}function NT(){return []}var RT=Object.prototype,MT=RT.propertyIsEnumerable,Kl=Object.getOwnPropertySymbols,VT=Kl?function(e){return e==null?[]:(e=Object(e),Zl(Kl(e),function(t){return MT.call(e,t)}))}:NT;function FT(e,t,r){var n=t(e);return xr(e)?n:zl(n,r(e))}function Yl(e){return FT(e,aT,VT)}var ni=ct(Ee,"DataView"),oi=ct(Ee,"Promise"),Ht=ct(Ee,"Set"),Jl="[object Map]",QT="[object Object]",Xl="[object Promise]",ed="[object Set]",td="[object WeakMap]",rd="[object DataView]",UT=st(ni),BT=st(Lr),GT=st(oi),zT=st(Ht),WT=st(ei),dt=qr;(ni&&dt(new ni(new ArrayBuffer(1)))!=rd||Lr&&dt(new Lr)!=Jl||oi&&dt(oi.resolve())!=Xl||Ht&&dt(new Ht)!=ed||ei&&dt(new ei)!=td)&&(dt=function(e){var t=qr(e),r=t==QT?e.constructor:void 0,n=r?st(r):"";if(n)switch(n){case UT:return rd;case BT:return Jl;case GT:return Xl;case zT:return ed;case WT:return td}return t});var nd=dt,od=Ee.Uint8Array,HT="__lodash_hash_undefined__";function ZT(e){return this.__data__.set(e,HT),this}function KT(e){return this.__data__.has(e)}function Zt(e){var t=-1,r=e==null?0:e.length;for(this.__data__=new lt;++t<r;)this.add(e[t]);}Zt.prototype.add=Zt.prototype.push=ZT,Zt.prototype.has=KT;function YT(e,t){for(var r=-1,n=e==null?0:e.length;++r<n;)if(t(e[r],r,e))return !0;return !1}function ai(e,t){return e.has(t)}var JT=1,XT=2;function ad(e,t,r,n,o,a){var i=r&JT,s=e.length,c=t.length;if(s!=c&&!(i&&c>s))return !1;var u=a.get(e),l=a.get(t);if(u&&l)return u==t&&l==e;var d=-1,f=!0,g=r&XT?new Zt:void 0;for(a.set(e,t),a.set(t,e);++d<s;){var v=e[d],h=t[d];if(n)var b=i?n(h,v,d,t,e,a):n(v,h,d,e,t,a);if(b!==void 0){if(b)continue;f=!1;break}if(g){if(!YT(t,function(m,_){if(!ai(g,_)&&(v===m||o(v,m,r,n,a)))return g.push(_)})){f=!1;break}}else if(!(v===h||o(v,h,r,n,a))){f=!1;break}}return a.delete(e),a.delete(t),f}function eq(e){var t=-1,r=Array(e.size);return e.forEach(function(n,o){r[++t]=[o,n];}),r}function ii(e){var t=-1,r=Array(e.size);return e.forEach(function(n){r[++t]=n;}),r}var tq=1,rq=2,nq="[object Boolean]",oq="[object Date]",aq="[object Error]",iq="[object Map]",sq="[object Number]",cq="[object RegExp]",uq="[object Set]",lq="[object String]",dq="[object Symbol]",pq="[object ArrayBuffer]",fq="[object DataView]",id=Be?Be.prototype:void 0,si=id?id.valueOf:void 0;function vq(e,t,r,n,o,a,i){switch(r){case fq:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return !1;e=e.buffer,t=t.buffer;case pq:return !(e.byteLength!=t.byteLength||!a(new od(e),new od(t)));case nq:case oq:case sq:return Tl(+e,+t);case aq:return e.name==t.name&&e.message==t.message;case cq:case lq:return e==t+"";case iq:var s=eq;case uq:var c=n&tq;if(s||(s=ii),e.size!=t.size&&!c)return !1;var u=i.get(e);if(u)return u==t;n|=rq,i.set(e,t);var l=ad(s(e),s(t),n,o,a,i);return i.delete(e),l;case dq:if(si)return si.call(e)==si.call(t)}return !1}var gq=1,hq=Object.prototype,mq=hq.hasOwnProperty;function bq(e,t,r,n,o,a){var i=r&gq,s=Yl(e),c=s.length,u=Yl(t),l=u.length;if(c!=l&&!i)return !1;for(var d=c;d--;){var f=s[d];if(!(i?f in t:mq.call(t,f)))return !1}var g=a.get(e),v=a.get(t);if(g&&v)return g==t&&v==e;var h=!0;a.set(e,t),a.set(t,e);for(var b=i;++d<c;){f=s[d];var m=e[f],_=t[f];if(n)var A=i?n(_,m,f,t,e,a):n(m,_,f,e,t,a);if(!(A===void 0?m===_||o(m,_,r,n,a):A)){h=!1;break}b||(b=f=="constructor");}if(h&&!b){var w=e.constructor,S=t.constructor;w!=S&&"constructor"in e&&"constructor"in t&&!(typeof w=="function"&&w instanceof w&&typeof S=="function"&&S instanceof S)&&(h=!1);}return a.delete(e),a.delete(t),h}var yq=1,sd="[object Arguments]",cd="[object Array]",Hn="[object Object]",_q=Object.prototype,ud=_q.hasOwnProperty;function $q(e,t,r,n,o,a){var i=xr(e),s=xr(t),c=i?cd:nd(e),u=s?cd:nd(t);c=c==sd?Hn:c,u=u==sd?Hn:u;var l=c==Hn,d=u==Hn,f=c==u;if(f&&ti(e)){if(!ti(t))return !1;i=!0,l=!1;}if(f&&!l)return a||(a=new Ge),i||Gl(e)?ad(e,t,r,n,o,a):vq(e,t,c,r,n,o,a);if(!(r&yq)){var g=l&&ud.call(e,"__wrapped__"),v=d&&ud.call(t,"__wrapped__");if(g||v){var h=g?e.value():e,b=v?t.value():t;return a||(a=new Ge),o(h,b,r,n,a)}}return f?(a||(a=new Ge),bq(e,t,r,n,o,a)):!1}function ld(e,t,r,n,o){return e===t?!0:e==null||t==null||!Wt(e)&&!Wt(t)?e!==e&&t!==t:$q(e,t,r,n,ld,o)}function wq(e){return Wt(e)&&kl(e)}function dd(e,t,r){for(var n=-1,o=e==null?0:e.length;++n<o;)if(r(t,e[n]))return !0;return !1}var Aq=200;function Oq(e,t,r,n){var o=-1,a=Il,i=!0,s=e.length,c=[],u=t.length;if(!s)return c;r&&(t=kE(t,Fl(r))),n?(a=dd,i=!1):t.length>=Aq&&(a=ai,i=!1,t=new Zt(t));e:for(;++o<s;){var l=e[o],d=r==null?l:r(l);if(l=n||l!==0?l:0,i&&d===d){for(var f=u;f--;)if(t[f]===d)continue e;c.push(l);}else a(t,d,n)||c.push(l);}return c}function Sq(e){var t=e==null?0:e.length;return t?e[t-1]:void 0}function pd(e,t){return ld(e,t)}var jq=1/0,Cq=Ht&&1/ii(new Ht([,-0]))[1]==jq?function(e){return new Ht(e)}:JE,Pq=200;function fd(e,t,r){var n=-1,o=Il,a=e.length,i=!0,s=[],c=s;if(r)i=!1,o=dd;else if(a>=Pq){var u=t?null:Cq(e);if(u)return ii(u);i=!1,o=ai,c=new Zt;}else c=t?[]:s;e:for(;++n<a;){var l=e[n],d=t?t(l):l;if(l=r||l!==0?l:0,i&&d===d){for(var f=c.length;f--;)if(c[f]===d)continue e;t&&c.push(d),s.push(l);}else o(c,d,r)||(c!==s&&c.push(d),s.push(l));}return s}function Eq(e,t,r){var n=e.length;if(n<2)return n?fd(e[0]):[];for(var o=-1,a=Array(n);++o<n;)for(var i=e[o],s=-1;++s<n;)s!=o&&(a[o]=Oq(a[o]||i,e[s],t,r));return fd(Hl(a,1),t,r)}var Iq=vI(function(e){var t=Sq(e);return t=typeof t=="function"?t:void 0,Eq(Zl(e,wq),void 0,t)}),Nr={},Zn={};Object.defineProperty(Zn,"__esModule",{value:!0}),Zn.CA_DL=void 0,Zn.CA_DL={AB:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],BC:[{regex:/^[0-9]{7}$/,description:"7 numbers"}],MB:[{regex:/^[A-Z0-9]{12}$/,description:"any combination of letters or numbers for a total of 12 characters"}],NB:[{regex:/^[0-9]{5,7}$/,description:"5-7 numbers"}],NL:[{regex:/^[A-Z]{1}[0-9]{9}$/,description:"1 letter followed by 9 numbers"}],NT:[{regex:/^[0-9]{6}$/,description:"6 numbers"}],NS:[{regex:/^[A-Z]{5}[0-9]{9}$/,description:"5 letters followed by 9 numbers"}],NU:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],ON:[{regex:/^[A-Z]{1}[0-9]{14}$/,description:"1 letter followed by 14 numbers"}],PE:[{regex:/^[0-9]{5,6}$/,description:"5-6 numbers"}],QC:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],SK:[{regex:/^[0-9]{8}$/,description:"8 numbers"}],YT:[{regex:/^[0-9]{10}$/,description:"10 numbers"}]};var Kn={};Object.defineProperty(Kn,"__esModule",{value:!0}),Kn.US_DL=void 0,Kn.US_DL={AL:[{regex:/^[0-9]{1,8}$/,description:"1-8 numbers"}],AK:[{regex:/^[0-9]{1,7}$/,description:"1-7 numbers"}],AZ:[{regex:/^[A-Z]{1}[0-9]{8,9}$/,description:"1 letter followed by 8-9 numbers"},{regex:/^[A-Z]{2}[0-9]{2,5}$/,description:"2 letters followed by 2-5 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],AR:[{regex:/^[0-9]{4,9}$/,description:"4-9 numbers"}],CA:[{regex:/^[A-Z]{1}[0-9]{7}$/,description:"1 letter followed by 7 numbers"}],CO:[{regex:/^[0-9]{9}$/,description:"9 numbers"},{regex:/^[A-Z]{1}[0-9]{3,6}$/,description:"1 letter followed by 3-6 numbers"},{regex:/^[A-Z]{2}[0-9]{2,5}$/,description:"2 letters followed by 2-5 numbers"}],CT:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],DE:[{regex:/^[0-9]{1,7}$/,description:"1-7 numbers"}],DC:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],FL:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],GA:[{regex:/^[0-9]{7,9}$/,description:"7-9 numbers"}],HI:[{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],ID:[{regex:/^[A-Z]{2}[0-9]{6}[A-Z]{1}$/,description:"2 letters followed by 6 numbers followed by 1 letter"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],IL:[{regex:/^[A-Z]{1}[0-9]{11,12}$/,description:"1 letter followed by 11-12 numbers"}],IN:[{regex:/^[A-Z]{1}[0-9]{9}$/,description:"1 letter followed by 9 numbers"},{regex:/^[0-9]{9,10}$/,description:"9-10 numbers"}],IA:[{regex:/^[0-9]{9}$/,description:"9 numbers"},{regex:/^[0-9]{3}[A-Z]{2}[0-9]{4}$/,description:"3 numbers followed by 2 letters followed by 4 numbers"}],KS:[{regex:/^([A-Z]{1}[0-9]{1}){2}[A-Z]{1}$/,description:"1 letter then 1 number then 1 letter then 1 number then 1 letter"},{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],KY:[{regex:/^[A-Z]{1}[0-9]{8,9}$/,description:"1 letter followed by 8-9 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],LA:[{regex:/^[0-9]{1,9}$/,description:"1-9 numbers"}],ME:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[0-9]{7}[A-Z]{1}$/,description:"7 numbers followed by 1 letter"},{regex:/^[0-9]{8}$/,description:"8 numbers"}],MD:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],MA:[{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],MI:[{regex:/^[A-Z]{1}[0-9]{10}$/,description:"1 letter followed by 10 numbers"},{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],MN:[{regex:/^[A-Z]{1}[0-9]{12}$/,description:"1 letter followed by 12 numbers"}],MS:[{regex:/^[0-9]{9}$/,description:"9 numbers"}],MO:[{regex:/^[A-Z]{1}[0-9]{5,9}$/,description:"1 letter followed by 5-9 numbers"},{regex:/^[A-Z]{1}[0-9]{6}[R]{1}$/,description:'1 letter followed by 6 numbers followed by "R"'},{regex:/^[0-9]{8}[A-Z]{2}$/,description:"8 numbers followed by 2 letters"},{regex:/^[0-9]{9}[A-Z]{1}$/,description:"9 numbers followed by 1 letter"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],MT:[{regex:/^[A-Z]{1}[0-9]{8}$/,description:"1 letter followed by 8 numbers"},{regex:/^[0-9]{13}$/,description:"13 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"},{regex:/^[0-9]{14}$/,description:"14 numbers"}],NE:[{regex:/^[A-Z]{1}[0-9]{6,8}$/,description:"1 letter followed by 6-8 numbers"}],NV:[{regex:/^[0-9]{9,10}$/,description:"9-10 numbers"},{regex:/^[0-9]{12}$/,description:"12 numbers"},{regex:/^[X]{1}[0-9]{8}$/,description:'"X" followed by 8 numbers'}],NH:[{regex:/^[0-9]{2}[A-Z]{3}[0-9]{5}$/,description:"2 numbers followed by 3 letters followed by 5 numbers"}],NJ:[{regex:/^[A-Z]{1}[0-9]{14}$/,description:"1 letter followed by 14 numbers"}],NM:[{regex:/^[0-9]{8,9}$/,description:"8-9 numbers"}],NY:[{regex:/^[A-Z]{1}[0-9]{7}$/,description:"1 letter followed by 7 numbers"},{regex:/^[A-Z]{1}[0-9]{18}$/,description:"1 letter followed by 18 numbers"},{regex:/^[0-9]{8,9}$/,description:"8-9 numbers"},{regex:/^[0-9]{16}$/,description:"16 numbers"},{regex:/^[A-Z]{8}$/,description:"8 letters"}],NC:[{regex:/^[0-9]{1,12}$/,description:"1-12 numbers"}],ND:[{regex:/^[A-Z]{3}[0-9]{6}$/,description:"3 letters followed by 6 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],OH:[{regex:/^[A-Z]{1}[0-9]{4,8}$/,description:"1 letter followed by 4-8 numbers"},{regex:/^[A-Z]{2}[0-9]{3,7}$/,description:"2 letters followed by 3-7 numbers"},{regex:/^[0-9]{8}$/,description:"8 numbers"}],OK:[{regex:/^[A-Z]{1}[0-9]{9}$/,description:"1 letter followed by 9 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],OR:[{regex:/^[0-9]{1,9}$/,description:"1-9 numbers"}],PA:[{regex:/^[0-9]{8}$/,description:"8 numbers"}],RI:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[A-Z]{1}[0-9]{6}$/,description:"1 letter followed by 6 numbers"}],SC:[{regex:/^[0-9]{5,11}$/,description:"5-11 numbers"}],SD:[{regex:/^[0-9]{6,10}$/,description:"6-10 numbers"},{regex:/^[0-9]{12}$/,description:"12 numbers"}],TN:[{regex:/^[0-9]{7,9}$/,description:"7-9 numbers"}],TX:[{regex:/^[0-9]{7,8}$/,description:"7-8 numbers"}],UT:[{regex:/^[0-9]{4,10}$/,description:"4-10 numbers"}],VT:[{regex:/^[0-9]{8}$/,description:"8 numbers"},{regex:/^[0-9]{7}[A]$/,description:'7 numbers followed by "A"'}],VA:[{regex:/^[A-Z]{1}[0-9]{8,11}$/,description:"1 letter followed by 8-11 numbers"},{regex:/^[0-9]{9}$/,description:"9 numbers"}],WA:[{regex:/^(?=.{12}$)[A-Z]{1,7}[A-Z0-9\\*]{4,11}$/,description:'1-7 letters followed by any combination of letters, numbers, or "*" for a total of 12 characters'}],WV:[{regex:/^[0-9]{7}$/,description:"7 numbers"},{regex:/^[A-Z]{1,2}[0-9]{5,6}$/,description:"1-2 letters followed by 5-6 numbers"}],WI:[{regex:/^[A-Z]{1}[0-9]{13}$/,description:"1 letter followed by 13 numbers"}],WY:[{regex:/^[0-9]{9,10}$/,description:"9-10 numbers"}]},Object.defineProperty(Nr,"__esModule",{value:!0}),Nr.getMatches=Nr.isValid=void 0;var Tq=Zn,qq=Kn;function xq(e,t){t===void 0&&(t={});var r=vd(e,t);return (r==null?void 0:r.length)>0}Nr.isValid=xq;function vd(e,t){t===void 0&&(t={});var r=[],n,o;switch(t.country){case"CA":n=Tq.CA_DL;break;case"US":default:n=qq.US_DL;break}return t.states?Array.isArray(t.states)||(o=[t.states]):o=Object.keys(n),o.forEach(function(a){var i=n[a];if(!i)throw new Error('Could not find state "'+a+'"!');i.forEach(function(s){var c=s.regex,u=s.description,l=t.ignoreCase?new RegExp(c,"i"):c;l.test(e)&&r.push({description:u,state:a});});}),r.length?r:null}Nr.getMatches=vd;const kq=["email","number","url","zipcode","year"],Dq=["ssn","fein","phone","currency","percentage"],Lq=["radio","dropdown"],ci=e=>{let t=e;if(kq.includes(e))t="text";else if(Dq.includes(e))t="masked";else if(Lq.includes(e))return "dropdown";return t};function ui(e){return ("0"+e.toString(16)).substr(-2)}function gd(e){const t=new Uint8Array((e-1)/2),r=window.crypto||window.msCrypto;if(!r)return "";r.getRandomValues(t);const n=ui(Math.floor(Math.random()*6)+10).substr(-1);let o="";return e%2===0&&(o=ui(Math.floor(Math.random()*6)+10).substr(-1)),n+Array.from(t,ui).join("")+o}function Nq(e,t){let r=!0;switch(typeof e){case"string":r=!0;break;case"object":Array.isArray(e)&&Array.isArray(t)?r=Iq(e,t,pd).length>0:!Array.isArray(e)&&!Array.isArray(t)&&(r=!pd(e,t));break;case"number":r=!0;break}return r}function Kt(e,t){let r=!0;return e===t?r=!1:e&&!t||!e&&t?r=!0:typeof e==typeof t&&(r=Nq(e,t)),r}function Rq(e,t,r=!1){const n=r?-1:1;let o=0;return t>e?o=1:t<e&&(o=-1),o*n}function Mq(e){const t=e.question,r=e.choice_id||e.choice_ids;if(r.length>0){const n=t.choices.reduce((o,a)=>(o[a.id]=a,o),{});return Array.isArray(r)?r.map(o=>n[o].value):n[r].value}return e.value}function Vq(e){var t;const r=e==null?void 0:e.question;if(!(r!=null&&r.field_type))return !!(e!=null&&e.value);switch(ci(r.field_type)){case O.DropDown:return r.multiple?!!((t=e.choice_ids)!=null&&t.length):!!e.choice_id;case O.Masked:case O.Text:default:return e.value!==null}}function Fq(e,t){return t instanceof RegExp?e.filter(r=>t.test(r.question.code)):e.filter(r=>r.question.code.includes(t))}var pe={},li={},Rr={},di={};/*!
* cookie
* Copyright(c) 2012-2014 Roman Shtylman
* Copyright(c) 2015 Douglas Christopher Wilson
* MIT Licensed
*/di.parse=Gq,di.serialize=zq;var Qq=decodeURIComponent,Uq=encodeURIComponent,Bq=/; */,Yn=/^[\u0009\u0020-\u007e\u0080-\u00ff]+$/;function Gq(e,t){if(typeof e!="string")throw new TypeError("argument str must be a string");for(var r={},n=t||{},o=e.split(Bq),a=n.decode||Qq,i=0;i<o.length;i++){var s=o[i],c=s.indexOf("=");if(!(c<0)){var u=s.substr(0,c).trim(),l=s.substr(++c,s.length).trim();l[0]=='"'&&(l=l.slice(1,-1)),r[u]==null&&(r[u]=Wq(l,a));}}return r}function zq(e,t,r){var n=r||{},o=n.encode||Uq;if(typeof o!="function")throw new TypeError("option encode is invalid");if(!Yn.test(e))throw new TypeError("argument name is invalid");var a=o(t);if(a&&!Yn.test(a))throw new TypeError("argument val is invalid");var i=e+"="+a;if(n.maxAge!=null){var s=n.maxAge-0;if(isNaN(s))throw new Error("maxAge should be a Number");i+="; Max-Age="+Math.floor(s);}if(n.domain){if(!Yn.test(n.domain))throw new TypeError("option domain is invalid");i+="; Domain="+n.domain;}if(n.path){if(!Yn.test(n.path))throw new TypeError("option path is invalid");i+="; Path="+n.path;}if(n.expires){if(typeof n.expires.toUTCString!="function")throw new TypeError("option expires is invalid");i+="; Expires="+n.expires.toUTCString();}if(n.httpOnly&&(i+="; HttpOnly"),n.secure&&(i+="; Secure"),n.sameSite){var c=typeof n.sameSite=="string"?n.sameSite.toLowerCase():n.sameSite;switch(c){case!0:i+="; SameSite=Strict";break;case"lax":i+="; SameSite=Lax";break;case"strict":i+="; SameSite=Strict";break;default:throw new TypeError("option sameSite is invalid")}}return i}function Wq(e,t){try{return t(e)}catch(r){return e}}Object.defineProperty(Rr,"__esModule",{value:!0});var Hq=function(){function e(t,r){for(var n=0;n<r.length;n++){var o=r[n];o.enumerable=o.enumerable||!1,o.configurable=!0,"value"in o&&(o.writable=!0),Object.defineProperty(t,o.key,o);}}return function(t,r,n){return r&&e(t.prototype,r),n&&e(t,n),t}}();Rr.hasCookies=Jq;var Zq=di,Jn=Kq(Zq);function Kq(e){return e&&e.__esModule?e:{default:e}}function Yq(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}var ze="lS_",hd=function(){function e(){var t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};Yq(this,e),this.cookieOptions=Object.assign({path:"/"},t),ze=t.prefix===void 0?ze:t.prefix;}return Hq(e,[{key:"getItem",value:function(t){var r=Jn.default.parse(document.cookie);return !r||!r.hasOwnProperty(ze+t)?null:r[ze+t]}},{key:"setItem",value:function(t,r){return document.cookie=Jn.default.serialize(ze+t,r,this.cookieOptions),r}},{key:"removeItem",value:function(t){var r=Object.assign({},this.cookieOptions,{maxAge:-1});return document.cookie=Jn.default.serialize(ze+t,"",r),null}},{key:"clear",value:function(){var t=Jn.default.parse(document.cookie);for(var r in t)r.indexOf(ze)===0&&this.removeItem(r.substr(ze.length));return null}}]),e}();Rr.default=hd;function Jq(){var e=new hd;try{var t="__test";e.setItem(t,"1");var r=e.getItem(t);return e.removeItem(t),r==="1"}catch(n){return !1}}Object.defineProperty(li,"__esModule",{value:!0}),li.default=ex;var Xq=Rr,md="__test";function bd(e){try{var t=window[e];return t.setItem(md,"1"),t.removeItem(md),!0}catch(r){return !1}}function ex(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"localStorage",t=String(e).replace(/storage$/i,"").toLowerCase();if(t==="local")return bd("localStorage");if(t==="session")return bd("sessionStorage");if(t==="cookie")return (0, Xq.hasCookies)();if(t==="memory")return !0;throw new Error("Storage method `"+e+"` is not available.\n    Please use one of the following: localStorage, sessionStorage, cookieStorage, memoryStorage.")}var pi={};Object.defineProperty(pi,"__esModule",{value:!0});var tx=function(){function e(t,r){for(var n=0;n<r.length;n++){var o=r[n];o.enumerable=o.enumerable||!1,o.configurable=!0,"value"in o&&(o.writable=!0),Object.defineProperty(t,o.key,o);}}return function(t,r,n){return r&&e(t.prototype,r),n&&e(t,n),t}}();function rx(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}var nx=function(){function e(){rx(this,e),this._data={};}return tx(e,[{key:"getItem",value:function(t){return this._data.hasOwnProperty(t)?this._data[t]:null}},{key:"setItem",value:function(t,r){return this._data[t]=String(r)}},{key:"removeItem",value:function(t){return delete this._data[t]}},{key:"clear",value:function(){return this._data={}}}]),e}();pi.default=nx,Object.defineProperty(pe,"__esModule",{value:!0}),pe.MemoryStorage=pe.CookieStorage=pe.isSupported=pe.storage=void 0;var ox=li,Xn=fi(ox),ax=Rr,yd=fi(ax),ix=pi,_d=fi(ix);function fi(e){return e&&e.__esModule?e:{default:e}}var Yt=null;(0, Xn.default)("localStorage")?pe.storage=Yt=window.localStorage:(0, Xn.default)("sessionStorage")?pe.storage=Yt=window.sessionStorage:(0, Xn.default)("cookieStorage")?pe.storage=Yt=new yd.default:pe.storage=Yt=new _d.default;var vi=pe.default=Yt;pe.storage=Yt,pe.isSupported=Xn.default,pe.CookieStorage=yd.default,pe.MemoryStorage=_d.default;const eo="bp-sdk-";function Mr(e){return vi.getItem(`${eo}${e}`)}function gi(e,t){vi.setItem(`${eo}${e}`,t);}function sx(e){vi.removeItem(`${eo}${e}`);}function cx(){Object.keys(Rt).forEach(e=>sx(e));}function ux(){var e;return JSON.parse((e=Mr(Rt.user))!=null?e:"{}")}function lx(e){return (e==null?void 0:e.token_type)==="cookie"&&e!=null&&e.token_expires_at?e.token_expires_at*1e3:e!=null&&e.expires_in?Date.now()+e.expires_in*1e3:0}function dx(e){var t,r,n,o,a,i,s,c,u,l,d,f,g,v,h,b,m,_,A,w,S,K,re,ne,fe,ve,ue,Ne,ks,Ds;return {token:(t=e==null?void 0:e.access_token)!=null?t:"",expiresAt:lx(e),email:(n=(r=e==null?void 0:e.extra)==null?void 0:r.raw_info.email)!=null?n:"",firstName:(a=(o=e==null?void 0:e.extra)==null?void 0:o.raw_info.first_name)!=null?a:"",lastName:(s=(i=e==null?void 0:e.extra)==null?void 0:i.raw_info.last_name)!=null?s:"",name:(u=(c=e==null?void 0:e.extra)==null?void 0:c.raw_info.name)!=null?u:"",permissions:(f=(d=(l=e==null?void 0:e.extra)==null?void 0:l.raw_info)==null?void 0:d.permissions)!=null?f:[],tenantLogoUrl:(h=(v=(g=e==null?void 0:e.extra)==null?void 0:g.raw_info)==null?void 0:v.tenant_logo_url)!=null?h:"",tenantName:(_=(m=(b=e==null?void 0:e.extra)==null?void 0:b.raw_info)==null?void 0:m.tenant_name)!=null?_:"",tenantTheme:(S=(w=(A=e==null?void 0:e.extra)==null?void 0:A.raw_info)==null?void 0:w.tenant_theme)!=null?S:"",tenantState:(ne=(re=(K=e==null?void 0:e.extra)==null?void 0:K.raw_info)==null?void 0:re.tenant_state)!=null?ne:"",tenantWholesalerId:(ue=(ve=(fe=e==null?void 0:e.extra)==null?void 0:fe.raw_info)==null?void 0:ve.tenant_wholesaler_id)!=null?ue:"",user:e,userId:(Ds=(ks=(Ne=e==null?void 0:e.extra)==null?void 0:Ne.raw_info)==null?void 0:ks.user_id)!=null?Ds:""}}const px=e=>{switch(e){case In.TakeOver:return {model:"application_forms/clones",action:"create"};case In.SendToExchange:return {model:"application_forms/application_forms_question_sets",action:"create"};case In.FeatureCheck:return {model:"features",action:"index"};default:return {model:"",action:""}}};function fx({user:e=ux(),model:t,action:r,object:n}){var o,a;if(!((a=(o=e.extra)==null?void 0:o.raw_info)!=null&&a.permissions))return !1;let i=!1,s=n==null;for(const c of e.extra.raw_info.permissions){const[u,l,d,f]=c.split(":");if(d===t&&f===r)if(i=!0,n)switch(u){case"Tenant":l===n.tenant_id&&(s=!0);break;case"Owner":l===n.owner_id&&(s=!0);break;case"UserGroup":l===n.user_group_id&&(s=!0);break}else break}return i&&s}var vx=Object.defineProperty,gx=Object.defineProperties,hx=Object.getOwnPropertyDescriptors,$d=Object.getOwnPropertySymbols,mx=Object.prototype.hasOwnProperty,bx=Object.prototype.propertyIsEnumerable,wd=(e,t,r)=>t in e?vx(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,yx=(e,t)=>{for(var r in t||(t={}))mx.call(t,r)&&wd(e,r,t[r]);if($d)for(var r of $d(t))bx.call(t,r)&&wd(e,r,t[r]);return e},_x=(e,t)=>gx(e,hx(t)),$x=(e,t)=>t.reduce((r,n)=>_x(yx({},r),{[n]:e[n]}),{}),wx=Ya;function Ax(e,t){return wx(e,t)}var Ox=Ax;Cr(Ox);function Sx(e,t,r){switch(r.length){case 0:return e.call(t);case 1:return e.call(t,r[0]);case 2:return e.call(t,r[0],r[1]);case 3:return e.call(t,r[0],r[1],r[2])}return e.apply(t,r)}var jx=Sx,Cx=jx,Ad=Math.max;function Px(e,t,r){return t=Ad(t===void 0?e.length-1:t,0),function(){for(var n=arguments,o=-1,a=Ad(n.length-t,0),i=Array(a);++o<a;)i[o]=n[t+o];o=-1;for(var s=Array(t+1);++o<t;)s[o]=n[o];return s[t]=r(i),Cx(e,this,s)}}var Ex=Px;function Ix(e){return function(){return e}}var Tx=Ix,qx=at,xx=function(){try{var e=qx(Object,"defineProperty");return e({},"",{}),e}catch(t){}}(),Od=xx,kx=Tx,Sd=Od,Dx=Gn,Lx=Sd?function(e,t){return Sd(e,"toString",{configurable:!0,enumerable:!1,value:kx(t),writable:!0})}:Dx,Nx=Lx,Rx=800,Mx=16,Vx=Date.now;function Fx(e){var t=0,r=0;return function(){var n=Vx(),o=Mx-(n-r);if(r=n,o>0){if(++t>=Rx)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}var Qx=Fx,Ux=Nx,Bx=Qx,Gx=Bx(Ux),zx=Gx,Wx=Gn,Hx=Ex,Zx=zx;function Kx(e,t){return Zx(Hx(e,t,Wx),e+"")}var Yx=Kx;function Jx(e,t,r,n){for(var o=e.length,a=r+(n?1:-1);n?a--:++a<o;)if(t(e[a],a,e))return a;return -1}var Xx=Jx;function ek(e){return e!==e}var tk=ek;function rk(e,t,r){for(var n=r-1,o=e.length;++n<o;)if(e[n]===t)return n;return -1}var nk=rk,ok=Xx,ak=tk,ik=nk;function sk(e,t,r){return t===t?ik(e,t,r):ok(e,ak,r)}var ck=sk,uk=ck;function lk(e,t){var r=e==null?0:e.length;return !!r&&uk(e,t,0)>-1}var jd=lk;function dk(e,t,r){for(var n=-1,o=e==null?0:e.length;++n<o;)if(r(t,e[n]))return !0;return !1}var Cd=dk,pk=Na,fk=jd,vk=Cd,gk=Ea,hk=Ua,mk=Ra,bk=200;function yk(e,t,r,n){var o=-1,a=fk,i=!0,s=e.length,c=[],u=t.length;if(!s)return c;r&&(t=gk(t,hk(r))),n?(a=vk,i=!1):t.length>=bk&&(a=mk,i=!1,t=new pk(t));e:for(;++o<s;){var l=e[o],d=r==null?l:r(l);if(l=n||l!==0?l:0,i&&d===d){for(var f=u;f--;)if(t[f]===d)continue e;c.push(l);}else a(t,d,n)||c.push(l);}return c}var _k=yk,Pd=Pr,$k=Fa,wk=he,Ed=Pd?Pd.isConcatSpreadable:void 0;function Ak(e){return wk(e)||$k(e)||!!(Ed&&e&&e[Ed])}var Ok=Ak,Sk=Bu,jk=Ok;function Id(e,t,r,n,o){var a=-1,i=e.length;for(r||(r=jk),o||(o=[]);++a<i;){var s=e[a];t>0&&r(s)?t>1?Id(s,t-1,r,n,o):Sk(o,s):n||(o[o.length]=s);}return o}var Ck=Id;function Pk(){}var Ek=Pk,hi=tl,Ik=Ek,Tk=Ma,qk=1/0,xk=hi&&1/Tk(new hi([,-0]))[1]==qk?function(e){return new hi(e)}:Ik,kk=xk,Dk=Na,Lk=jd,Nk=Cd,Rk=Ra,Mk=kk,Vk=Ma,Fk=200;function Qk(e,t,r){var n=-1,o=Lk,a=e.length,i=!0,s=[],c=s;if(r)i=!1,o=Nk;else if(a>=Fk){var u=t?null:Mk(e);if(u)return Vk(u);i=!1,o=Rk,c=new Dk;}else c=t?[]:s;e:for(;++n<a;){var l=e[n],d=t?t(l):l;if(l=r||l!==0?l:0,i&&d===d){for(var f=c.length;f--;)if(c[f]===d)continue e;t&&c.push(d),s.push(l);}else o(c,d,r)||(c!==s&&c.push(d),s.push(l));}return s}var Td=Qk,Uk=_k,Bk=Ck,qd=Td;function Gk(e,t,r){var n=e.length;if(n<2)return n?qd(e[0]):[];for(var o=-1,a=Array(n);++o<n;)for(var i=e[o],s=-1;++s<n;)s!=o&&(a[o]=Uk(a[o]||i,e[s],t,r));return qd(Bk(a,1),t,r)}var zk=Gk,Wk=Un,Hk=Ft;function Zk(e){return Hk(e)&&Wk(e)}var Kk=Zk;function Yk(e){var t=e==null?0:e.length;return t?e[t-1]:void 0}var Jk=Yk,Xk=Gu,eD=Yx,tD=zk,rD=Kk,nD=Jk,oD=eD(function(e){var t=nD(e);return t=typeof t=="function"?t:void 0,tD(Xk(e,rD),void 0,t)}),aD=oD;Cr(aD);function iD(e){var t,r;const n=new Map;if((r=(t=e==null?void 0:e.question_set)==null?void 0:t.question_groups)!=null&&r.length){const o=e.question_set.question_groups,a=o.flatMap(i=>i.question_ids);o.forEach(i=>{const s=i.parent_question_id,c=a.includes(s),u=i.question_ids.includes(s);c&&!u||n.set(s,ro(e,i));});}return n}function sD(e,t,r){const n=o=>t.answers.some(a=>a.question.id===r&&a.answer_pool_id===o);return e.reduce(([o,a],i)=>n(i)?[[...o,i],a]:[o,[...a,i]],[[],[]])}function cD(e,t){let r=new Map;const n=(o,a)=>{const i=e.answers.find(c=>c.answer_pool_id===o),s=e.answers.find(c=>c.answer_pool_id===a);return (i.created_at instanceof Date?i.created_at.getTime():new Date(i.created_at).getTime())-(s.created_at instanceof Date?s.created_at.getTime():new Date(s.created_at).getTime())};return OE(Array.from(t),["1.parentQuestionAnswer.question.order","1.parentQuestionAnswer.created_at"]).forEach(([o,a])=>{const[i,s]=sD(Array.from(new Set(a.answers.map(u=>u.answer_pool_id))),e,o);let c=0;i.sort(n).forEach(u=>{const l=to({appFormQuestionSet:e,questionGroupsWithAns:a,parentQuestionAnswer:e.answers.find(d=>d.question.id===o&&d.answer_pool_id===u),answerPoolId:u,order:c++});r.set(l.parentQuestionAnswer.id,l);}),s.forEach(()=>{r=xd(e,r,o,c);});}),r}function xd(e,t,r,n){return e.answers.filter(o=>o.question.id===r).forEach(o=>{const a=e.question_set.question_groups.find(i=>i.parent_question_id===o.question.id);if(a){const i=ro(e,a),s=to({appFormQuestionSet:e,questionGroupsWithAns:i,parentQuestionAnswer:o,answerPoolId:o.answer_pool_id||null,order:n++});t.set(o.id,s);}}),t}function to({appFormQuestionSet:e,questionGroupsWithAns:t,parentQuestionAnswer:r,answerPoolId:n,order:o}){const a=t.answers.filter(s=>s.answer_pool_id===n),i=uD(e,t,n);return i.forEach((s,c)=>{a.find(u=>u.id===c)||a.push(s.parentQuestionAnswer);}),{answers:a.sort(mi),group_type:t.group_type,nestedGroupsWithAnswers:i,label:t.label,order:o,parentQuestionAnswer:r,groupId:t.groupId,poolId:r.answer_pool_id,questionIds:t.questionIds}}function uD(e,t,r){var n;const o=new Map;let a=0;return (n=t.nestedGroupsWithAnswers)==null||n.forEach((i,s)=>{const c=e.answers.find(u=>u.question.id===s&&u.answer_pool_id===r);if(c){const u=to({appFormQuestionSet:e,questionGroupsWithAns:i,parentQuestionAnswer:c,answerPoolId:r,order:a++});o.set(u.parentQuestionAnswer.id,u);}else r?c||lD(e,o,s,r,a):xd(e,o,s,a);}),o}function lD(e,t,r,n,o){return e.answers.filter(a=>a.question.id===r&&a.parent_answer_pool_id===n).forEach(a=>{const i=e.question_set.question_groups.find(s=>s.parent_question_id===a.question.id);if(i){const s=ro(e,i),c=to({appFormQuestionSet:e,questionGroupsWithAns:s,parentQuestionAnswer:a,answerPoolId:a.answer_pool_id||"",order:o++});t.set(a.id,c);}}),t}function ro(e,t){const r=e.answers.filter(o=>t.question_ids.includes(o.question.id)&&o.display_type!=="private").sort(mi),n=e.answers.find(o=>o.question.id===t.parent_question_id);return {answers:r,group_type:t.group_type,nestedGroupsWithAnswers:dD(e,t.id),label:t.label,order:0,parentQuestionAnswer:n,poolId:n==null?void 0:n.answer_pool_id,groupId:t.id,questionIds:t.question_ids}}function dD(e,t){const r=new Map,n=e.question_set.question_groups.find(o=>o.id===t);return e.question_set.question_groups.map(o=>o.parent_question_id).filter(o=>o!==(n==null?void 0:n.parent_question_id)).forEach(o=>{if(n!=null&&n.question_ids.includes(o)){const a=e.question_set.question_groups.find(i=>i.parent_question_id===o);a&&r.set(a.parent_question_id,ro(e,a));}}),r}function mi(e,t){return e.question.order===t.question.order?new Date(e.created_at).getTime()-new Date(t.created_at).getTime():e.question.order-t.question.order}function kd(e){return e.group_type===Ca.Stepper}function pD(e){return e.group_type===Ca.Step}function fD(e,t){return e.parentQuestionAnswer.question.order-t.parentQuestionAnswer.question.order}var Dd=Od;function vD(e,t,r){t=="__proto__"&&Dd?Dd(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r;}var gD=vD;function hD(e,t,r,n){for(var o=-1,a=e==null?0:e.length;++o<a;){var i=e[o];t(n,i,r(i),e);}return n}var mD=hD,bD=yl;function yD(e,t,r,n){return bD(e,function(o,a,i){t(n,o,r(o),i);}),n}var _D=yD,$D=mD,wD=_D,AD=bl,OD=he;function SD(e,t){return function(r,n){var o=OD(r)?$D:wD,a=t?t():{};return o(r,e,AD(n),a)}}var jD=SD,CD=gD,PD=jD,ED=Object.prototype,ID=ED.hasOwnProperty,TD=PD(function(e,t,r){ID.call(e,r)?e[r].push(t):CD(e,r,[t]);}),qD=TD;Cr(qD);var xD=Td;function kD(e){return e&&e.length?xD(e):[]}var DD=kD;Cr(DD);const Ld=e=>nw.includes(e),Jt=e=>(t,r)=>{if(t)return r.push(e)},no=Jt(se.Required),bi=Jt(se.MinimumValue),LD=Jt(se.MinimumDate),yi=Jt(se.MaximumValue),ND=Jt(se.MaximumDate),_i=Jt(se.Pattern),RD=e=>{const{field_type:t,reference_type:r,min_date:n,min_value:o,max_date:a,max_value:i,pattern:s}=e.question,{required:c}=e,u=[];switch(ci(t)){case O.Text:no(c,u),r===Vt.Accumulator?(yi(o!=null,u),bi(i!=null,u)):r!==Vt.Street1&&(bi(o!=null,u),yi(i!=null,u),_i(s!=null,u));break;case O.Masked:no(c,u),bi(o!=null,u),yi(i!=null,u),_i(s!=null,u);break;case O.Date:no(c,u),LD(n!=null,u),ND(a!=null,u),_i(s!=null,u);break;case O.DropDown:no(c,u);}return u},Nd=e=>e!=null&&e.question?RD(e):[],MD=(e,t)=>{var r,n,o,a,i,s,c,u,l;let d;switch(t){case se.Required:ci(e.question.field_type)===O.DropDown?d=!!((n=e.choice_id)!=null?n:((r=e.choice_ids)!=null?r:[]).length):d=e.value===0||e.value!=null&&e.value!="";break;case se.MinimumValue:Ld(e.question.field_type)?d=+((o=e.value)!=null?o:0)>=e.question.min_value:d=((i=(a=e.value)!=null?a:"")==null?void 0:i.length)>=e.question.min_value;break;case se.MaximumValue:Ld(e.question.field_type)?d=+((s=e.value)!=null?s:0)<=e.question.max_value:d=((u=(c=e.value)!=null?c:"")==null?void 0:u.length)<=e.question.max_value;break;case se.MinimumDate:d=!!e.value&&new Date(e.value)>=new Date(e.question.min_date);break;case se.MaximumDate:d=!!e.value&&new Date(e.value)<=new Date(e.question.max_date);break;case se.Pattern:d=!!((l=e.value)!=null?l:"").toString().match(new RegExp(e.question.pattern));break;default:d=!0;}return d},VD=e=>(t,r)=>MD(e,r)?t:[...t,r],$i=(e,t=[])=>{if(!e.required&&!Vq(e))return [];const r=t.reduce(VD(e),[]);return r.includes(se.Required)?[se.Required]:r.includes(se.Pattern)?[se.Pattern]:r},Rd=e=>{const t=Nd(e);return $i(e,t)};function Md(e){var t;return e?((t=e.question)==null?void 0:t.reference_type)===Vt.Accumulator?!e.active||Rd(e).length===0:!e.active||e.is_completed||Rd(e).length===0:!0}function Vd(e){return (e==null?void 0:e.find(t=>!Md(t)))===void 0}const FD=["mqs_add_driver","mqs_add_vehicle"];function QD(e){let t=!1;const r=e.question_set.question_pools.map(n=>n.parent_question_id);return r.length>0&&(t=!e.answers.some(n=>!r.includes(n.question.id)||!FD.includes(n.question.code))),t}function wi(e){return e.answers.length===0?!0:Vd(e.answers)&&!QD(e)}function Fd(e,t){return t.parentQuestionSets.find(r=>r.question_set_id===e.question.question_set_id)}function UD(e,t){return (e!=null?e:[]).find(r=>r.question_set.code===t)}function BD(e,t){return window.dispatchEvent(new CustomEvent(`${eo}${e}`,{detail:t}))}function R(e){return !!(window&&window.dispatchEvent(new ErrorEvent("error",{error:e})))}var GD=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});function zD(e,t){return GD(this,null,function*(){var r,n;gi(Rt.token,e),gi(Rt.cableToken,t);try{(n=(r=window.BpSdk)==null?void 0:r.store)==null||n.dispatch({type:"auth/Set Auth",payload:{token:e}});}catch(o){}BD(Rt.token,e);})}const WD=[];for(let e=0;e<256;++e)WD.push((e+256).toString(16).slice(1));typeof crypto!="undefined"&&crypto.randomUUID&&crypto.randomUUID.bind(crypto);var HD=Object.defineProperty,ZD=Object.defineProperties,KD=Object.getOwnPropertyDescriptors,Qd=Object.getOwnPropertySymbols,YD=Object.prototype.hasOwnProperty,JD=Object.prototype.propertyIsEnumerable,Ud=(e,t,r)=>t in e?HD(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Bd=(e,t)=>{for(var r in t||(t={}))YD.call(t,r)&&Ud(e,r,t[r]);if(Qd)for(var r of Qd(t))JD.call(t,r)&&Ud(e,r,t[r]);return e},Gd=(e,t)=>ZD(e,KD(t));const le=(e,t,r)=>{const n=Gd(Bd({},e.map),{[t]:r}),o=Object.values(n).reduce((a,i)=>a===ye.Error||i===ye.Error?ye.Error:a===ye.Loading||i===ye.Loading?ye.Loading:a,ye.Loaded);return Gd(Bd({},e),{state:o,map:n})};var XD=Object.defineProperty,eL=Object.defineProperties,tL=Object.getOwnPropertyDescriptors,zd=Object.getOwnPropertySymbols,rL=Object.prototype.hasOwnProperty,nL=Object.prototype.propertyIsEnumerable,Wd=(e,t,r)=>t in e?XD(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,oo=(e,t)=>{for(var r in t||(t={}))rL.call(t,r)&&Wd(e,r,t[r]);if(zd)for(var r of zd(t))nL.call(t,r)&&Wd(e,r,t[r]);return e},ao=(e,t)=>eL(e,tL(t));const Ai=e=>e.reduce((t,r)=>ao(oo({},t),{[r.id]:r.value}),{}),Oi=e=>e.reduce((t,r)=>ao(oo({},t),{[r.id]:r.name}),{}),Si=e=>e.reduce((t,r)=>ao(oo({},t),{[r.id]:r.name}),{}),ji=e=>e.reduce((t,r)=>ao(oo({},t),{[r.id]:r.name}),{}),Ci=({configurations:e})=>e!=null?e:[],oL=e=>`${e.street_1}${e.street_2?" "+e.street_2:""} ${e.city}, ${e.state} ${e.zipcode}`,aL=e=>{switch(e){case Ue.ROUTE:case Ue.STREET_NUMBER:case Ue.STREET_ADDRESS:return Nt.STREET_1;case Ue.SUBPREMISE:return Nt.STREET_2;case Ue.LOCALITY:return Nt.CITY;case Ue.STATE:return Nt.STATE;case Ue.POSTAL_CODE:return Nt.ZIPCODE}};var iL=Object.defineProperty,Hd=Object.getOwnPropertySymbols,sL=Object.prototype.hasOwnProperty,cL=Object.prototype.propertyIsEnumerable,Zd=(e,t,r)=>t in e?iL(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,uL=(e,t)=>{for(var r in t||(t={}))sL.call(t,r)&&Zd(e,r,t[r]);if(Hd)for(var r of Hd(t))cL.call(t,r)&&Zd(e,r,t[r]);return e};const lL=e=>e.application,We=p([lL],e=>e.questionSets),me=p(We,e=>e.list),Pi=p(We,e=>e.active),Kd=p(We,e=>e.activeAnswerId),Oe=p(me,Pi,(e,t)=>e.find(r=>r.question_set_id===t)),Yd=p(We,e=>e.idOfNextActiveQuestionSetNotInState);p(Oe,e=>{var t;return (t=e==null?void 0:e.question_set)==null?void 0:t.code});p(me,e=>e.filter(t=>t.is_active&&t.answers.some(r=>r.active)));p(me,e=>e.filter(t=>t.is_active&&t.is_completed));const Jd=p(me,e=>e.filter(t=>t.is_active&&!t.is_completed));p([me,(e,t)=>t],(e,t)=>e.find(r=>r.id===t));const gL=p([me,(e,t)=>t],(e,t)=>UD(e,t));p(We,e=>e.completedQuestionSetAnswerHasChanged);const mL=p(We,e=>e.parentQuestionSets),bL=p([We,(e,t)=>t],(e,t)=>{var r,n;const o=uL({},t);return (r=e.pendingActiveAnswerUpdate)!=null&&r.question.reference_type&&o[e.pendingActiveAnswerUpdate.question.reference_type]&&e.pendingActiveAnswerUpdate.id===o[e.pendingActiveAnswerUpdate.question.reference_type].id&&(o[(n=e.pendingActiveAnswerUpdate)==null?void 0:n.question.reference_type]=e.pendingActiveAnswerUpdate),o});p([Oe,(e,t)=>t],(e,t)=>e==null?void 0:e.answers.find(r=>r.id===t));const Xd=p([Oe,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e==null?void 0:e.answers)==null?void 0:r.filter(o=>(o==null?void 0:o.answer_pool_id)===t))!=null?n:[]}),ep=p([Oe,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e==null?void 0:e.answers)==null?void 0:r.filter(o=>(o==null?void 0:o.question.question_group_id)===t))!=null?n:[]}),_L=p([ep,Xd,Kd,(e,t)=>t],(e,t,r)=>!!(e!=null&&e.find(n=>(n==null?void 0:n.id)===r))||!!(t!=null&&t.find(n=>(n==null?void 0:n.id)===r))),$L=p([Oe,(e,t)=>t],(e,t)=>{var r;return (r=e==null?void 0:e.answers.filter(n=>n.question.question_group_id===t.groupId&&n.answer_pool_id===t.poolId))!=null?r:[]});p([me,(e,t)=>t],(e,t)=>t instanceof RegExp?e.flatMap(r=>r.answers).filter(r=>t.test(r.question.code)):e.flatMap(r=>r.answers).filter(r=>r.question.code.includes(t)));p([me,(e,t)=>t],(e,t)=>t instanceof RegExp?e.flatMap(r=>r.answers).filter(r=>t.test(r.question.reference_type)):e.flatMap(r=>r.answers).filter(r=>r.question.code.includes(t)));p([me],e=>{var t,r;const n=e.flatMap(a=>a.answers).find(a=>a.question.reference_type==="business_class"),o=(r=(t=n==null?void 0:n.question)==null?void 0:t.choices)==null?void 0:r.find(a=>a.id===(n==null?void 0:n.choice_id));return o==null?void 0:o.value});const tp=e=>e.application,_e=p(tp,e=>e.form),SL=p(_e,e=>e.id);p(_e,e=>{var t;return (t=e.features)!=null?t:[]});p(_e,e=>{var t;return (t=e.reference_id)!=null?t:""});p(_e,e=>e.percent_completed);p(tp,e=>e.progress.meta.form.forwardProgressPercent);const IL=p(_e,e=>e.state);p(_e,e=>({source:e.source,metadata:e.source_metadata}));const rp=p(_e,e=>e.appetite),np=e=>e.application.form.buckets;p(np,_e,(e,t)=>e.find(r=>r.id===t.current_bucket_id));p(_e,e=>e.storefront_active_enrolled_product_codes);p(rp,e=>{const t=DL(e);return LL(t)});function DL(e){return e.flatMap(t=>t.carriers.filter(r=>r.is_enrolled))}function LL(e){const t=new Set;return e.reduce((r,n)=>(t.has(n.tenant_id)||(t.add(n.tenant_id),r.push(n)),r),[])}const ae=e=>e.auth;p(ae,e=>e==null?void 0:e.refreshCount);const op=p(ae,e=>e==null?void 0:e.user);p(ae,e=>e==null?void 0:e.token);p(ae,e=>{var t;return (t=e==null?void 0:e.expiresAt)!=null?t:Date.now()-1});const ap=p(ae,e=>{var t;return (t=e==null?void 0:e.expiresAt)!=null?t:Date.now()-1}),ip=p(ae,e=>e==null?void 0:e.userId);p(ae,e=>e==null?void 0:e.email);p(ae,e=>e==null?void 0:e.name);p(ae,e=>e==null?void 0:e.firstName);p(ae,e=>e==null?void 0:e.lastName);p(ae,e=>e==null?void 0:e.tenantLogoUrl);p(ae,e=>e==null?void 0:e.tenantName);p(ae,e=>e==null?void 0:e.tenantTheme);p(ae,e=>e==null?void 0:e.terminalTheme);const sp=p(ap,e=>e>Date.now()),Ei=e=>e.application.carriersProducts;p(Ei,e=>e.filter(t=>t.carrier.online));p(Ei,e=>e.filter(t=>!t.carrier.online));const KL=e=>e.status.remote.carriersProducts.state,cp=e=>e.application.choices,YL=p([cp,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list)!=null?n:[]}),Vr=e=>e.config,up=p([Vr,ae],(e,t)=>({env:e.env,uri:e.uri,token:t.token}));p(Vr,e=>e.googleApiToken);p(Vr,e=>e.cableToken);const eN=p(Vr,e=>e.debug||!1),Te=p(Vr,e=>e.options),tN=p(Te,e=>!!e.enableAutocomplete);p(Te,e=>!!e.enableTrailerSupport);const nN=p(Te,e=>!!e.enableDetailedRelevanceChips),oN=p(Te,e=>!!e.enableQuestionSetNavigation),aN=p(Te,e=>!!e.showParentQuestionSetsInNavigation),iN=p(Te,e=>!!e.enableAddressValidation),sN=p(Te,e=>e.addressValidationOptions),cN=p(Te,e=>!!e.shouldQuoteCardRenderSupplierLogos);var uN=Object.defineProperty,lN=Object.defineProperties,dN=Object.getOwnPropertyDescriptors,lp=Object.getOwnPropertySymbols,pN=Object.prototype.hasOwnProperty,fN=Object.prototype.propertyIsEnumerable,dp=(e,t,r)=>t in e?uN(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,pp=(e,t)=>{for(var r in t||(t={}))pN.call(t,r)&&dp(e,r,t[r]);if(lp)for(var r of lp(t))fN.call(t,r)&&dp(e,r,t[r]);return e},fp=(e,t)=>lN(e,dN(t));const Xt=e=>e.application.form.eligible_carriers_products;p(Xt,e=>Ii(e));const vp=p(Xt,e=>Ti(e)),io=p(Xt,e=>e.filter(t=>t.market?t.market===va.BP_QUOTABLE:t.carrier.online));p(io,e=>Ii(e));const gp=p(io,e=>Ti(e)),so=p(Xt,e=>e.filter(t=>t.market?t.market===va.OFFLINE:!t.carrier.online));p(so,e=>Ii(e));p(so,e=>Ti(e));p([Xt,(e,t)=>t],(e,t)=>e.reduce((r,n)=>n.carrier.id===t?[...r,n.product]:r,[]));function Ii(e){return Object.values(e.reduce((t,r)=>fp(pp({},t),{[r.carrier.id]:r.carrier}),{}))}function Ti(e){const t=e.reduce((r,n)=>{const o=r.get(n.carrier.id);return o?o.products.push(n.product):r.set(n.carrier.id,fp(pp({},n.carrier),{products:[n.product]})),r},new Map);return Array.from(t.values())}const yN=e=>e.application;p(yN,e=>e.enrollments);const co=e=>e.application.form,hp=e=>e.status.remote,wN=p([co],e=>e.locked);p([co],e=>e.taken_over);const ON=p([co],e=>e.cloned);p([co],e=>e.inactive);const jN=p([hp],e=>e.application.form===y.Loaded),mp=p([hp],e=>e.application.form===y.Loading),er=e=>e.application.form.messages||[];p([er,(e,t)=>t],(e,t)=>e.filter(r=>r.component===et.Answer&&r.metadata.answer_id===t&&r.location!==yn.Question));p([er,Oe],(e,t)=>{const r=(t==null?void 0:t.answers.map(n=>n.id))||[];return !e.length||!r.length?[]:e.filter(n=>n.component===et.Answer&&n.location!==yn.Question&&r.includes(n.metadata.answer_id||""))});p([er,(e,t)=>t],(e,t)=>e.filter(r=>r.component===et.Answer&&r.metadata.answer_id===t&&r.location===yn.Question));p([er,(e,t)=>t],(e,t)=>e.filter(r=>r.component===et.QuoteRequest&&r.metadata.quote_request_id===t));const TN=p(er,e=>e.filter(t=>t.component===et.Application)),qN=e=>e.application.pendingProcesses,xN=p(qN,Pi,(e,t)=>e.some(r=>r.question_set_id===t&&(r.state===bn.Pending||r.state===bn.Queued||r.state===bn.Executing)));var kN=Object.defineProperty,bp=Object.getOwnPropertySymbols,DN=Object.prototype.hasOwnProperty,LN=Object.prototype.propertyIsEnumerable,yp=(e,t,r)=>t in e?kN(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,NN=(e,t)=>{for(var r in t||(t={}))DN.call(t,r)&&yp(e,r,t[r]);if(bp)for(var r of bp(t))LN.call(t,r)&&yp(e,r,t[r]);return e};p(ae,e=>e.permissions);const MN=p([ip,_e],(e,t)=>t!=null&&e===t.owner_id),VN=p([op,(e,t)=>NN({},t)],(e,{permission:t,object:r,action:n})=>{const o=px(t);return fx({user:e,object:r,action:n!=null?n:o.action,model:o.model||t})});p(ae,e=>!!e.options.impersonated);p(Oe,(e,t)=>t,(e,t)=>{var r;if(e)return (r=e.question_set)==null?void 0:r.question_pools.find(n=>n.parent_question_id===t.question.id)});const pt=e=>e.application.quotes,UN=p([pt,(e,t)=>t],(e,t)=>e.find(r=>r.id===t));p([pt,(e,t)=>t],(e,t)=>e.filter(r=>r.request_status===t));p([pt,(e,t)=>t],(e,t)=>e.filter(r=>r.products.includes(t)));p([pt,(e,t)=>t],(e,t)=>e.filter(r=>r.products.filter(n=>n.id===t).length));p([pt,(e,t)=>t],(e,t)=>e.filter(r=>r.products.filter(n=>n.name===t).length));const _p=e=>e.status.remote.quotes.state===y.Loading;p(mp,_p,(e,t)=>e||!t);var ZN=Object.defineProperty,KN=Object.defineProperties,YN=Object.getOwnPropertyDescriptors,$p=Object.getOwnPropertySymbols,JN=Object.prototype.hasOwnProperty,XN=Object.prototype.propertyIsEnumerable,wp=(e,t,r)=>t in e?ZN(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,uo=(e,t)=>{for(var r in t||(t={}))JN.call(t,r)&&wp(e,r,t[r]);if($p)for(var r of $p(t))XN.call(t,r)&&wp(e,r,t[r]);return e},lo=(e,t)=>KN(e,YN(t));const eR=e=>e.application,qi=p([eR],e=>e.form.real_time_eligibility.carriers||[]);p([qi],e=>e.filter(t=>t.eligible));p([qi],e=>e.filter(t=>!t.eligible));const Ap=p([me],e=>Fq(e.flatMap(t=>t.answers),"coverage_type"));p([Ap],e=>{const t=e.length?Mq(e[0]):null;return Array.isArray(t)?t:t?[t]:[]});const xi=(e,t,r=0)=>{const n={active:!0,annual_price:"",application_form_id:"",bindable:!0,calls_to_action:[],carrier:{code:e.code,id:e.id,logo_url:e.logo_url,logo_url_small:e.logo_url_small,name:e.name,online:e.online,labels:e.labels,opt_in_required:e.opt_in_required,opted_in:e.opted_in},consumer_selected:!0,coverage:{disclaimer:"",limit_and_deductible:[],limit_and_deductible_default:[]},created_at:new Date().toISOString(),customizable:!1,downpayment:"",effective_date:null,expired:!1,finished_at:new Date().toISOString(),focused:!1,id:e.id,monthly_price:"",number_of_installments:0,online:e.online||!1,order:r,owner_id:"",permitted_to_bind:!0,preferred:!1,price_details:{fees:[],new_total_premium:"",taxes:[],term:""},products:e.products.map(o=>lo(uo({},o),{from_name:null})),quote_number:"",recommended:!1,request_status:t,retrieve_url:"",semiannual_price:"",selected:!1,started_at:new Date().toISOString(),surcharge:0,tenant_id:"",translated_messages:[],updated_at:new Date().toISOString(),actions:[],attachments:[],supplier_code:null,supplier_name:null,supplier_logo_url:null,supplier_logo_url_full:null};return lo(uo({},n),{quotes:[n]})},Op=e=>{var t;return (t=(e==null?void 0:e.opt_in_required)&&!(e!=null&&e.opted_in))!=null?t:!1},Sp=(e,t)=>{const r=t.some(n=>n.question_set.tenant_id===(e==null?void 0:e.id));return Op(e)&&!r},jp="not_opted_in",Cp="incomplete_question_set",Pp=p(Jd,gp,(e,t)=>e.reduce((r,n)=>{const o=t.find(a=>a.id===n.question_set.tenant_id);return [...r,...o?[xi(o,Cp,null)]:[]]},[])),Ep=(e=[])=>{const t=[ga.NON_ADMITTED,ga.ADDITIONAL_MARKET];return e.some(r=>t.includes(r))},Ip=p(vp,e=>e.filter(t=>Ep(t.labels))),Tp=p(Ip,me,(e,t)=>e.filter(r=>Sp(r,t)).map(r=>xi(r,jp))),qp=p(so,io,(e,t)=>e.filter(r=>!t.some(n=>n.carrier.id===r.carrier.id&&n.product.id===r.product.id)).map(r=>{const n={active:!0,annual_price:"",application_form_id:r.application_form_id,bindable:!0,calls_to_action:[],carrier:r.carrier,consumer_selected:!0,coverage:{disclaimer:"",limit_and_deductible:[],limit_and_deductible_default:[]},created_at:new Date().toISOString(),customizable:!1,downpayment:"",effective_date:null,expired:!1,finished_at:new Date().toISOString(),focused:!1,id:r.carriers_product_id,monthly_price:"",number_of_installments:0,online:!1,order:0,owner_id:"",permitted_to_bind:!0,preferred:!1,price_details:{fees:[],new_total_premium:"",taxes:[],term:""},products:[lo(uo({},r.product),{from_name:null})],quote_number:"",recommended:!1,request_status:ha.referral,retrieve_url:"",semiannual_price:"",selected:!1,started_at:new Date().toISOString(),surcharge:0,tenant_id:"",translated_messages:[],updated_at:new Date().toISOString(),actions:[],attachments:[],supplier_code:null,supplier_name:null,supplier_logo_url:null,supplier_logo_url_full:null};return lo(uo({},n),{quotes:[n]})}));p(pt,Pp,Tp,qp,(e,t,r,n)=>[...e||[],...t||[],...r||[],...n||[]]);const po=e=>e.application.refilters;p([po,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list)!=null?n:[]});p([po,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list.filter(o=>o.value===null))!=null?n:[]});p([po,(e,t)=>t],(e,t)=>{var r,n;return (n=(r=e[t])==null?void 0:r.list.filter(o=>o.value!==null))!=null?n:[]});const cR=e=>e.status.remote.answers.state,uR=e=>e.status.remote.answers.map,lR=e=>e.status.remote.application.form,xp=e=>e.status.remote,dR=p(xp,e=>e.questionPools.state);p(xp,(e,t)=>t,(e,t)=>e.questionPools.map[t]);const kp=e=>e.status.remote,fR=p(kp,e=>e.questionSets.state),ki=p(kp,e=>e.questionSets.map);p(ki,Oe,Yd,(e,t,r)=>r?!0:t?e[t.id]===y.Loading:!1);p(ki,Oe,(e,t)=>t,(e,t,r)=>t?e[r]===y.Loading:!1);const hR=e=>e.status.remote,mR=p(hR,e=>e.quotes.state),$e=e=>e.status.local;p($e,e=>e.health);p($e,e=>e.state);p($e,e=>e.state===q.Loading);p($e,e=>e.config===q.Configured);p($e,e=>e.auth===q.Configured);const AR=p($e,e=>e.user===q.Configured);p($e,e=>e.auth===q.Configured);p($e,e=>e.auth===q.Configured&&e.user===q.Configured);p($e,e=>e.auth===q.Configured&&e.config===q.Configured&&e.user===q.Configured);p($e,e=>e.socket);const PR=e=>e.viewState,qe=p(PR,e=>e.vehicleOptions);p([(e,t)=>({state:e,questionGroupId:t})],({state:e,questionGroupId:t})=>({configurations:Bp(e,t),makes:Qp(e,t),models:Up(e,t),selection:Gp(e,t),vin:zp(e,t),years:Fp(e,t)}));const Dp=p(qe,e=>e.years),Lp=p(qe,e=>e.makes),Np=p(qe,e=>e.models),Rp=p(qe,e=>e.configurations),Mp=p(qe,e=>e.selection),Vp=p(qe,e=>e.vin),Fp=p([Dp,(e,t)=>t],(e,t)=>e[t]),Qp=p([Lp,(e,t)=>t],(e,t)=>e[t]),Up=p([Np,(e,t)=>t],(e,t)=>e[t]),Bp=p([Rp,(e,t)=>t],(e,t)=>e[t]),Gp=p([Mp,(e,t)=>t],(e,t)=>e[t]),zp=p([Vp,(e,t)=>t],(e,t)=>e[t]),IR=p([qe,(e,t)=>t],(e,t)=>{var r,n,o,a,i,s,c,u,l,d,f,g,v,h;const b=e.selection,m={year:(n=(r=b==null?void 0:b[t])==null?void 0:r.yearId)!=null?n:"",make:"",model:"",configuration:""},_=(o=b[t])==null?void 0:o.makeId;_&&(m.make=(s=(i=(a=e.makes)==null?void 0:a[t])==null?void 0:i[_])!=null?s:"");const A=(c=b[t])==null?void 0:c.modelId;A&&(m.model=(d=(l=(u=e.models)==null?void 0:u[t])==null?void 0:l[A])!=null?d:"");const w=(f=b[t])==null?void 0:f.configurationId;return w&&(m.configuration=(h=(v=(g=e.configurations)==null?void 0:g[t])==null?void 0:v[w])!=null?h:""),m}),TR=p([qe,(e,t)=>t],(e,t)=>e.selection[t]),qR=e=>e.status.remote,ft=p(qR,e=>e.vehicles);p(ft,e=>e.configurations);p(ft,e=>e.makes);p(ft,e=>e.models);const Wp=p(ft,e=>e.selection),Hp=p(ft,e=>e.vins);p(ft,e=>e.years);const NR=p([Wp,(e,t)=>t],(e,t)=>e[t]),RR=p([Hp,(e,t)=>t],(e,t)=>e[t]),X=e=>e.viewState,Zp=p([X],e=>e.features),Di=p([X],e=>e.quotes);p([X],e=>e.addressValidation);const Kp=p([X,(e,t)=>t],(e,t)=>e.addressValidation.validations[t]),VR=p([X,(e,t)=>t],(e,t)=>t?!!e.addressValidation.validations[t]:null),FR=p([X,(e,t)=>t],(e,t)=>{var r;return (r=e.addressValidation.validations[t])==null?void 0:r.verdict}),QR=p([X,(e,t)=>t],(e,t)=>{var r;return ((r=e.addressValidation.validations[t])==null?void 0:r.validationStatus)===ie.COMPLETED}),UR=p([X,(e,t)=>t],(e,t)=>{var r;return ((r=e.addressValidation.validations[t])==null?void 0:r.validationStatus)===ie.VALIDATION_ERROR}),BR=p([X,(e,t)=>t],(e,t)=>{var r,n,o,a;return ((r=e.addressValidation.validations[t])==null?void 0:r.validationStatus)===ie.VALIDATED||((n=e.addressValidation.validations[t])==null?void 0:n.validationStatus)===ie.COMPLETED||((o=e.addressValidation.validations[t])==null?void 0:o.validationStatus)===ie.VALIDATION_ERROR||((a=e.addressValidation.validations[t])==null?void 0:a.validationStatus)===ie.PROCEED_NEXT}),GR=p([X],e=>Object.entries(e.addressValidation.validations).filter(([,t])=>t.validationStatus===ie.REQUIRED).map(([t])=>t)),zR=p([X,(e,t)=>t],(e,t)=>{var r;return ((r=e.addressValidation.validations[t])==null?void 0:r.validationStatus)===ie.READY_TO_VALIDATE}),WR=p([X,(e,t)=>t],(e,t)=>{var r;return (r=e.addressValidation.validations[t])==null?void 0:r.validationStatus}),HR=p([X,(e,t)=>t],(e,t)=>t&&!!Object.entries(e.addressValidation.validations).filter(([r,n])=>(t.answers.map(o=>o.answer_pool_id).includes(r)||t.question_set.question_groups.map(o=>o.id).includes(r))&&n.validationStatus===ie.VALIDATED).length),ZR=p([X],e=>Object.fromEntries(Object.entries(e.addressValidation.validations).map(([t,r])=>[t,r.validationStatus]))),KR=p([X,(e,t)=>t],(e,t)=>{var r;return (r=e.addressValidation.validations[t])==null?void 0:r.address}),YR=p([X,(e,t)=>t],(e,t)=>{var r;return (r=e.addressValidation.validations[t])==null?void 0:r.attempt}),JR=p([X,(e,t)=>t],(e,t)=>{var r;return (r=e.addressValidation.validations[t])==null?void 0:r.stagedAnswers}),XR=p([X,(e,t)=>t],(e,t)=>{var r;return (r=e.addressValidation.validations[t])==null?void 0:r.originalAddress}),eM=p([X],e=>e.addressValidation.shouldOverrideValidationTrigger),tM=p([Zp,(e,t)=>t],(e,t)=>e.list.includes(t));p([Di],e=>e.filteredOnlineQuotes);p([Di],e=>e.filteredOfflineQuotes);var oM=Object.defineProperty,aM=Object.defineProperties,iM=Object.getOwnPropertyDescriptors,Yp=Object.getOwnPropertySymbols,sM=Object.prototype.hasOwnProperty,cM=Object.prototype.propertyIsEnumerable,Jp=(e,t,r)=>t in e?oM(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,fo=(e,t)=>{for(var r in t||(t={}))sM.call(t,r)&&Jp(e,r,t[r]);if(Yp)for(var r of Yp(t))cM.call(t,r)&&Jp(e,r,t[r]);return e},Xp=(e,t)=>aM(e,iM(t)),He=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});function ef(e){return He(this,null,function*(){var t,r;return (r=(t=window==null?void 0:window.BpSdk)==null?void 0:t.refreshGuestSetup(e))!=null?r:Promise.reject("SDK must be configured before loading a guest application")})}const vt="applicationForms",Fr=L(`${vt}/LoadOne`,(e,t)=>He(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield nt.getApplicationForm(r.id,r.config)).parsedBody}catch(o){const a=new D("Error loading app form",o);throw R(a),n(a),a}})),tr=L(`${vt}/LoadLatestOne`,(e,t)=>He(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield nt.getLatestApplicationForm(r.id,r.config)).parsedBody}catch(o){const a=new D("Error loading app form",o);throw R(a),n(a),a}})),gt=L(`${vt}/LoadOneFromToken`,(e,t)=>He(void 0,[e,t],function*(r,{rejectWithValue:n}){var o;try{const a=yield nt.getApplicationFormFromToken(r.token_id,r);if(a.parsedBody)return gi(Tt.cableToken,(o=a.parsedBody.application_form)==null?void 0:o.cable_token),a.parsedBody.application_form;throw new D("Application form does not exist on HTTP response body")}catch(a){const i=new D("Error loading app form from token",a);throw R(i),n(i),i}})),vo=L(`${vt}/LoadGuest`,(e,t)=>He(void 0,[e,t],function*(r,{rejectWithValue:n,dispatch:o,getState:a}){try{const i=Mr(Tt.token),s=Mr(Tt.cableToken),c=a();if(i&&s&&sp(c))return o(gt(Xp(fo({},r),{token:i,token_id:s})));const{parsedBody:u}=yield nt.getTokenForAuthGuestFlow(r);return yield zD(u.token,u.token_id),yield ef(fo(fo({},c.config),r)),o(gt(Xp(fo({},r),{token:Mr(Tt.token),token_id:Mr(Tt.cableToken)})))}catch(i){const s=new D("Error loading guest application form",i);throw R(s),n(s)}})),Qr=L(`${vt}/Clone`,(e,t)=>He(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield nt.cloneApplicationForm(r.appFormId,r.config)).parsedBody}catch(o){const a=new D("Error cloning application form",o);throw R(a),n(a),a}})),Ur=L(`${vt}/CreateAndLoad`,(e,t)=>He(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield nt.createApplicationForm(r.data,r.config)).parsedBody}catch(o){const a=new D("Error loading app form",o);throw R(a),n(a),a}})),tf=L(`${vt}/GetEligibilityResults`,(e,t)=>He(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield nt.getEligibilityResults(r.appId,r.config)).parsedBody}catch(o){const a=new D("Error getting eligibility results",o);throw R(a),n(a),a}})),Li=[Bc,Sr],k=[...Li,Fr.pending,tr.pending,gt.pending,Qr.pending,Ur.pending],rf=[Fr.pending,tr.pending,gt.pending,Qr.pending,Ur.pending],be=[Fr.fulfilled,tr.fulfilled,gt.fulfilled,Qr.fulfilled,Ur.fulfilled],Ni=[Fr.rejected,tr.rejected,gt.rejected,Qr.rejected,Ur.rejected];var uM=Object.defineProperty,lM=Object.defineProperties,dM=Object.getOwnPropertyDescriptors,nf=Object.getOwnPropertySymbols,pM=Object.prototype.hasOwnProperty,fM=Object.prototype.propertyIsEnumerable,of=(e,t,r)=>t in e?uM(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ze=(e,t)=>{for(var r in t||(t={}))pM.call(t,r)&&of(e,r,t[r]);if(nf)for(var r of nf(t))fM.call(t,r)&&of(e,r,t[r]);return e},Ri=(e,t)=>lM(e,dM(t));const Mi=Ze({},Vy),af=P({name:"form",initialState:Mi,reducers:{handleMessage:(e,{payload:t})=>sf(e,t),receiveAppetite:(e,{payload:t})=>cf(e,t),receiveApplicationForm:(e,{payload:t})=>uf(e,t),receiveApplicationFormProgress:(e,{payload:t})=>lf(e,t),receiveCarrierProducts:(e,{payload:t})=>df(e,t),receiveRealTimeEligibility:(e,{payload:t})=>pf(e,t)},extraReducers:e=>{e.addCase(tf.fulfilled,(t,{payload:r})=>{Kt(r,t.eligible_carriers_products)&&(t.eligible_carriers_products=r);}).addCase(Ae,(t,{payload:r})=>{let n=Ze({},t);return r.events.forEach(o=>{switch(o.event){case x.Message:n=sf(n,o.data);break;case x.Appetite:n=cf(n,o.data);break;case x.ApplicationForm:n=uf(n,o.data);break;case x.ApplicationFormPercentCompleted:n=lf(n,o.data);break;case x.CarrierProducts:n=df(n,o.data);break;case x.RealTimeEligibility:n=pf(n,o.data);break}}),n}).addMatcher($(...k),t=>Ze({},Mi)).addMatcher($(...be),(t,{payload:r})=>{const n=$x(r,Object.keys(Mi));return Ze(Ze({},t),n)});}}),sf=(e,t)=>{let r=[...e.messages];const n=r.findIndex(o=>o.id===t.id);if(n===-1)r=t.active?[...e.messages,t]:[...e.messages];else {const o=r[n],a=new Date(o.updated_at);new Date(t.updated_at)>a&&(t.active?r.splice(n,1,t):r.splice(n,1));}return r.sort((o,a)=>Rq(o.priority,a.priority)),Ri(Ze({},e),{messages:r})},cf=(e,t)=>Kt(e.appetite,t.appetite)?Ri(Ze({},e),{appetite:t.appetite}):e,uf=(e,t)=>{var r;const n=new Date(e.updated_at);return new Date(t.updated_at)>n?Ri(Ze({},e),{appetite:t.appetite,buckets:t.buckets,cloned:t.cloned,copy:(r=t==null?void 0:t.copy)!=null?r:!1,completed_at:t.completed_at,coverage_start_date:t.coverage_start_date,current_bucket_id:t.current_bucket_id,deleted_at:t.deleted_at,features:t.features,finished_quote_requests_count:t.finished_quote_requests_count,locked:t.locked,quote_requests_count:t.quote_requests_count,state:t.state,updated_at:t.updated_at,taken_over:t.taken_over,sent_to_wholesaler_at:t.sent_to_wholesaler_at,received_by_wholesaler_at:t.received_by_wholesaler_at,source:t.source,source_metadata:t.source_metadata,storefront_active_enrolled_product_codes:t.storefront_active_enrolled_product_codes,percent_completed:t.percent_completed,real_time_eligibility:t.real_time_eligibility,wholesaler_preferred_quotes:t.wholesaler_preferred_quotes}):e},lf=(e,t)=>(t.percent_completed!==e.percent_completed&&(e.percent_completed=t.percent_completed),e),df=(e,t)=>(Kt(t,e.eligible_carriers_products)&&(e.eligible_carriers_products=t),e),pf=(e,t)=>(Kt(t,e.real_time_eligibility)&&(e.real_time_eligibility=t),e),{handleMessage:go,receiveAppetite:ff,receiveApplicationForm:Br,receiveApplicationFormProgress:Vi,receiveCarrierProducts:Fi,receiveRealTimeEligibility:vf}=af.actions;var vM=Object.defineProperty,gf=Object.getOwnPropertySymbols,gM=Object.prototype.hasOwnProperty,hM=Object.prototype.propertyIsEnumerable,hf=(e,t,r)=>t in e?vM(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,mf=(e,t)=>{for(var r in t||(t={}))gM.call(t,r)&&hf(e,r,t[r]);if(gf)for(var r of gf(t))hM.call(t,r)&&hf(e,r,t[r]);return e};const mM=[],bf=P({name:"notifications",initialState:mM,reducers:{addNewNotification:(e,{payload:t})=>{if(!t)return e;e.push(mf({id:gd(8)},t));},removeNotification:(e,{payload:t})=>{if(!t)return e;const r=e.findIndex(n=>n.id===t.id);r>-1&&e.splice(r,1);}},extraReducers:e=>{e.addCase(Ae,(t,{payload:r})=>(r.events.forEach(n=>{n.event===x.Notification&&t.push(mf({id:gd(8)},n.data));}),t));}}),{addNewNotification:yf,removeNotification:bM}=bf.actions,yM=[],_f=P({name:"pendingProcesses",initialState:yM,reducers:{receivedApplicationFormPendingProcess:(e,{payload:t})=>{wf(e,t);}},extraReducers:e=>{e.addCase(Br,(t,{payload:r})=>Af(t,r)).addCase(Ae,(t,{payload:r})=>{let n=[...t];return r.events.forEach(o=>{switch(o.event){case x.ApplicationFormProcess:n=wf(n,o.data);break;case x.ApplicationForm:n=Af(n,o.data);break}}),n}),e.addMatcher($(...be),(t,{payload:r})=>{var n;return (n=r.pending_processes)!=null?n:[]});}}),{receivedApplicationFormPendingProcess:$f}=_f.actions,wf=(e,t)=>{if(!t)return [];const r=e.findIndex(n=>n.id===t.id);return r>=0?e[r]=t:e.push(t),e},Af=(e,t)=>{const{pending_processes:r=[]}=t;return r.length?e:[]};var Of=typeof global=="object"&&global&&global.Object===Object&&global,_M=typeof self=="object"&&self&&self.Object===Object&&self,Se=Of||_M||Function("return this")(),rr=Se.Symbol,Sf=Object.prototype,$M=Sf.hasOwnProperty,wM=Sf.toString,Gr=rr?rr.toStringTag:void 0;function AM(e){var t=$M.call(e,Gr),r=e[Gr];try{e[Gr]=void 0;var n=!0;}catch(a){}var o=wM.call(e);return n&&(t?e[Gr]=r:delete e[Gr]),o}var OM=Object.prototype,SM=OM.toString;function jM(e){return SM.call(e)}var CM="[object Null]",PM="[object Undefined]",jf=rr?rr.toStringTag:void 0;function zr(e){return e==null?e===void 0?PM:CM:jf&&jf in Object(e)?AM(e):jM(e)}function Wr(e){return e!=null&&typeof e=="object"}var Qi=Array.isArray;function Hr(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}var EM="[object AsyncFunction]",IM="[object Function]",TM="[object GeneratorFunction]",qM="[object Proxy]";function Cf(e){if(!Hr(e))return !1;var t=zr(e);return t==IM||t==TM||t==EM||t==qM}var Ui=Se["__core-js_shared__"],Pf=function(){var e=/[^.]+$/.exec(Ui&&Ui.keys&&Ui.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function xM(e){return !!Pf&&Pf in e}var kM=Function.prototype,DM=kM.toString;function ht(e){if(e!=null){try{return DM.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var LM=/[\\^$.*+?()[\]{}|]/g,NM=/^\[object .+?Constructor\]$/,RM=Function.prototype,MM=Object.prototype,VM=RM.toString,FM=MM.hasOwnProperty,QM=RegExp("^"+VM.call(FM).replace(LM,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function UM(e){if(!Hr(e)||xM(e))return !1;var t=Cf(e)?QM:NM;return t.test(ht(e))}function BM(e,t){return e==null?void 0:e[t]}function mt(e,t){var r=BM(e,t);return UM(r)?r:void 0}var Bi=mt(Se,"WeakMap"),Ef=Object.create,GM=function(){function e(){}return function(t){if(!Hr(t))return {};if(Ef)return Ef(t);e.prototype=t;var r=new e;return e.prototype=void 0,r}}();function zM(e,t){var r=-1,n=e.length;for(t||(t=Array(n));++r<n;)t[r]=e[r];return t}var If=function(){try{var e=mt(Object,"defineProperty");return e({},"",{}),e}catch(t){}}();function WM(e,t){for(var r=-1,n=e==null?0:e.length;++r<n&&t(e[r],r,e)!==!1;);return e}var HM=9007199254740991,ZM=/^(?:0|[1-9]\d*)$/;function KM(e,t){var r=typeof e;return t=t==null?HM:t,!!t&&(r=="number"||r!="symbol"&&ZM.test(e))&&e>-1&&e%1==0&&e<t}function Tf(e,t,r){t=="__proto__"&&If?If(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r;}function qf(e,t){return e===t||e!==e&&t!==t}var YM=Object.prototype,JM=YM.hasOwnProperty;function xf(e,t,r){var n=e[t];(!(JM.call(e,t)&&qf(n,r))||r===void 0&&!(t in e))&&Tf(e,t,r);}function ho(e,t,r,n){var o=!r;r||(r={});for(var a=-1,i=t.length;++a<i;){var s=t[a],c=n?n(r[s],e[s],s,r,e):void 0;c===void 0&&(c=e[s]),o?Tf(r,s,c):xf(r,s,c);}return r}var XM=9007199254740991;function kf(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=XM}function Df(e){return e!=null&&kf(e.length)&&!Cf(e)}var e2=Object.prototype;function Gi(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||e2;return e===r}function t2(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var r2="[object Arguments]";function Lf(e){return Wr(e)&&zr(e)==r2}var Nf=Object.prototype,n2=Nf.hasOwnProperty,o2=Nf.propertyIsEnumerable,a2=Lf(function(){return arguments}())?Lf:function(e){return Wr(e)&&n2.call(e,"callee")&&!o2.call(e,"callee")};function i2(){return !1}var Rf=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Mf=Rf&&typeof module=="object"&&module&&!module.nodeType&&module,s2=Mf&&Mf.exports===Rf,Vf=s2?Se.Buffer:void 0,c2=Vf?Vf.isBuffer:void 0,Ff=c2||i2,u2="[object Arguments]",l2="[object Array]",d2="[object Boolean]",p2="[object Date]",f2="[object Error]",v2="[object Function]",g2="[object Map]",h2="[object Number]",m2="[object Object]",b2="[object RegExp]",y2="[object Set]",_2="[object String]",$2="[object WeakMap]",w2="[object ArrayBuffer]",A2="[object DataView]",O2="[object Float32Array]",S2="[object Float64Array]",j2="[object Int8Array]",C2="[object Int16Array]",P2="[object Int32Array]",E2="[object Uint8Array]",I2="[object Uint8ClampedArray]",T2="[object Uint16Array]",q2="[object Uint32Array]",B={};B[O2]=B[S2]=B[j2]=B[C2]=B[P2]=B[E2]=B[I2]=B[T2]=B[q2]=!0,B[u2]=B[l2]=B[w2]=B[d2]=B[A2]=B[p2]=B[f2]=B[v2]=B[g2]=B[h2]=B[m2]=B[b2]=B[y2]=B[_2]=B[$2]=!1;function x2(e){return Wr(e)&&kf(e.length)&&!!B[zr(e)]}function zi(e){return function(t){return e(t)}}var Qf=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Zr=Qf&&typeof module=="object"&&module&&!module.nodeType&&module,k2=Zr&&Zr.exports===Qf,Wi=k2&&Of.process,nr=function(){try{var e=Zr&&Zr.require&&Zr.require("util").types;return e||Wi&&Wi.binding&&Wi.binding("util")}catch(t){}}(),Uf=nr&&nr.isTypedArray,D2=Uf?zi(Uf):x2,L2=Object.prototype,N2=L2.hasOwnProperty;function Bf(e,t){var r=Qi(e),n=!r&&a2(e),o=!r&&!n&&Ff(e),a=!r&&!n&&!o&&D2(e),i=r||n||o||a,s=i?t2(e.length,String):[],c=s.length;for(var u in e)(t||N2.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||KM(u,c)))&&s.push(u);return s}function Gf(e,t){return function(r){return e(t(r))}}var R2=Gf(Object.keys,Object),M2=Object.prototype,V2=M2.hasOwnProperty;function F2(e){if(!Gi(e))return R2(e);var t=[];for(var r in Object(e))V2.call(e,r)&&r!="constructor"&&t.push(r);return t}function Hi(e){return Df(e)?Bf(e):F2(e)}function Q2(e){var t=[];if(e!=null)for(var r in Object(e))t.push(r);return t}var U2=Object.prototype,B2=U2.hasOwnProperty;function G2(e){if(!Hr(e))return Q2(e);var t=Gi(e),r=[];for(var n in e)n=="constructor"&&(t||!B2.call(e,n))||r.push(n);return r}function Zi(e){return Df(e)?Bf(e,!0):G2(e)}var Kr=mt(Object,"create");function z2(){this.__data__=Kr?Kr(null):{},this.size=0;}function W2(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var H2="__lodash_hash_undefined__",Z2=Object.prototype,K2=Z2.hasOwnProperty;function Y2(e){var t=this.__data__;if(Kr){var r=t[e];return r===H2?void 0:r}return K2.call(t,e)?t[e]:void 0}var J2=Object.prototype,X2=J2.hasOwnProperty;function eV(e){var t=this.__data__;return Kr?t[e]!==void 0:X2.call(t,e)}var tV="__lodash_hash_undefined__";function rV(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=Kr&&t===void 0?tV:t,this}function bt(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}bt.prototype.clear=z2,bt.prototype.delete=W2,bt.prototype.get=Y2,bt.prototype.has=eV,bt.prototype.set=rV;function nV(){this.__data__=[],this.size=0;}function mo(e,t){for(var r=e.length;r--;)if(qf(e[r][0],t))return r;return -1}var oV=Array.prototype,aV=oV.splice;function iV(e){var t=this.__data__,r=mo(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():aV.call(t,r,1),--this.size,!0}function sV(e){var t=this.__data__,r=mo(t,e);return r<0?void 0:t[r][1]}function cV(e){return mo(this.__data__,e)>-1}function uV(e,t){var r=this.__data__,n=mo(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}function xe(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}xe.prototype.clear=nV,xe.prototype.delete=iV,xe.prototype.get=sV,xe.prototype.has=cV,xe.prototype.set=uV;var Yr=mt(Se,"Map");function lV(){this.size=0,this.__data__={hash:new bt,map:new(Yr||xe),string:new bt};}function dV(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}function bo(e,t){var r=e.__data__;return dV(t)?r[typeof t=="string"?"string":"hash"]:r.map}function pV(e){var t=bo(this,e).delete(e);return this.size-=t?1:0,t}function fV(e){return bo(this,e).get(e)}function vV(e){return bo(this,e).has(e)}function gV(e,t){var r=bo(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}function or(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}or.prototype.clear=lV,or.prototype.delete=pV,or.prototype.get=fV,or.prototype.has=vV,or.prototype.set=gV;function zf(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var Wf=Gf(Object.getPrototypeOf,Object);function hV(){this.__data__=new xe,this.size=0;}function mV(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}function bV(e){return this.__data__.get(e)}function yV(e){return this.__data__.has(e)}var _V=200;function $V(e,t){var r=this.__data__;if(r instanceof xe){var n=r.__data__;if(!Yr||n.length<_V-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new or(n);}return r.set(e,t),this.size=r.size,this}function ar(e){var t=this.__data__=new xe(e);this.size=t.size;}ar.prototype.clear=hV,ar.prototype.delete=mV,ar.prototype.get=bV,ar.prototype.has=yV,ar.prototype.set=$V;function wV(e,t){return e&&ho(t,Hi(t),e)}function AV(e,t){return e&&ho(t,Zi(t),e)}var Hf=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Zf=Hf&&typeof module=="object"&&module&&!module.nodeType&&module,OV=Zf&&Zf.exports===Hf,Kf=OV?Se.Buffer:void 0,Yf=Kf?Kf.allocUnsafe:void 0;function SV(e,t){if(t)return e.slice();var r=e.length,n=Yf?Yf(r):new e.constructor(r);return e.copy(n),n}function jV(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}function Jf(){return []}var CV=Object.prototype,PV=CV.propertyIsEnumerable,Xf=Object.getOwnPropertySymbols,Ki=Xf?function(e){return e==null?[]:(e=Object(e),jV(Xf(e),function(t){return PV.call(e,t)}))}:Jf;function EV(e,t){return ho(e,Ki(e),t)}var IV=Object.getOwnPropertySymbols,ev=IV?function(e){for(var t=[];e;)zf(t,Ki(e)),e=Wf(e);return t}:Jf;function TV(e,t){return ho(e,ev(e),t)}function tv(e,t,r){var n=t(e);return Qi(e)?n:zf(n,r(e))}function qV(e){return tv(e,Hi,Ki)}function xV(e){return tv(e,Zi,ev)}var Yi=mt(Se,"DataView"),Ji=mt(Se,"Promise"),Xi=mt(Se,"Set"),rv="[object Map]",kV="[object Object]",nv="[object Promise]",ov="[object Set]",av="[object WeakMap]",iv="[object DataView]",DV=ht(Yi),LV=ht(Yr),NV=ht(Ji),RV=ht(Xi),MV=ht(Bi),yt=zr;(Yi&&yt(new Yi(new ArrayBuffer(1)))!=iv||Yr&&yt(new Yr)!=rv||Ji&&yt(Ji.resolve())!=nv||Xi&&yt(new Xi)!=ov||Bi&&yt(new Bi)!=av)&&(yt=function(e){var t=zr(e),r=t==kV?e.constructor:void 0,n=r?ht(r):"";if(n)switch(n){case DV:return iv;case LV:return rv;case NV:return nv;case RV:return ov;case MV:return av}return t});var es=yt,VV=Object.prototype,FV=VV.hasOwnProperty;function QV(e){var t=e.length,r=new e.constructor(t);return t&&typeof e[0]=="string"&&FV.call(e,"index")&&(r.index=e.index,r.input=e.input),r}var sv=Se.Uint8Array;function ts(e){var t=new e.constructor(e.byteLength);return new sv(t).set(new sv(e)),t}function UV(e,t){var r=t?ts(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.byteLength)}var BV=/\w*$/;function GV(e){var t=new e.constructor(e.source,BV.exec(e));return t.lastIndex=e.lastIndex,t}var cv=rr?rr.prototype:void 0,uv=cv?cv.valueOf:void 0;function zV(e){return uv?Object(uv.call(e)):{}}function WV(e,t){var r=t?ts(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.length)}var HV="[object Boolean]",ZV="[object Date]",KV="[object Map]",YV="[object Number]",JV="[object RegExp]",XV="[object Set]",eF="[object String]",tF="[object Symbol]",rF="[object ArrayBuffer]",nF="[object DataView]",oF="[object Float32Array]",aF="[object Float64Array]",iF="[object Int8Array]",sF="[object Int16Array]",cF="[object Int32Array]",uF="[object Uint8Array]",lF="[object Uint8ClampedArray]",dF="[object Uint16Array]",pF="[object Uint32Array]";function fF(e,t,r){var n=e.constructor;switch(t){case rF:return ts(e);case HV:case ZV:return new n(+e);case nF:return UV(e,r);case oF:case aF:case iF:case sF:case cF:case uF:case lF:case dF:case pF:return WV(e,r);case KV:return new n;case YV:case eF:return new n(e);case JV:return GV(e);case XV:return new n;case tF:return zV(e)}}function vF(e){return typeof e.constructor=="function"&&!Gi(e)?GM(Wf(e)):{}}var gF="[object Map]";function hF(e){return Wr(e)&&es(e)==gF}var lv=nr&&nr.isMap,mF=lv?zi(lv):hF,bF="[object Set]";function yF(e){return Wr(e)&&es(e)==bF}var dv=nr&&nr.isSet,_F=dv?zi(dv):yF,$F=1,wF=2,AF=4,pv="[object Arguments]",OF="[object Array]",SF="[object Boolean]",jF="[object Date]",CF="[object Error]",fv="[object Function]",PF="[object GeneratorFunction]",EF="[object Map]",IF="[object Number]",vv="[object Object]",TF="[object RegExp]",qF="[object Set]",xF="[object String]",kF="[object Symbol]",DF="[object WeakMap]",LF="[object ArrayBuffer]",NF="[object DataView]",RF="[object Float32Array]",MF="[object Float64Array]",VF="[object Int8Array]",FF="[object Int16Array]",QF="[object Int32Array]",UF="[object Uint8Array]",BF="[object Uint8ClampedArray]",GF="[object Uint16Array]",zF="[object Uint32Array]",V={};V[pv]=V[OF]=V[LF]=V[NF]=V[SF]=V[jF]=V[RF]=V[MF]=V[VF]=V[FF]=V[QF]=V[EF]=V[IF]=V[vv]=V[TF]=V[qF]=V[xF]=V[kF]=V[UF]=V[BF]=V[GF]=V[zF]=!0,V[CF]=V[fv]=V[DF]=!1;function yo(e,t,r,n,o,a){var i,s=t&$F,c=t&wF,u=t&AF;if(r&&(i=o?r(e,n,o,a):r(e)),i!==void 0)return i;if(!Hr(e))return e;var l=Qi(e);if(l){if(i=QV(e),!s)return zM(e,i)}else {var d=es(e),f=d==fv||d==PF;if(Ff(e))return SV(e,s);if(d==vv||d==pv||f&&!o){if(i=c||f?{}:vF(e),!s)return c?TV(e,AV(i,e)):EV(e,wV(i,e))}else {if(!V[d])return o?e:{};i=fF(e,d,s);}}a||(a=new ar);var g=a.get(e);if(g)return g;a.set(e,i),_F(e)?e.forEach(function(b){i.add(yo(b,t,r,b,e,a));}):mF(e)&&e.forEach(function(b,m){i.set(m,yo(b,t,r,m,e,a));});var v=u?c?xV:qV:c?Zi:Hi,h=l?void 0:v(e);return WM(h||e,function(b,m){h&&(m=b,b=e[m]),xf(i,m,yo(b,t,r,m,e,a));}),i}var WF=1,HF=4;function rs(e){return yo(e,WF|HF)}const ZF=(e,t)=>(e.parentQuestionSets=tQ(t,e.parentQuestionSets),e.list=JF(e.list,t),e),KF=e=>e.filter(t=>gv(t)),gv=e=>{var t;return (t=e==null?void 0:e.question_set.question_groups)==null?void 0:t.some(r=>kd(r))},YF=(e,t)=>{const r=e;return t.forEach(n=>{const o=bv(n),a=wv(e,n.order);r.splice(a,1,...o);}),r},hv=e=>!!e.parentQuestionSetId,mv=(e,t,r)=>{const n=r.list.filter(o=>o.parentQuestionSetId===e.id).filter(o=>!o.is_blank);return n.length&&n.findIndex(o=>o.question_set_id===t.question_set_id)===n.length-1?void 0:t};function JF(e,t){const r=e.filter(a=>a.parentQuestionSetId!==t.id),n=bv(t),o=wv(r,t.order);return r.splice(o,0,...n),r}function bv(e){const t=cD(e,iD(e));return Array.from(t.values()).filter(r=>kd(r)).flatMap(r=>Array.from(r.nestedGroupsWithAnswers.values()).filter(n=>pD(n)).sort((n,o)=>fD(n,o)).map(n=>XF(n,e)))}function XF(e,t){const r=t.question_set.question_groups.find(a=>a.id===e.groupId),n=yv(e),o={answers:n,application_form_id:t.application_form_id,completed_at:null,created_at:t.created_at,id:e.groupId,is_active:t.is_active,is_blank:eQ(n,t.question_set.question_pools),next_active_question_set_id:t.next_active_question_set_id,order:t.order,question_set:{code:e.parentQuestionAnswer.question.code,created_at:t.question_set.created_at,id:e.parentQuestionAnswer.id,name:e.label,owner_id:t.question_set.owner_id,question_groups:_v(t.question_set.question_groups,r),question_ids:$v(t.question_set.question_groups,r),question_pools:t.question_set.question_pools,subdomain:t.question_set.subdomain,tenant_id:t.question_set.tenant_id,title:e.label,updated_at:t.updated_at},question_set_id:e.parentQuestionAnswer.id,updated_at:t.updated_at,parentQuestionSetId:t.id};return o.is_completed=wi(o),o}function yv(e){let t=e.answers;return e.nestedGroupsWithAnswers.forEach(r=>{t=[...t,...yv(r)];}),t.sort(mi)}function _v(e,t){if(!t)return [];let r=[];return e.forEach(n=>{t.question_ids.includes(n.parent_question_id)&&(r.push(n),r=[...r,..._v(e,n)]);}),r}function $v(e,t){if(!t)return [];let r=t==null?void 0:t.question_ids;return e.forEach(n=>{t.question_ids.includes(n.parent_question_id)&&(r=[...r,...$v(e,n)]);}),r}function eQ(e,t){return e.length>0?e.every(r=>r.question.field_type===C.Anchor)?e.find(r=>t.find(n=>n.parent_question_id===r.question.id))===void 0:!1:!0}function wv(e,t){let r=e.findIndex(n=>n.order>=t);return r===-1&&(r=e.length),r}function tQ(e,t){return [...t.filter(r=>r.question_set_id!==e.question_set_id),e]}const _o=(e,t,r)=>{if(r){const n=e.list.findIndex(o=>o.question_set_id===r.question_set_id);return t==="backward"?rQ(e.list,n):jv(e.list,n)}return ir(e.list)},Av=(e,t,r)=>t?_o(e,t,r||void 0):ir(e.list),Ov=(e,t,r,n)=>{const o=t.next_active_question_set_id;return o&&!n?$o(e,o):r?_o(e,r,n||t):ir(e.list)},ir=e=>{const t=e.filter(n=>Jr(n));if(t.length===0)return "";const r=t.find(n=>!n.is_completed);if(r)return r.question_set_id;{const n=e.filter(Jr);return n[n.length-1].question_set_id}};function $o(e,t){var r;const n=e.list.findIndex(o=>o.id===t);if(n!==-1)return e.list[n].is_blank?jv(e.list,n):e.list[n].question_set_id;if(e.parentQuestionSets.find(o=>o.id===t))return (r=e.list.filter(Jr).find(o=>o.parentQuestionSetId===t))==null?void 0:r.question_set_id}function Sv(e,t){const r=e.list.find(n=>n.question_set_id===e.active);return r&&!r.next_active_question_set_id&&r.is_completed&&r.order+1===t.order?$o(e,t.id):e.active}function Jr(e){return e.is_active&&!e.is_blank&&e.answers.length>0}function rQ(e,t){if(t<=0)return e[0].question_set_id;let r;for(let n=t-1;n>=0;n--)if(Jr(e[n])){r=e[n].question_set_id;break}return r!=null?r:e[t].question_set_id}function jv(e,t){if(t>=e.length-1)return e[e.length-1].question_set_id;let r;for(let n=t+1;n<e.length;n++)if(Jr(e[n])){r=e[n].question_set_id;break}return r!=null?r:e[t].question_set_id}var wo=typeof globalThis!="undefined"?globalThis:typeof window!="undefined"?window:typeof global!="undefined"?global:typeof self!="undefined"?self:{};function nQ(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}function oQ(){this.__data__=[],this.size=0;}var aQ=oQ;function iQ(e,t){return e===t||e!==e&&t!==t}var Cv=iQ,sQ=Cv;function cQ(e,t){for(var r=e.length;r--;)if(sQ(e[r][0],t))return r;return -1}var Ao=cQ,uQ=Ao,lQ=Array.prototype,dQ=lQ.splice;function pQ(e){var t=this.__data__,r=uQ(t,e);if(r<0)return !1;var n=t.length-1;return r==n?t.pop():dQ.call(t,r,1),--this.size,!0}var fQ=pQ,vQ=Ao;function gQ(e){var t=this.__data__,r=vQ(t,e);return r<0?void 0:t[r][1]}var hQ=gQ,mQ=Ao;function bQ(e){return mQ(this.__data__,e)>-1}var yQ=bQ,_Q=Ao;function $Q(e,t){var r=this.__data__,n=_Q(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}var wQ=$Q,AQ=aQ,OQ=fQ,SQ=hQ,jQ=yQ,CQ=wQ;function sr(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}sr.prototype.clear=AQ,sr.prototype.delete=OQ,sr.prototype.get=SQ,sr.prototype.has=jQ,sr.prototype.set=CQ;var Oo=sr,PQ=Oo;function EQ(){this.__data__=new PQ,this.size=0;}var IQ=EQ;function TQ(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}var qQ=TQ;function xQ(e){return this.__data__.get(e)}var kQ=xQ;function DQ(e){return this.__data__.has(e)}var LQ=DQ,NQ=typeof wo=="object"&&wo&&wo.Object===Object&&wo,Pv=NQ,RQ=Pv,MQ=typeof self=="object"&&self&&self.Object===Object&&self,VQ=RQ||MQ||Function("return this")(),je=VQ,FQ=je,QQ=FQ.Symbol,ns=QQ,Ev=ns,Iv=Object.prototype,UQ=Iv.hasOwnProperty,BQ=Iv.toString,Xr=Ev?Ev.toStringTag:void 0;function GQ(e){var t=UQ.call(e,Xr),r=e[Xr];try{e[Xr]=void 0;var n=!0;}catch(a){}var o=BQ.call(e);return n&&(t?e[Xr]=r:delete e[Xr]),o}var zQ=GQ,WQ=Object.prototype,HQ=WQ.toString;function ZQ(e){return HQ.call(e)}var KQ=ZQ,Tv=ns,YQ=zQ,JQ=KQ,XQ="[object Null]",eU="[object Undefined]",qv=Tv?Tv.toStringTag:void 0;function tU(e){return e==null?e===void 0?eU:XQ:qv&&qv in Object(e)?YQ(e):JQ(e)}var So=tU;function rU(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}var en=rU,nU=So,oU=en,aU="[object AsyncFunction]",iU="[object Function]",sU="[object GeneratorFunction]",cU="[object Proxy]";function uU(e){if(!oU(e))return !1;var t=nU(e);return t==iU||t==sU||t==aU||t==cU}var xv=uU,lU=je,dU=lU["__core-js_shared__"],pU=dU,os=pU,kv=function(){var e=/[^.]+$/.exec(os&&os.keys&&os.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function fU(e){return !!kv&&kv in e}var vU=fU,gU=Function.prototype,hU=gU.toString;function mU(e){if(e!=null){try{return hU.call(e)}catch(t){}try{return e+""}catch(t){}}return ""}var Dv=mU,bU=xv,yU=vU,_U=en,$U=Dv,wU=/[\\^$.*+?()[\]{}|]/g,AU=/^\[object .+?Constructor\]$/,OU=Function.prototype,SU=Object.prototype,jU=OU.toString,CU=SU.hasOwnProperty,PU=RegExp("^"+jU.call(CU).replace(wU,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function EU(e){if(!_U(e)||yU(e))return !1;var t=bU(e)?PU:AU;return t.test($U(e))}var IU=EU;function TU(e,t){return e==null?void 0:e[t]}var qU=TU,xU=IU,kU=qU;function DU(e,t){var r=kU(e,t);return xU(r)?r:void 0}var _t=DU,LU=_t,NU=je,RU=LU(NU,"Map"),as=RU,MU=_t,VU=MU(Object,"create"),jo=VU,Lv=jo;function FU(){this.__data__=Lv?Lv(null):{},this.size=0;}var QU=FU;function UU(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var BU=UU,GU=jo,zU="__lodash_hash_undefined__",WU=Object.prototype,HU=WU.hasOwnProperty;function ZU(e){var t=this.__data__;if(GU){var r=t[e];return r===zU?void 0:r}return HU.call(t,e)?t[e]:void 0}var KU=ZU,YU=jo,JU=Object.prototype,XU=JU.hasOwnProperty;function eB(e){var t=this.__data__;return YU?t[e]!==void 0:XU.call(t,e)}var tB=eB,rB=jo,nB="__lodash_hash_undefined__";function oB(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=rB&&t===void 0?nB:t,this}var aB=oB,iB=QU,sB=BU,cB=KU,uB=tB,lB=aB;function cr(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}cr.prototype.clear=iB,cr.prototype.delete=sB,cr.prototype.get=cB,cr.prototype.has=uB,cr.prototype.set=lB;var dB=cr,Nv=dB,pB=Oo,fB=as;function vB(){this.size=0,this.__data__={hash:new Nv,map:new(fB||pB),string:new Nv};}var gB=vB;function hB(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}var mB=hB,bB=mB;function yB(e,t){var r=e.__data__;return bB(t)?r[typeof t=="string"?"string":"hash"]:r.map}var Co=yB,_B=Co;function $B(e){var t=_B(this,e).delete(e);return this.size-=t?1:0,t}var wB=$B,AB=Co;function OB(e){return AB(this,e).get(e)}var SB=OB,jB=Co;function CB(e){return jB(this,e).has(e)}var PB=CB,EB=Co;function IB(e,t){var r=EB(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}var TB=IB,qB=gB,xB=wB,kB=SB,DB=PB,LB=TB;function ur(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1]);}}ur.prototype.clear=qB,ur.prototype.delete=xB,ur.prototype.get=kB,ur.prototype.has=DB,ur.prototype.set=LB;var NB=ur,RB=Oo,MB=as,VB=NB,FB=200;function QB(e,t){var r=this.__data__;if(r instanceof RB){var n=r.__data__;if(!MB||n.length<FB-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new VB(n);}return r.set(e,t),this.size=r.size,this}var UB=QB,BB=Oo,GB=IQ,zB=qQ,WB=kQ,HB=LQ,ZB=UB;function lr(e){var t=this.__data__=new BB(e);this.size=t.size;}lr.prototype.clear=GB,lr.prototype.delete=zB,lr.prototype.get=WB,lr.prototype.has=HB,lr.prototype.set=ZB;var KB=lr;function YB(e,t){for(var r=-1,n=e==null?0:e.length;++r<n&&t(e[r],r,e)!==!1;);return e}var JB=YB,XB=_t,e9=function(){try{var e=XB(Object,"defineProperty");return e({},"",{}),e}catch(t){}}(),t9=e9,Rv=t9;function r9(e,t,r){t=="__proto__"&&Rv?Rv(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r;}var Mv=r9,n9=Mv,o9=Cv,a9=Object.prototype,i9=a9.hasOwnProperty;function s9(e,t,r){var n=e[t];(!(i9.call(e,t)&&o9(n,r))||r===void 0&&!(t in e))&&n9(e,t,r);}var Vv=s9,c9=Vv,u9=Mv;function l9(e,t,r,n){var o=!r;r||(r={});for(var a=-1,i=t.length;++a<i;){var s=t[a],c=n?n(r[s],e[s],s,r,e):void 0;c===void 0&&(c=e[s]),o?u9(r,s,c):c9(r,s,c);}return r}var Po=l9;function d9(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var p9=d9;function f9(e){return e!=null&&typeof e=="object"}var tn=f9,v9=So,g9=tn,h9="[object Arguments]";function m9(e){return g9(e)&&v9(e)==h9}var b9=m9,Fv=b9,y9=tn,Qv=Object.prototype,_9=Qv.hasOwnProperty,$9=Qv.propertyIsEnumerable,w9=Fv(function(){return arguments}())?Fv:function(e){return y9(e)&&_9.call(e,"callee")&&!$9.call(e,"callee")},A9=w9,O9=Array.isArray,is=O9,Eo={exports:{}};function S9(){return !1}var j9=S9;(function(e,t){var r=je,n=j9,o=t&&!t.nodeType&&t,a=o&&!0&&e&&!e.nodeType&&e,i=a&&a.exports===o,s=i?r.Buffer:void 0,c=s?s.isBuffer:void 0,u=c||n;e.exports=u;})(Eo,Eo.exports);var Uv=Eo.exports,C9=9007199254740991,P9=/^(?:0|[1-9]\d*)$/;function E9(e,t){var r=typeof e;return t=t==null?C9:t,!!t&&(r=="number"||r!="symbol"&&P9.test(e))&&e>-1&&e%1==0&&e<t}var I9=E9,T9=9007199254740991;function q9(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=T9}var Bv=q9,x9=So,k9=Bv,D9=tn,L9="[object Arguments]",N9="[object Array]",R9="[object Boolean]",M9="[object Date]",V9="[object Error]",F9="[object Function]",Q9="[object Map]",U9="[object Number]",B9="[object Object]",G9="[object RegExp]",z9="[object Set]",W9="[object String]",H9="[object WeakMap]",Z9="[object ArrayBuffer]",K9="[object DataView]",Y9="[object Float32Array]",J9="[object Float64Array]",X9="[object Int8Array]",e3="[object Int16Array]",t3="[object Int32Array]",r3="[object Uint8Array]",n3="[object Uint8ClampedArray]",o3="[object Uint16Array]",a3="[object Uint32Array]",G={};G[Y9]=G[J9]=G[X9]=G[e3]=G[t3]=G[r3]=G[n3]=G[o3]=G[a3]=!0,G[L9]=G[N9]=G[Z9]=G[R9]=G[K9]=G[M9]=G[V9]=G[F9]=G[Q9]=G[U9]=G[B9]=G[G9]=G[z9]=G[W9]=G[H9]=!1;function i3(e){return D9(e)&&k9(e.length)&&!!G[x9(e)]}var s3=i3;function c3(e){return function(t){return e(t)}}var ss=c3,Io={exports:{}};(function(e,t){var r=Pv,n=t&&!t.nodeType&&t,o=n&&!0&&e&&!e.nodeType&&e,a=o&&o.exports===n,i=a&&r.process,s=function(){try{var c=o&&o.require&&o.require("util").types;return c||i&&i.binding&&i.binding("util")}catch(u){}}();e.exports=s;})(Io,Io.exports);var cs=Io.exports,u3=s3,l3=ss,Gv=cs,zv=Gv&&Gv.isTypedArray,d3=zv?l3(zv):u3,p3=d3,f3=p9,v3=A9,g3=is,h3=Uv,m3=I9,b3=p3,y3=Object.prototype,_3=y3.hasOwnProperty;function $3(e,t){var r=g3(e),n=!r&&v3(e),o=!r&&!n&&h3(e),a=!r&&!n&&!o&&b3(e),i=r||n||o||a,s=i?f3(e.length,String):[],c=s.length;for(var u in e)(t||_3.call(e,u))&&!(i&&(u=="length"||o&&(u=="offset"||u=="parent")||a&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||m3(u,c)))&&s.push(u);return s}var Wv=$3,w3=Object.prototype;function A3(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||w3;return e===r}var us=A3;function O3(e,t){return function(r){return e(t(r))}}var Hv=O3,S3=Hv,j3=S3(Object.keys,Object),C3=j3,P3=us,E3=C3,I3=Object.prototype,T3=I3.hasOwnProperty;function q3(e){if(!P3(e))return E3(e);var t=[];for(var r in Object(e))T3.call(e,r)&&r!="constructor"&&t.push(r);return t}var x3=q3,k3=xv,D3=Bv;function L3(e){return e!=null&&D3(e.length)&&!k3(e)}var Zv=L3,N3=Wv,R3=x3,M3=Zv;function V3(e){return M3(e)?N3(e):R3(e)}var ls=V3,F3=Po,Q3=ls;function U3(e,t){return e&&F3(t,Q3(t),e)}var B3=U3;function G3(e){var t=[];if(e!=null)for(var r in Object(e))t.push(r);return t}var z3=G3,W3=en,H3=us,Z3=z3,K3=Object.prototype,Y3=K3.hasOwnProperty;function J3(e){if(!W3(e))return Z3(e);var t=H3(e),r=[];for(var n in e)n=="constructor"&&(t||!Y3.call(e,n))||r.push(n);return r}var X3=J3,eG=Wv,tG=X3,rG=Zv;function nG(e){return rG(e)?eG(e,!0):tG(e)}var ds=nG,oG=Po,aG=ds;function iG(e,t){return e&&oG(t,aG(t),e)}var sG=iG,To={exports:{}};(function(e,t){var r=je,n=t&&!t.nodeType&&t,o=n&&!0&&e&&!e.nodeType&&e,a=o&&o.exports===n,i=a?r.Buffer:void 0,s=i?i.allocUnsafe:void 0;function c(u,l){if(l)return u.slice();var d=u.length,f=s?s(d):new u.constructor(d);return u.copy(f),f}e.exports=c;})(To,To.exports);var cG=To.exports;function uG(e,t){var r=-1,n=e.length;for(t||(t=Array(n));++r<n;)t[r]=e[r];return t}var lG=uG;function dG(e,t){for(var r=-1,n=e==null?0:e.length,o=0,a=[];++r<n;){var i=e[r];t(i,r,e)&&(a[o++]=i);}return a}var pG=dG;function fG(){return []}var Kv=fG,vG=pG,gG=Kv,hG=Object.prototype,mG=hG.propertyIsEnumerable,Yv=Object.getOwnPropertySymbols,bG=Yv?function(e){return e==null?[]:(e=Object(e),vG(Yv(e),function(t){return mG.call(e,t)}))}:gG,ps=bG,yG=Po,_G=ps;function $G(e,t){return yG(e,_G(e),t)}var wG=$G;function AG(e,t){for(var r=-1,n=t.length,o=e.length;++r<n;)e[o+r]=t[r];return e}var Jv=AG,OG=Hv,SG=OG(Object.getPrototypeOf,Object),Xv=SG,jG=Jv,CG=Xv,PG=ps,EG=Kv,IG=Object.getOwnPropertySymbols,TG=IG?function(e){for(var t=[];e;)jG(t,PG(e)),e=CG(e);return t}:EG,eg=TG,qG=Po,xG=eg;function kG(e,t){return qG(e,xG(e),t)}var DG=kG,LG=Jv,NG=is;function RG(e,t,r){var n=t(e);return NG(e)?n:LG(n,r(e))}var tg=RG,MG=tg,VG=ps,FG=ls;function QG(e){return MG(e,FG,VG)}var UG=QG,BG=tg,GG=eg,zG=ds;function WG(e){return BG(e,zG,GG)}var HG=WG,ZG=_t,KG=je,YG=ZG(KG,"DataView"),JG=YG,XG=_t,ez=je,tz=XG(ez,"Promise"),rz=tz,nz=_t,oz=je,az=nz(oz,"Set"),iz=az,sz=_t,cz=je,uz=sz(cz,"WeakMap"),lz=uz,fs=JG,vs=as,gs=rz,hs=iz,ms=lz,rg=So,dr=Dv,ng="[object Map]",dz="[object Object]",og="[object Promise]",ag="[object Set]",ig="[object WeakMap]",sg="[object DataView]",pz=dr(fs),fz=dr(vs),vz=dr(gs),gz=dr(hs),hz=dr(ms),$t=rg;(fs&&$t(new fs(new ArrayBuffer(1)))!=sg||vs&&$t(new vs)!=ng||gs&&$t(gs.resolve())!=og||hs&&$t(new hs)!=ag||ms&&$t(new ms)!=ig)&&($t=function(e){var t=rg(e),r=t==dz?e.constructor:void 0,n=r?dr(r):"";if(n)switch(n){case pz:return sg;case fz:return ng;case vz:return og;case gz:return ag;case hz:return ig}return t});var bs=$t,mz=Object.prototype,bz=mz.hasOwnProperty;function yz(e){var t=e.length,r=new e.constructor(t);return t&&typeof e[0]=="string"&&bz.call(e,"index")&&(r.index=e.index,r.input=e.input),r}var _z=yz,$z=je,wz=$z.Uint8Array,Az=wz,cg=Az;function Oz(e){var t=new e.constructor(e.byteLength);return new cg(t).set(new cg(e)),t}var ys=Oz,Sz=ys;function jz(e,t){var r=t?Sz(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.byteLength)}var Cz=jz,Pz=/\w*$/;function Ez(e){var t=new e.constructor(e.source,Pz.exec(e));return t.lastIndex=e.lastIndex,t}var Iz=Ez,ug=ns,lg=ug?ug.prototype:void 0,dg=lg?lg.valueOf:void 0;function Tz(e){return dg?Object(dg.call(e)):{}}var qz=Tz,xz=ys;function kz(e,t){var r=t?xz(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.length)}var Dz=kz,Lz=ys,Nz=Cz,Rz=Iz,Mz=qz,Vz=Dz,Fz="[object Boolean]",Qz="[object Date]",Uz="[object Map]",Bz="[object Number]",Gz="[object RegExp]",zz="[object Set]",Wz="[object String]",Hz="[object Symbol]",Zz="[object ArrayBuffer]",Kz="[object DataView]",Yz="[object Float32Array]",Jz="[object Float64Array]",Xz="[object Int8Array]",e4="[object Int16Array]",t4="[object Int32Array]",r4="[object Uint8Array]",n4="[object Uint8ClampedArray]",o4="[object Uint16Array]",a4="[object Uint32Array]";function i4(e,t,r){var n=e.constructor;switch(t){case Zz:return Lz(e);case Fz:case Qz:return new n(+e);case Kz:return Nz(e,r);case Yz:case Jz:case Xz:case e4:case t4:case r4:case n4:case o4:case a4:return Vz(e,r);case Uz:return new n;case Bz:case Wz:return new n(e);case Gz:return Rz(e);case zz:return new n;case Hz:return Mz(e)}}var s4=i4,c4=en,pg=Object.create,u4=function(){function e(){}return function(t){if(!c4(t))return {};if(pg)return pg(t);e.prototype=t;var r=new e;return e.prototype=void 0,r}}(),l4=u4,d4=l4,p4=Xv,f4=us;function v4(e){return typeof e.constructor=="function"&&!f4(e)?d4(p4(e)):{}}var g4=v4,h4=bs,m4=tn,b4="[object Map]";function y4(e){return m4(e)&&h4(e)==b4}var _4=y4,$4=_4,w4=ss,fg=cs,vg=fg&&fg.isMap,A4=vg?w4(vg):$4,O4=A4,S4=bs,j4=tn,C4="[object Set]";function P4(e){return j4(e)&&S4(e)==C4}var E4=P4,I4=E4,T4=ss,gg=cs,hg=gg&&gg.isSet,q4=hg?T4(hg):I4,x4=q4,k4=KB,D4=JB,L4=Vv,N4=B3,R4=sG,M4=cG,V4=lG,F4=wG,Q4=DG,U4=UG,B4=HG,G4=bs,z4=_z,W4=s4,H4=g4,Z4=is,K4=Uv,Y4=O4,J4=en,X4=x4,eW=ls,tW=ds,rW=1,nW=2,oW=4,mg="[object Arguments]",aW="[object Array]",iW="[object Boolean]",sW="[object Date]",cW="[object Error]",bg="[object Function]",uW="[object GeneratorFunction]",lW="[object Map]",dW="[object Number]",yg="[object Object]",pW="[object RegExp]",fW="[object Set]",vW="[object String]",gW="[object Symbol]",hW="[object WeakMap]",mW="[object ArrayBuffer]",bW="[object DataView]",yW="[object Float32Array]",_W="[object Float64Array]",$W="[object Int8Array]",wW="[object Int16Array]",AW="[object Int32Array]",OW="[object Uint8Array]",SW="[object Uint8ClampedArray]",jW="[object Uint16Array]",CW="[object Uint32Array]",F={};F[mg]=F[aW]=F[mW]=F[bW]=F[iW]=F[sW]=F[yW]=F[_W]=F[$W]=F[wW]=F[AW]=F[lW]=F[dW]=F[yg]=F[pW]=F[fW]=F[vW]=F[gW]=F[OW]=F[SW]=F[jW]=F[CW]=!0,F[cW]=F[bg]=F[hW]=!1;function qo(e,t,r,n,o,a){var i,s=t&rW,c=t&nW,u=t&oW;if(r&&(i=o?r(e,n,o,a):r(e)),i!==void 0)return i;if(!J4(e))return e;var l=Z4(e);if(l){if(i=z4(e),!s)return V4(e,i)}else {var d=G4(e),f=d==bg||d==uW;if(K4(e))return M4(e,s);if(d==yg||d==mg||f&&!o){if(i=c||f?{}:H4(e),!s)return c?Q4(e,R4(i,e)):F4(e,N4(i,e))}else {if(!F[d])return o?e:{};i=W4(e,d,s);}}a||(a=new k4);var g=a.get(e);if(g)return g;a.set(e,i),X4(e)?e.forEach(function(b){i.add(qo(b,t,r,b,e,a));}):Y4(e)&&e.forEach(function(b,m){i.set(m,qo(b,t,r,m,e,a));});var v=u?c?B4:U4:c?tW:eW,h=l?void 0:v(e);return D4(h||e,function(b,m){h&&(m=b,b=e[m]),L4(i,m,qo(b,t,r,m,e,a));}),i}var PW=qo,EW=PW,IW=1,TW=4;function qW(e){return EW(e,IW|TW)}var xW=qW,kW=nQ(xW),DW=Object.defineProperty,LW=Object.defineProperties,NW=Object.getOwnPropertyDescriptors,_g=Object.getOwnPropertySymbols,RW=Object.prototype.hasOwnProperty,MW=Object.prototype.propertyIsEnumerable,$g=(e,t,r)=>t in e?DW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,VW=(e,t)=>{for(var r in t||(t={}))RW.call(t,r)&&$g(e,r,t[r]);if(_g)for(var r of _g(t))MW.call(t,r)&&$g(e,r,t[r]);return e},FW=(e,t)=>LW(e,NW(t)),QW=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});function wg(e){return FW(VW({},kW(e)),{updated_at:new Date().toISOString()})}const ee=L("answers/Update",(e,t)=>QW(void 0,[e,t],function*(r,{getState:n,rejectWithValue:o,dispatch:a}){try{const i=n(),s=$i(r.answer,Nd(r.answer));return i.queues.answers.outbound[r.answer.id]?s.length?wg(r.answer):(yield D$.updateAnswer(r.answer,r.config)).parsedBody:(a(On(r.answer)),wg(r.answer))}catch(i){const s=new D("Error updating answer",i);throw R(s),o(s)}}));var UW=Object.defineProperty,BW=Object.defineProperties,GW=Object.getOwnPropertyDescriptors,Ag=Object.getOwnPropertySymbols,zW=Object.prototype.hasOwnProperty,WW=Object.prototype.propertyIsEnumerable,Og=(e,t,r)=>t in e?UW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Sg=(e,t)=>{for(var r in t||(t={}))zW.call(t,r)&&Og(e,r,t[r]);if(Ag)for(var r of Ag(t))WW.call(t,r)&&Og(e,r,t[r]);return e},jg=(e,t)=>BW(e,GW(t)),_s=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const $s="viewState/AddressValidation",rn=L(`${$s}/validateAddress`,(e,t)=>_s(void 0,[e,t],function*(r,{getState:n,rejectWithValue:o}){var a;const i=`https://addressvalidation.googleapis.com/v1:validateAddress?key=${r.googleApiKey}`,s={address:{addressLines:[r.street_1,r.street_2,r.city,r.state,r.zipcode],regionCode:"US"},previousResponseId:""},c=n();s.previousResponseId=(a=Kp(c,r.addressValidationId).responseId)!=null?a:"";try{const u=yield fetch(i,{method:"POST",headers:{"Content-Type":"application/json"},body:JSON.stringify(s)});if(!u.ok)throw new Error("There was an error with this request");const l=yield u.json();return l.addressValidationId=r.addressValidationId,l.originalAddressComponents={street_1:r.street_1,street_2:r.street_2,city:r.city,state:r.state,zipcode:r.zipcode},l}catch(u){const l=new D("Error validating address",u);throw o(l)}})),HW=L(`${$s}/saveValidatedAddressAnswers`,(e,t)=>_s(void 0,[e,t],function*(r,{dispatch:n,rejectWithValue:o}){try{const a=ZW(r.addressComponents),i=yield Promise.allSettled(Object.entries(a).map(([c,u])=>{const l=r.answerMap[c].question.reference_type==="state"?jg(Sg({},r.answerMap[c]),{choice_id:KW(u,r.answerMap[c])}):jg(Sg({},r.answerMap[c]),{value:u});return n(ee({answer:l,config:r.config}))})),s=i.filter(Pg);return i.filter(Cg).map(c=>{throw new Error(c.reason)}),s.map(c=>c.value.payload)}catch(a){const i=new D("Error saving address",a);throw R(i),o(i)}})),ws=L(`${$s}/saveAddressValidationMissingAnswers`,(e,t)=>_s(void 0,[e,t],function*(r,{rejectWithValue:n,dispatch:o}){try{const a=yield Promise.allSettled(r.answersToSave.map(s=>o(ee({answer:s,config:r.config})))),i=a.filter(Pg);return a.filter(Cg).map(s=>{throw new Error(s.reason)}),{addressValidationId:r.addressValidationId,answers:i.map(s=>s.value.payload)}}catch(a){const i=new D("Error saving address",a);throw R(i),n(i)}})),Cg=e=>e.status==="rejected",Pg=e=>e.status==="fulfilled",ZW=e=>{const t=e.filter(r=>(r.inferred||r.replaced||r.spellCorrected)&&r.componentType!==Xe.POSTAL_CODE_SUFFIX);return Object.fromEntries(t.map(r=>{switch(r.componentType){case Xe.ROUTE:case Xe.STREET_NUMBER:case Xe.STREET_ADDRESS:return [Ac.STREET_1,`${Eg(Xe.STREET_NUMBER,e)} ${Eg(Xe.ROUTE,e)}`];default:return [aL(r.componentType),r.componentName.text]}}))},Eg=(e,t)=>{var r,n;return (n=(r=t.find(o=>o.componentType===e))==null?void 0:r.componentName.text)!=null?n:""},KW=(e,t)=>{var r,n;return (n=(r=t.question.choices.find(o=>o.alias===e))==null?void 0:r.id)!=null?n:""};var YW=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const pr=L("quoteableCarriersProduct/Load",(e,t)=>YW(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield N$.getApplicationFormCarriersProducts(r.appFormId,r.config)).parsedBody}catch(o){const a=new D("Error getting carrier products",o);throw R(a),n(a),a}}));var Ig=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const nn=L("choices/Search",(e,t)=>Ig(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield su.searchChoices(r.questionId,{filter:r.filter},r.config)).parsedBody}catch(o){const a=new D("Error searching choices",o);throw R(a),n(a),a}})),ke=L("choicesWithNudges/Search",(e,t)=>Ig(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield su.searchChoicesWithNudges(r.questionId,{filter:r.filter,refilter:r.refilter},r.config)).parsedBody}catch(o){const a=new D("Error searching choices",o);throw R(a),n(a),a}}));var JW=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const xo=L("features/Search",(e,t)=>JW(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield L$.getFeatures(r.config)).parsedBody}catch(o){const a=new D("Error getting features",o);throw R(a),n(a),a}}));var XW=Object.defineProperty,e8=Object.defineProperties,t8=Object.getOwnPropertyDescriptors,Tg=Object.getOwnPropertySymbols,r8=Object.prototype.hasOwnProperty,n8=Object.prototype.propertyIsEnumerable,qg=(e,t,r)=>t in e?XW(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,o8=(e,t)=>{for(var r in t||(t={}))r8.call(t,r)&&qg(e,r,t[r]);if(Tg)for(var r of Tg(t))n8.call(t,r)&&qg(e,r,t[r]);return e},a8=(e,t)=>e8(e,t8(t)),xg=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const kg="questionPools",ko=L(`${kg}/Add`,(e,t)=>xg(void 0,[e,t],function*(r,{getState:n,rejectWithValue:o}){var a,i;try{const s=n();if(r.parent_question_pool_id){const c=(a=s.application.questionSets.list.filter(u=>u.question_set.question_pools))==null?void 0:a.find(u=>u.question_set.question_pools.findIndex(l=>l.id===r.parent_question_pool_id)!==-1);c&&(r=a8(o8({},r),{parent_answer_pool_id:(i=c.answers.find(u=>u.answer_pool_id===r.selected_answer_pool_id))==null?void 0:i.answer_pool_id}));}return (yield cu.addQuestionPool(r)).parsedBody}catch(s){const c=new D("Error adding question pool",s);return R(c),o(c)}})),Do=L(`${kg}/Remove`,(e,t)=>xg(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=yield cu.removeQuestionPool(r);return r.callback&&r.callback(),o.parsedBody}catch(o){const a=new D("Error adding question pool",o);return R(a),n(a)}}));var i8=Object.defineProperty,s8=Object.defineProperties,c8=Object.getOwnPropertyDescriptors,Dg=Object.getOwnPropertySymbols,u8=Object.prototype.hasOwnProperty,l8=Object.prototype.propertyIsEnumerable,Lg=(e,t,r)=>t in e?i8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ng=(e,t)=>{for(var r in t||(t={}))u8.call(t,r)&&Lg(e,r,t[r]);if(Dg)for(var r of Dg(t))l8.call(t,r)&&Lg(e,r,t[r]);return e},Rg=(e,t)=>s8(e,c8(t)),As=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Os="questionSets",on=L(`${Os}/LoadOne`,(e,t)=>As(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ja.getQuestionSet(r.appForm,{id:r.id},r.config)).parsedBody}catch(o){const a=new D("Error loading single question set",o);throw R(a),n(a),a}})),d8=L(`${Os}/Create`,(e,t)=>As(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield ja.createQuestionSet(r.appFormQuestionSet,r.config)).parsedBody}catch(o){const a=new D("Error loading single question set",o);throw R(a),n(a),a}})),an=L(`${Os}/Save`,(e,t)=>As(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=r.parentQuestionSet?Rg(Ng({},r.parentQuestionSet),{is_completed:!0}):Rg(Ng({},r.appFormQuestionSet),{is_completed:!0});return (yield ja.updateQuestionSet(o,r.config)).parsedBody}catch(o){const a=new D("Error saving question set",o);return R(a),n(a)}}));var Mg=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const sn=L("bindQuote/Bind",(e,t)=>Mg(void 0,[e,t],function*(r,{rejectWithValue:n}){var o;try{const a=yield uu.bindQuote(r.quoteId,r.appFormId,r.config);return r.afterResponse&&r.afterResponse(((o=a.parsedBody)==null?void 0:o.retrieve_url)||""),a.parsedBody}catch(a){const i=new D("Error binding quote request",a);throw R(i),n(i),i}})),Lo=L("bindQuote/action",(e,t)=>Mg(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const{action:o,quote:a,config:i}=r;return yield uu.bindQuoteAction(o,a,i)}catch(o){const a=new D("Error binding quote request",o);throw n(a),a}}));var Ke=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const wt="vehicleOptions",De=L(`${wt}/LoadFull`,(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=de.getYears(r.config),a=de.getMakes(r.year,r.config,{type:r.type}),i=de.getModels(r.year,r.makeId,r.config,r.type),s=de.getConfigurations(r.year,r.makeId,r.modelId,r.config,r.type),{parsedBody:c}=yield o,{parsedBody:u}=yield a,{parsedBody:l}=yield i,{parsedBody:d}=yield s;return {years:c,makes:u,models:l,configurations:d}}catch(o){const a=new D("Error loading full vehicle options",o);throw R(a),n(a),a}})),fr=L(`${wt}/LazyLoad`,(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){var o,a,i,s,c,u;try{const l=de.getYears(r.config),d=de.getMakes(r.year,r.config,{type:r.type}),{parsedBody:f}=yield l,{parsedBody:g}=yield d,v=(a=(o=g.find(A=>A.name.toUpperCase()===r.make.toUpperCase()))==null?void 0:o.id)==null?void 0:a.toString();let h=[];v&&(h=(yield de.getModels(r.year,v,r.config,r.type)).parsedBody);const b=(s=(i=h.find(A=>A.name.toUpperCase()===r.model.toUpperCase()))==null?void 0:i.id)==null?void 0:s.toString();let m=[];v&&b&&(m=(yield de.getConfigurations(r.year,v,b,r.config,r.type)).parsedBody);const _=(u=(c=m.find(A=>{var w;return A.name.toUpperCase()===((w=r.configuration)==null?void 0:w.toUpperCase())}))==null?void 0:c.id)==null?void 0:u.toString();return {years:f,makes:g,models:h,configurations:m,yearId:r.year,makeId:v,modelId:b,configurationId:_,questionGroupId:r.questionGroupId}}catch(l){const d=new D("Error loading full vehicle options",l);throw R(d),n(d),d}})),M=L(`${wt}/LoadYears`,(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield de.getYears(r.config)).parsedBody}catch(o){const a=new D("Error loading vehicle years",o);throw R(a),n(a),a}})),H=L(`${wt}/LoadMakes`,(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield de.getMakes(r.year,r.config,{type:r.type})).parsedBody}catch(o){const a=new D("Error loading vehicle makes",o);throw R(a),n(a),a}})),ce=L(`${wt}/LoadModels`,(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield de.getModels(r.year,r.makeId,r.config,r.type)).parsedBody}catch(o){const a=new D("Error loading vehicle makes",o);throw R(a),n(a),a}})),cn=L(`${wt}/LoadConfigurations`,(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return (yield de.getConfigurations(r.year,r.makeId,r.modelId,r.config,r.type)).parsedBody}catch(o){const a=new D("Error loading vehicle configurations",o);throw R(a),n(a),a}})),N=L(`${wt}/LoadVin`,(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){try{return yield de.getVinDetails(r.vin,r.config)}catch(o){const a=new D("Error loading VIN",o);throw R(a),n(a),a}})),No=L("selectVehicle/Save",(e,t)=>Ke(void 0,[e,t],function*(r,{rejectWithValue:n}){try{const o=yield de.selectVehicle(r);return o&&r.callback&&r.callback(),o.parsedBody}catch(o){const a=new D("Error saving vehicle selection",o);throw R(a),n(a),a}}));var p8=Object.defineProperty,f8=Object.defineProperties,v8=Object.getOwnPropertyDescriptors,Vg=Object.getOwnPropertySymbols,g8=Object.prototype.hasOwnProperty,h8=Object.prototype.propertyIsEnumerable,Fg=(e,t,r)=>t in e?p8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,At=(e,t)=>{for(var r in t||(t={}))g8.call(t,r)&&Fg(e,r,t[r]);if(Vg)for(var r of Vg(t))h8.call(t,r)&&Fg(e,r,t[r]);return e},Ro=(e,t)=>f8(e,v8(t));const Qg=At({},Qy),Ug=P({name:"questionSets",initialState:Qg,reducers:{switchActiveQuestionSet:(e,t)=>{const r=e.list.find(n=>n.question_set_id===e.active);r&&hv(r)&&(r.is_completed=wi(r)),e.active=_o(e,t.payload,r);},updateActiveQuestionSet:(e,{payload:t})=>{Jg(e),e.active=t;},goToCarrierQuestionSet:(e,{payload:t})=>{const r=e.list.find(n=>n.question_set.tenant_id===t);r&&(Jg(e),e.active=r.question_set_id);},markActiveQuestionSetCompleted:e=>{const t=e.list.findIndex(r=>r.question_set_id===e.active);t!==-1&&(e.list[t].is_completed=Vd(e.list[t].answers));},receiveQuestionSet:(e,{payload:t})=>Hg(e,t),removeQuestionSet:(e,{payload:t})=>Kg(e,t),receiveUpdatedAnswer:(e,{payload:t})=>Zg(e,t),updateActiveAnswerId:(e,{payload:t})=>{const r=Ro(At({},e),{activeAnswerId:t,pendingActiveAnswerUpdate:void 0});return e.pendingActiveAnswerUpdate?Le(r,e.pendingActiveAnswerUpdate):r},removeActiveAnswerId:e=>Wg(e)},extraReducers:e=>{e.addCase(ws.fulfilled,t=>Wg(t)).addCase(an.fulfilled,(t,{payload:r,meta:n})=>{if(I8(t,r))return t;k8(r)?A8(t,n):O8(t,r,n);}).addCase(ee.pending,(t,{meta:r})=>Le(t,Ro(At({},r.arg.answer),{version:r.arg.answer.version+1}),!1)).addCase(ee.rejected,(t,{meta:r})=>Xg(t,r.arg.answer)?t:Le(t,r.arg.answer,!1)).addCase(ee.fulfilled,(t,{payload:r})=>Xg(t,r)?t:Le(t,r)).addCase(wa,(t,{payload:r})=>{const n=Ro(At({},Le(t,r,!1)),{pendingActiveAnswerUpdate:void 0}),o=js(t,r);if(o!==-1){const a=n.list[o];a.is_completed=!1;}return n}).addCase(on.fulfilled,(t,{payload:r})=>Ss(t,r,!1)).addCase(Ae,(t,{payload:r})=>{let n=At({},t);return r.events.forEach(o=>{switch(o.event){case x.AddQuestionSet:n=Hg(n,o.data);break;case x.RemoveQuestionSet:n=Kg(n,o.data);break;case x.Answer:n=Zg(n,o.data);break}}),n}),e.addMatcher($(...be),(t,{payload:r})=>{const n=KF(r.application_forms_question_sets);t.list=S8(r.application_forms_question_sets,n),t.active=ir(t.list),t.activeAnswerId="",t.parentQuestionSets=n;}).addMatcher($(...k),t=>At({},Qg));}}),{switchActiveQuestionSet:m8,updateActiveQuestionSet:b8,receiveQuestionSet:Bg,removeQuestionSet:Gg,receiveUpdatedAnswer:zg,updateActiveAnswerId:y8,removeActiveAnswerId:_8,goToCarrierQuestionSet:$8,markActiveQuestionSetCompleted:w8}=Ug.actions,Wg=e=>{const t=Ro(At({},e),{activeAnswerId:"",pendingActiveAnswerUpdate:void 0});return e.pendingActiveAnswerUpdate?Le(t,e.pendingActiveAnswerUpdate):t},A8=(e,t)=>{e.completedQuestionSetAnswerHasChanged=!(e.completedQuestionSetAnswerHasChanged&&Cs(e.list));const r=t.arg.parentQuestionSet?mv(t.arg.parentQuestionSet,t.arg.appFormQuestionSet,e):void 0;e.active=Av(e,t.arg.navigationDirection,r);},O8=(e,t,r)=>{const n=Ss(e,t,!1);e.list=n.list,e.parentQuestionSets=n.parentQuestionSets;const o=r.arg.parentQuestionSet?mv(r.arg.parentQuestionSet,r.arg.appFormQuestionSet,e):void 0,a=Ov(n,t,r.arg.navigationDirection,o);a?e.active=a:t.next_active_question_set_id?e.idOfNextActiveQuestionSetNotInState=t.next_active_question_set_id:e.active=ir(n.list);},Hg=(e,t)=>{const r=Ss(e,t);return e.list=r.list,e.completedQuestionSetAnswerHasChanged=r.completedQuestionSetAnswerHasChanged,e.parentQuestionSets=r.parentQuestionSets,e.idOfNextActiveQuestionSetNotInState===t.id?(e.active=$o(e,e.idOfNextActiveQuestionSetNotInState),e.idOfNextActiveQuestionSetNotInState=void 0):e.active=Sv(e,t),e},Zg=(e,t)=>Le(e,t),Kg=(e,t)=>(e.list=e.list.filter(r=>r.id!==t.id&&r.parentQuestionSetId!==t.id),e.parentQuestionSets=e.parentQuestionSets.filter(r=>r.id!==t.id),e);function Le(e,t,r=!0){const n=js(e,t);if(n!==-1){const o=rs(e),{answer:a,index:i}=T8(o,t),s=o.list[n];if(a)D8(a,t)&&(x8(t)?eh(e,o,a,t,r):s.answers.splice(i,1));else if(t.active&&t.display_type!=="private"){const c=L8(s,t);s.answers.splice(c,0,t);}return s.parentQuestionSetId&&(s.is_blank=s.answers.length===0),o}else if(Fd(t,e)){const o=rs(e),a=Fd(t,o),i=a==null?void 0:a.answers.find(s=>s.id===t.id);return i&&eh(e,o,i,t,r),o}return e}function Ss(e,t,r=!0){let n=rs(e),o=n.completedQuestionSetAnswerHasChanged;if(gv(t))n=ZF(n,t);else {const a=n.list.findIndex(i=>i.question_set_id===t.question_set_id);if(a>-1){const i=n.list[a],s=!e.completedQuestionSetAnswerHasChanged&&Yg(i.is_completed,t.is_completed);if(i.completed_at=t.completed_at,i.is_active=t.is_active,i.is_completed=t.is_completed,i.order=t.order,i.updated_at=t.updated_at,i.is_blank=t.is_blank,r)for(const c of t.answers)n=Le(n,c);s&&(o=!0);}else n.list=P8(n.list,t);}return n.completedQuestionSetAnswerHasChanged&&Cs(n.list)&&(o=!1),n.completedQuestionSetAnswerHasChanged=o,n}function S8(e,t){return YF(e,t)}function j8(e,t){return e.findIndex(r=>r.question_set_id===t)}function C8(e,t,r){const n=t.list.findIndex(o=>o.answers.find(a=>a.question.id===r));if(n===-1){const o=e.question_set.question_groups.find(a=>a.question_ids.includes(r));if(o){const a=t.list.findIndex(i=>i.question_set.question_groups.find(s=>s.id===o.id));return a===-1?t.list.findIndex(i=>i.id===o.id):a}}return n}const Yg=(e,t)=>!!e&&!t;function js(e,t){const r=e.parentQuestionSets.find(n=>n.question_set_id===t.question.question_set_id);return r?C8(r,e,t.question.id):j8(e.list,t.question.question_set_id)}function Jg(e){const t=e.list.find(r=>r.question_set_id===e.active);t&&hv(t)&&(t.is_completed=wi(t));}function P8(e,t){const r=E8(e,t.order);return e.splice(r,0,t),e}function E8(e,t){let r=e.findIndex(n=>n.order>=t);return r===-1&&(r=e.length),r}function Cs(e){return e.length?e.slice(0,-1).every(t=>t.is_completed):!1}function I8(e,t){var r;return !!t.application_form_id&&((r=e.list[0])==null?void 0:r.application_form_id)!==t.application_form_id}function T8(e,t){const r=js(e,t);return q8(t,r>=0?e.list[r]:void 0)}function q8(e,t){const r=(a,i)=>({answer:i&&a>=0?i.answers[a]:void 0,index:a}),n=(a,i)=>{const s=c=>u=>u.id===c;return i.findIndex(s(a))};let o=-1;return t&&(o=n(e.id,t.answers)),r(o,t)}function x8(e){return e.active&&e.display_type!=="private"}function Xg(e,t){var r;return ((r=e.list[0])==null?void 0:r.application_form_id)!==t.application_form_id}function k8(e){return !e.id}function D8(e,t){const r=new Date(e.updated_at);return new Date(t.updated_at)>=r&&t.version>=e.version||t.created_at===e.created_at}function eh(e,t,r,n,o){e.activeAnswerId!==n.id?Object.assign(r,n):(Object.assign(r,n,{value:r.value,choice_id:r.choice_id,choice_ids:r.choice_ids,updated_at:r.updated_at}),o&&(t.pendingActiveAnswerUpdate=n));}function L8(e,t){let r=e.answers.length;for(let n=0;n<e.answers.length;n++)if(e.answers[n].question.order>=t.question.order){r=n;break}return r}var N8=Object.defineProperty,R8=Object.defineProperties,M8=Object.getOwnPropertyDescriptors,Mo=Object.getOwnPropertySymbols,th=Object.prototype.hasOwnProperty,rh=Object.prototype.propertyIsEnumerable,nh=(e,t,r)=>t in e?N8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,oh=(e,t)=>{for(var r in t||(t={}))th.call(t,r)&&nh(e,r,t[r]);if(Mo)for(var r of Mo(t))rh.call(t,r)&&nh(e,r,t[r]);return e},V8=(e,t)=>R8(e,M8(t)),F8=(e,t)=>{var r={};for(var n in e)th.call(e,n)&&t.indexOf(n)<0&&(r[n]=e[n]);if(e!=null&&Mo)for(var n of Mo(e))t.indexOf(n)<0&&rh.call(e,n)&&(r[n]=e[n]);return r};const Q8=[],ah=P({name:"quotes",initialState:Q8,reducers:{receiveQuoteRequest:(e,{payload:t})=>Ps(e,t)},extraReducers:e=>{e.addCase(Ae,(t,{payload:r})=>{let n=[...t];return r.events.forEach(o=>{switch(o.event){case x.QuoteRequest:n=Ps(n,o.data);break}}),n}).addMatcher($(...be),(t,{payload:r})=>{if(r!=null&&r.quote_requests){let n=[];return n=r.quote_requests.reduce((o,a)=>Ps(o,a),n),n}return t}).addMatcher($(...k),t=>[]);}}),{receiveQuoteRequest:ih}=ah.actions,Ps=(e,t)=>{var r;let n=[...e];const o=e.filter(a=>a.id===t.id||a.parentQuoteRequestId===t.id);if(o.length){const a=new Date(o[0].updated_at);new Date(t.updated_at)>=a&&(n=e.filter(i=>i.id!==t.id&&i.parentQuoteRequestId!==t.id));}if(t.active){const a=t,i=F8(a,["quotes"]);n.push(...((r=t.quotes)!=null?r:[]).map(s=>V8(oh(oh({},i),s),{id:`${t.id}${s.supplier_code?`-${s.supplier_code}`:""}`,parentQuoteRequestId:t.id})));}return n},U8=(e,t,r="")=>{const{data:n,event:o}=e;switch(o){case x.Appetite:t.dispatch(ff(n));break;case x.AddQuestionSet:t.dispatch(Bg(n));break;case x.RemoveQuestionSet:t.dispatch(Gg(n));break;case x.ApplicationForm:t.dispatch(Br(n));break;case x.Message:t.dispatch(go(n));break;case x.QuoteRequest:t.dispatch(ih(n));break;case x.Answer:t.dispatch(zg(n));break;case x.CarrierProducts:t.dispatch(Fi(n.length>0?n.filter(a=>a.application_form_id===r):n));break;case x.ApplicationFormPercentCompleted:t.dispatch(Vi(n));break;case x.ApplicationFormProcess:t.dispatch($f(n));break;case x.RealTimeEligibility:t.dispatch(vf(n));break;case x.Notification:t.dispatch(yf(n));break;case x.Batch:t.dispatch(Ae(n));break}},B8=(e,t)=>{if(!(e!=null&&e.data))return !0;switch(e.event){case x.ApplicationForm:return e.data.id!==t;case x.ApplicationFormPercentCompleted:return e.data.application_form_id!==t;case x.CarrierProducts:return e.data.length>0?e.data.filter(r=>r.application_form_id===t).length===0:!1;case x.Notification:return !1;case x.Batch:return !1;case x.Appetite:default:return e.data.application_form_id!==t}},G8=(e,t,r)=>{B8(t,r)||U8(t,e,r);},z8=e=>t=>r=>{G8(e,r,t);};let sh;const ch=(e,t,r)=>{e===It.Disconnected&&console.error("Websocket has disconnected. Attempting to reconnect."),r.dispatch(Sn({status:e,appId:t}));},W8=(e,t)=>()=>ch(It.Connected,e,t),H8=(e,t)=>()=>ch(It.Disconnected,e,t),Z8=(e,t,r=sh)=>{r!=null&&r.disconnect();const n=z8(e),{config:{wsBaseUri:o,options:{enableWebSockets:a}}}=e.getState(),{payload:{cable_token:i,id:s}}=t;a&&o&&(sh=r=new $_({wsBaseUri:o,token:i,id:s}),r.start(),r.subscribe("ApplicationFormChannel",{id:s},{received:n(s),connected:W8(s,e),disconnected:H8(s,e)}));},K8=e=>t=>r=>(be.findIndex(n=>n.type===r.type)>=0&&(e.dispatch(Aa({appId:r.payload.id})),e.dispatch(Sn({status:It.Connecting,appId:r.payload.id})),Z8(e,r)),t(r)),Y8=e=>t=>r=>{const{config:{debug:n}}=e.getState();if(n){console.group(r.type),console.info("dispatching",r);const o=t(r);return console.log("next state",e.getState()),console.groupEnd(),o}return t(r)};var J8=Object.defineProperty,X8=Object.defineProperties,e5=Object.getOwnPropertyDescriptors,uh=Object.getOwnPropertySymbols,t5=Object.prototype.hasOwnProperty,r5=Object.prototype.propertyIsEnumerable,lh=(e,t,r)=>t in e?J8(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,vr=(e,t)=>{for(var r in t||(t={}))t5.call(t,r)&&lh(e,r,t[r]);if(uh)for(var r of uh(t))r5.call(t,r)&&lh(e,r,t[r]);return e},n5=(e,t)=>X8(e,e5(t));const dh=vr({},Dy),ph=P({name:"auth",initialState:dh,reducers:{setAuth:(e,{payload:t})=>n5(vr(vr({},e),t),{refreshCount:e.refreshCount+1}),setUser:(e,{payload:t})=>vr(vr({},e),dx(t)),setImpersonatedOnAuth:(e,{payload:t})=>{e.options.impersonated=t.sudo;},loggedOut:e=>vr({},dh),setTheme:(e,{payload:t})=>{e.terminalTheme=t;}}}),{setAuth:fh,setUser:vh,setImpersonatedOnAuth:o5,loggedOut:un,setTheme:a5}=ph.actions;var i5=Object.defineProperty,s5=Object.defineProperties,c5=Object.getOwnPropertyDescriptors,gh=Object.getOwnPropertySymbols,u5=Object.prototype.hasOwnProperty,l5=Object.prototype.propertyIsEnumerable,hh=(e,t,r)=>t in e?i5(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,gr=(e,t)=>{for(var r in t||(t={}))u5.call(t,r)&&hh(e,r,t[r]);if(gh)for(var r of gh(t))l5.call(t,r)&&hh(e,r,t[r]);return e},d5=(e,t)=>s5(e,c5(t));const mh=gr({},Ry),bh=P({name:"config",initialState:mh,reducers:{setConfig:(e,{payload:t})=>d5(gr(gr({},e),t),{options:gr(gr({},e.options),t.options)}),setWebSocketConfig:(e,{payload:t})=>{e.options.enableWebSockets=t.enableWebsocket;}},extraReducers:e=>{e.addCase(Sr,t=>gr({},mh)).addMatcher($(...be),(t,{payload:r})=>{t.cableToken=r.cable_token;});}}),{setConfig:yh,setWebSocketConfig:p5}=bh.actions;var f5=Object.defineProperty,_h=Object.getOwnPropertySymbols,v5=Object.prototype.hasOwnProperty,g5=Object.prototype.propertyIsEnumerable,$h=(e,t,r)=>t in e?f5(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,wh=(e,t)=>{for(var r in t||(t={}))v5.call(t,r)&&$h(e,r,t[r]);if(_h)for(var r of _h(t))g5.call(t,r)&&$h(e,r,t[r]);return e};const Ah=wh({},Nc),Oh=P({name:"status/Local",initialState:Ah,reducers:{setLocalStatusState:(e,{payload:t})=>{e.state=t;},setLocalStatusHealth:(e,{payload:t})=>{e.health=t;}},extraReducers:e=>{e.addCase(Sr,t=>wh({},Ah)).addCase(fh,t=>{t.auth=q.Configured;}).addCase(yh,t=>{t.config=q.Configured;}).addCase(vh,t=>{t.user=q.Configured;}).addCase(Qc,t=>{t.auth=q.Error,t.user=q.Error;}).addCase(un,t=>{t.auth=q.NotLoaded,t.user=q.NotLoaded;}).addCase(Sn,(t,{payload:r})=>{t.socket=r.appId===t.socketId?r.status:t.socket;}).addCase(Aa,(t,{payload:r})=>{t.socketId=r.appId;});}}),{setLocalStatusState:Vo,setLocalStatusHealth:Fo}=Oh.actions;function Sh(e,t,r){t(ee({answer:r,config:up(e)}));}function h5(e,t){switch(t.type){case ee.fulfilled.type:return e.application.form.id!==t.payload.application_form_id;default:return !1}}function jh(e,t){const r=e.queues.answers.queue.length,n=Object.keys(e.queues.answers.outbound).length;r&&!n&&Sh(e,t.dispatch,e.queues.answers.queue[0]);}function m5(e,t,r){const n=e.queues.answers.queue.length,o=Object.keys(e.queues.answers.outbound).length;!n&&!o?Sh(e,t.dispatch,r):t.dispatch(On(r));}function b5(e,t,r){e.queues.answers.queue[0]&&e.queues.answers.queue[0].id!==r.payload.id&&jh(e,t);}const y5=e=>t=>r=>{const n=e.getState();if(h5(n,r))return t(r);switch(r.type){case ee.fulfilled.type:b5(n,e,r);break;case ee.pending.type:case Vo.type:case Fo.type:case go.type:case un.type:break;case wa.type:m5(n,e,r.payload);break;default:jh(n,e);break}return t(r)};var _5=Object.defineProperty,$5=Object.defineProperties,w5=Object.getOwnPropertyDescriptors,Ch=Object.getOwnPropertySymbols,A5=Object.prototype.hasOwnProperty,O5=Object.prototype.propertyIsEnumerable,Ph=(e,t,r)=>t in e?_5(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,ln=(e,t)=>{for(var r in t||(t={}))A5.call(t,r)&&Ph(e,r,t[r]);if(Ch)for(var r of Ch(t))O5.call(t,r)&&Ph(e,r,t[r]);return e},Es=(e,t)=>$5(e,w5(t));const Eh=ln({},My),Ih=P({name:"error",initialState:Eh,reducers:{updateErrors:(e,{payload:t})=>Es(ln({},e),{[t.answer.id]:t.errors}),removeError:(e,{payload:t})=>Es(ln({},e),{[t]:[]})},extraReducers:e=>{e.addCase(ee.pending,(t,{meta:r})=>{const{answer:n}=r.arg,o=$i(n);t[n.id]=o;}).addCase(ee.rejected,(t,{meta:r,payload:n})=>{var o,a,i;const s=(a=(o=n==null?void 0:n.response)==null?void 0:o.parsedBody)!=null?a:[],c=r.arg.answer.id,u=(i=t[c])!=null?i:[];return Es(ln({},t),{[c]:[...u,...s]})}),e.addMatcher($(...k),t=>ln({},Eh));}}),{updateErrors:S5,removeError:Th}=Ih.actions,j5=e=>t=>t.question.reference_type===mn.Accumulator&&C5(t.id,e)&&Md(t),C5=(e,t)=>t[e]!==void 0,P5=(e,t)=>{if(!e)return;const{errors:r,questionSets:{list:n}}=e;n.flatMap(o=>o.answers).filter(o=>j5(r)(o)).forEach(o=>{t.dispatch(Th(o.id));});},E5=e=>t=>r=>{if(r.type===ee.fulfilled.type){const{application:n}=e.getState();P5(n,e);}return t(r)},qh=e=>[e.status.remote.application.form,e.status.remote.questionSets.state,e.status.remote.questionPools.state,e.status.remote.answers.state,...Object.values(e.status.remote.bindQuotes),...Object.values(e.status.remote.choices),...Object.values(e.status.remote.refilters),...Object.values(e.status.remote.vehicles.configurations),...Object.values(e.status.remote.vehicles.makes),...Object.values(e.status.remote.vehicles.models),...Object.values(e.status.remote.vehicles.selection),...Object.values(e.status.remote.vehicles.vins),...Object.values(e.status.remote.vehicles.years)],I5=e=>[...qh(e),e.status.remote.quotes.state],T5=e=>e.status.remote.application.form,q5=e=>!!(e.auth.user&&e.config.clientId),x5=e=>qh(e).reduce((t,r)=>{switch(r){case y.Error:t=q.Configured;break;case y.NotLoaded:t=q.Configured;break;case y.Loading:t!==q.Configured&&(t=q.Loading);break;case y.Loaded:t!==q.Configured&&t!==q.Loading&&(t=q.Loaded);break}return t},q.Loaded),k5=e=>e===y.Error?qt.Error:qt.Healthy,D5=e=>I5(e).reduce((t,r)=>t===qt.Error?t:k5(r),qt.Healthy),L5=e=>{let t=q.NotLoaded;if(q5(e)){t=q.Configured;const r=T5(e);r===y.Loaded?t=x5(e):r===y.Loading&&(t=q.Loading);}return t},N5=e=>t=>r=>{const n=e.getState();switch(r.type){case Vo.type:case Fo.type:return t(r)}const o=L5(n),a=D5(n);return o!==n.status.local.state&&e.dispatch(Vo(o)),a!==n.status.local.health&&e.dispatch(Fo(a)),t(r)},R5=new D().name;function M5(e){return e.payload&&e.payload.error&&R5===e.payload.error.name}const V5=e=>t=>r=>{if(r.type===un.type)return cx(),t(r);const n=e.getState().config.debug;if(M5(r)){const o=r.payload.error.response;if((o==null?void 0:o.status)===401){n&&console.error("User is trying to make an unauthenticated request, logging out."),e.dispatch(un());const a={id:"logoutExpired",tenant_id:"",owner_id:"",target:"Your login has expired. Please log in again to continue.",application_form_id:"",active:!0,created_at:new Date,updated_at:new Date,code:"",level:Sc.warning,location:null,metadata:{},component:et.Application,priority:!1};e.dispatch(go(a));}}return t(r)};var F5=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Q5=e=>t=>r=>F5(void 0,null,function*(){if(be.map(n=>n.type).includes(r.type)){const n=localStorage.getItem(`bp-sdk-validation:${r.payload.id}`);if(n)return e.dispatch(Vh(JSON.parse(n))),t(r)}return t(r)});var U5=[Q5,y5,V5,Y8,N5,E5,K8],B5=Object.defineProperty,G5=Object.defineProperties,z5=Object.getOwnPropertyDescriptors,xh=Object.getOwnPropertySymbols,W5=Object.prototype.hasOwnProperty,H5=Object.prototype.propertyIsEnumerable,kh=(e,t,r)=>t in e?B5(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Z=(e,t)=>{for(var r in t||(t={}))W5.call(t,r)&&kh(e,r,t[r]);if(xh)for(var r of xh(t))H5.call(t,r)&&kh(e,r,t[r]);return e},te=(e,t)=>G5(e,z5(t));const Is=Z({},Wy),Dh=P({name:"viewState/AddressValidation/validateAddress",initialState:Is,reducers:{initializeAddressValidation:(e,t)=>e.validations[t.payload]?e:te(Z({},e),{validations:te(Z({},e.validations),{[t.payload]:Z({},Pc)})}),hydrateAddressValidation:(e,t)=>te(Z({},e),{validations:t.payload}),setAddressValidationTriggerOverridden:(e,t)=>te(Z({},e),{shouldOverrideValidationTrigger:t.payload}),stageAddressValidationAnswer:(e,t)=>{const{answerToStage:r,addressValidationId:n}=t.payload,o=e.validations[n].stagedAnswers.filter(a=>a.answer.id!==r.answer.id);return te(Z({},e),{validations:te(Z({},e.validations),{[n]:te(Z({},e.validations[n]),{stagedAnswers:[...o,r]})})})},setAddressValidationStatus:(e,t)=>te(Z({},e),{validations:te(Z({},e.validations),{[t.payload.addressValidationId]:te(Z({},e.validations[t.payload.addressValidationId]),{validationStatus:t.payload.validationStatus})})}),addAddressValidationAttempt:(e,t)=>te(Z({},e),{validations:te(Z({},e.validations),{[t.payload.addressValidationId]:te(Z({},e.validations[t.payload.addressValidationId]),{attempt:e.validations[t.payload.addressValidationId].attempt+1})})}),resetAddressValidation:(e,t)=>te(Z({},e),{validations:te(Z({},e.validations),{[t.payload]:te(Z({},e.validations[t.payload]),{verdict:null,address:null,stagedAnswers:[]})})}),resetAddressValidationAttempt:(e,t)=>te(Z({},e),{validations:te(Z({},e.validations),{[t.payload]:te(Z({},e.validations[t.payload]),{attempt:e.validations[t.payload].attempt-1})})})},extraReducers:e=>{e.addCase(rn.fulfilled,(t,r)=>{const{addressValidationId:n,responseId:o,originalAddressComponents:a,result:i}=r.payload,s=Z5(i);t.validations[n].responseId=t.validations[n].responseId?t.validations[n].responseId:o,t.validations[n].verdict=s,t.validations[n].address=i.address,t.validations[n].validationStatus=e6(s,t),t.validations[n].stagedAnswers=[],t.validations[n].attempt=t.validations[n].attempt+1,t.validations[n].originalAddress=oL(a);}),e.addCase(rn.rejected,(t,r)=>{const n=r.meta.arg.addressValidationId;t.validations[n].validationStatus=ie.VALIDATION_ERROR;}),e.addCase(ws.fulfilled,(t,r)=>{t.validations[r.payload.addressValidationId].validationStatus=Lh(t,r.payload.addressValidationId),t.validations[r.payload.addressValidationId].verdict=null,t.validations[r.payload.addressValidationId].stagedAnswers=[];}),e.addMatcher($(...be),t=>t.validations?t:Z({},Is)),e.addMatcher($(...k),t=>Z({},Is));}}),Z5=e=>K5(e.verdict)?kt.FIX:Y5(e.verdict)&&!X5(e.address.addressComponents)?kt.CONFIRM:J5(e.address.addressComponents)?kt.TYPO:kt.ACCEPT,K5=e=>!e.addressComplete||e.hasUnconfirmedComponents,Y5=e=>e.validationGranularity!=="OTHER"&&e.addressComplete&&(e.hasInferredComponents||e.hasReplacedComponents),J5=e=>e.some(t=>t.spellCorrected),X5=e=>{const t=e.filter(r=>r.inferred);return !e.filter(r=>r.replaced).length&&t.length===1&&t[0].componentType==="postal_code_suffix"},e6=(e,t)=>e===kt.ACCEPT?t.shouldOverrideValidationTrigger?ie.COMPLETED:ie.PROCEED_NEXT:ie.VALIDATED,Lh=(e,t)=>e.validations[t].attempt>=2?e.shouldOverrideValidationTrigger?ie.COMPLETED:ie.PROCEED_NEXT:ie.READY_TO_VALIDATE,{addAddressValidationAttempt:t6,initializeAddressValidation:r6,resetAddressValidation:Nh,resetAddressValidationAttempt:n6,stageAddressValidationAnswer:Rh,setAddressValidationStatus:Mh,setAddressValidationTriggerOverridden:o6,hydrateAddressValidation:Vh}=Dh.actions;var a6=(e,t,r)=>new Promise((n,o)=>{var a=c=>{try{s(r.next(c));}catch(u){o(u);}},i=c=>{try{s(r.throw(c));}catch(u){o(u);}},s=c=>c.done?n(c.value):Promise.resolve(c.value).then(a,i);s((r=r.apply(e,t)).next());});const Fh=py();Fh.startListening({matcher:$(Mh,Rh,Nh,rn.fulfilled,rn.rejected),effect:(e,t)=>a6(void 0,null,function*(){t.cancelActiveListeners();const r=t.getState();localStorage.setItem(`bp-sdk-validation:${r.application.form.id}`,JSON.stringify(r.viewState.addressValidation.validations));})});var i6=[Fh.middleware];const s6=[],c6=P({name:"carriersProducts",initialState:s6,reducers:{},extraReducers:e=>{e.addCase(Ae,(t,{payload:r})=>{let n=[...t];return r.events.forEach(o=>{o.event===x.CarrierProducts&&Kt(o.data,t)&&(n=o.data);}),n}),e.addMatcher($(Fi,pr.fulfilled),(t,{payload:r})=>Kt(r,t)?r:t),e.addMatcher($(pr.pending,...k),()=>[]);}});var u6=Object.defineProperty,Qh=Object.getOwnPropertySymbols,l6=Object.prototype.hasOwnProperty,d6=Object.prototype.propertyIsEnumerable,Uh=(e,t,r)=>t in e?u6(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Bh=(e,t)=>{for(var r in t||(t={}))l6.call(t,r)&&Uh(e,r,t[r]);if(Qh)for(var r of Qh(t))d6.call(t,r)&&Uh(e,r,t[r]);return e};const Gh=Bh({},Ly),p6=P({name:"choice",initialState:Gh,reducers:{},extraReducers:e=>{e.addCase(nn.fulfilled,(t,r)=>{var n;const o=r.meta.arg,a=t[o.questionId];if(a&&a.timestamp>o.timestamp)return t;t[o.questionId]={list:(n=r.payload)!=null?n:[],timestamp:o.timestamp};}).addCase(ke.fulfilled,(t,r)=>{var n;const o=r.meta.arg,a=t[o.questionId];if(a&&a.timestamp>o.timestamp)return t;t[o.questionId]={list:(n=r.payload.choices)!=null?n:[],timestamp:o.timestamp};}),e.addMatcher($(...k),t=>Bh({},Gh));}}),f6=[],v6=P({name:"enrollments",initialState:f6,reducers:{},extraReducers:e=>{e.addCase(tr.fulfilled,(t,{payload:r})=>r.enrollments).addMatcher($(...k),()=>[]);}});var g6=Object.defineProperty,zh=Object.getOwnPropertySymbols,h6=Object.prototype.hasOwnProperty,m6=Object.prototype.propertyIsEnumerable,Wh=(e,t,r)=>t in e?g6(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Hh=(e,t)=>{for(var r in t||(t={}))h6.call(t,r)&&Wh(e,r,t[r]);if(zh)for(var r of zh(t))m6.call(t,r)&&Wh(e,r,t[r]);return e};const Zh=Hh({},Fy),Qo=(e,t)=>{e.meta.form.percent=t,t>e.meta.form.forwardProgressPercent&&(e.meta.form.forwardProgressPercent=t);},b6=P({name:"progress",initialState:Zh,reducers:{},extraReducers:e=>{e.addCase(Br,(t,{payload:r})=>{Qo(t,r.percent_completed);}).addCase(Vi,(t,{payload:r})=>{Qo(t,r.percent_completed);}).addCase(Ae,(t,{payload:r})=>(r.events.forEach(n=>{switch(n.event){case x.ApplicationForm:case x.ApplicationFormPercentCompleted:Qo(t,n.data.percent_completed);break}}),t)),e.addMatcher($(...be),(t,{payload:r})=>{Qo(t,r.percent_completed);}).addMatcher($(...k),t=>Hh({},Zh));}});var y6=Object.defineProperty,Kh=Object.getOwnPropertySymbols,_6=Object.prototype.hasOwnProperty,$6=Object.prototype.propertyIsEnumerable,Yh=(e,t,r)=>t in e?y6(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Jh=(e,t)=>{for(var r in t||(t={}))_6.call(t,r)&&Yh(e,r,t[r]);if(Kh)for(var r of Kh(t))$6.call(t,r)&&Yh(e,r,t[r]);return e};const Xh=Jh({},Yy),w6=P({name:"queues",initialState:Xh,reducers:{},extraReducers:e=>{e.addCase(On,(t,{payload:r})=>{const n=t.answers.queue.findIndex(o=>o.id===r.id);n===-1?t.answers.queue.push(r):t.answers.queue[n]=r;}).addCase(ee.pending,(t,{meta:r})=>{if(!t.answers.outbound[r.arg.answer.id]){t.answers.outbound[r.arg.answer.id]=r.arg.answer;const n=t.answers.queue.findIndex(o=>o.id===r.arg.answer.id);n!==-1&&t.answers.queue.splice(n,1);}}).addCase(ee.fulfilled,(t,{payload:r})=>{const n=t.answers.queue.find(o=>o.id===r.id);n&&(n.version=r.version);}),e.addMatcher($(ee.fulfilled,ee.rejected),(t,{meta:r})=>{t.answers.outbound[r.arg.answer.id]&&delete t.answers.outbound[r.arg.answer.id];}).addMatcher($(...k),()=>Jh({},Xh));}});var A6=Object.defineProperty,em=Object.getOwnPropertySymbols,O6=Object.prototype.hasOwnProperty,S6=Object.prototype.propertyIsEnumerable,tm=(e,t,r)=>t in e?A6(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,rm=(e,t)=>{for(var r in t||(t={}))O6.call(t,r)&&tm(e,r,t[r]);if(em)for(var r of em(t))S6.call(t,r)&&tm(e,r,t[r]);return e};const nm=rm({},Uy),j6=P({name:"quoteBind",initialState:nm,reducers:{},extraReducers:e=>{e.addCase(sn.fulfilled,(t,{payload:r,meta:n})=>{const{retrieve_url:o}=r;t[n.arg.quoteId]=o;}),e.addMatcher($(...k),t=>rm({},nm));}});var C6=Object.defineProperty,om=Object.getOwnPropertySymbols,P6=Object.prototype.hasOwnProperty,E6=Object.prototype.propertyIsEnumerable,am=(e,t,r)=>t in e?C6(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,im=(e,t)=>{for(var r in t||(t={}))P6.call(t,r)&&am(e,r,t[r]);if(om)for(var r of om(t))E6.call(t,r)&&am(e,r,t[r]);return e};const sm=im({},By),I6=P({name:"refilters",initialState:sm,reducers:{},extraReducers:e=>{e.addCase(ke.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=t[a.questionId];if(i&&i.timestamp>a.timestamp)return t;t[a.questionId]={list:(o=r==null?void 0:r.refilter)!=null?o:[],timestamp:a.timestamp};}),e.addMatcher($(...k),t=>im({},sm));}}),T6={},q6=P({name:"status/remote/vehicles/configurations",initialState:T6,reducers:{},extraReducers:e=>{e.addCase(cn.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.NotLoaded;}).addCase(De.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loaded;}).addCase(cn.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Error;}),e.addMatcher($(ce.pending,H.pending,M.pending,N.pending,ce.rejected,ce.fulfilled,H.rejected,H.fulfilled,M.rejected,M.fulfilled,N.rejected,N.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=y.NotLoaded;}).addMatcher($(...k),t=>({}));}}),x6={},k6=P({name:"status/remote/vehicles/makes",initialState:x6,reducers:{},extraReducers:e=>{e.addCase(H.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loading;}).addCase(H.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loaded;}).addCase(H.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Error;}),e.addMatcher($(M.pending,N.pending,M.rejected,M.fulfilled,N.rejected,N.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=y.NotLoaded;}).addMatcher($(...k),t=>({}));}}),D6={},L6=P({name:"status/remote/vehicles/models",initialState:D6,reducers:{},extraReducers:e=>{e.addCase(ce.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loading;}).addCase(ce.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loaded;}).addCase(ce.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Error;}),e.addMatcher($(M.pending,M.rejected,M.fulfilled,H.pending,H.rejected,H.fulfilled,N.pending,N.rejected,N.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=y.NotLoaded;}).addMatcher($(...k),t=>({}));}}),N6={},R6=P({name:"status/remote/vehicles/selections",initialState:N6,reducers:{},extraReducers:e=>{e.addCase(No.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loading;}).addCase(No.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loaded;}).addCase(No.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Error;}).addCase(Gc,(t,{payload:r})=>{t[r.questionGroupId]=y.NotLoaded;}),e.addMatcher($(...k),t=>({}));}}),M6={},V6=P({name:"status/remote/vehicles/vins",initialState:M6,reducers:{},extraReducers:e=>{e.addCase(N.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loading;}).addCase(N.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loaded;}).addCase(N.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Error;}),e.addMatcher($(M.pending,M.rejected,M.fulfilled,H.pending,H.rejected,H.fulfilled,ce.pending,ce.rejected,ce.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=y.NotLoaded;}).addMatcher($(...k),t=>({}));}}),F6={},Q6=P({name:"status/remote/vehicles/years",initialState:F6,reducers:{},extraReducers:e=>{e.addCase(M.pending,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loading;}).addCase(M.fulfilled,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Loaded;}).addCase(M.rejected,(t,{meta:r})=>{t[r.arg.questionGroupId]=y.Error;}),e.addMatcher($(N.pending,N.rejected,N.fulfilled),(t,{meta:r})=>{t[r.arg.questionGroupId]=y.NotLoaded;}).addMatcher($(...k),t=>({}));}});var U6=combineReducers({configurations:q6.reducer,makes:k6.reducer,models:L6.reducer,selection:R6.reducer,vins:V6.reducer,years:Q6.reducer}),B6=Object.defineProperty,cm=Object.getOwnPropertySymbols,G6=Object.prototype.hasOwnProperty,z6=Object.prototype.propertyIsEnumerable,um=(e,t,r)=>t in e?B6(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,lm=(e,t)=>{for(var r in t||(t={}))G6.call(t,r)&&um(e,r,t[r]);if(cm)for(var r of cm(t))z6.call(t,r)&&um(e,r,t[r]);return e};const dm=lm({},Dt),W6=P({name:"status/remote/answers",initialState:dm,reducers:{},extraReducers:e=>{e.addCase(ee.pending,(t,{meta:r})=>le(t,r.arg.answer.id,y.Loading)).addCase(ee.fulfilled,(t,{meta:r})=>le(t,r.arg.answer.id,y.Loaded)).addCase(ee.rejected,(t,{meta:r})=>le(t,r.arg.answer.id,y.Error)),e.addMatcher($(...k),t=>lm({},dm));}});var H6=Object.defineProperty,pm=Object.getOwnPropertySymbols,Z6=Object.prototype.hasOwnProperty,K6=Object.prototype.propertyIsEnumerable,fm=(e,t,r)=>t in e?H6(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Y6=(e,t)=>{for(var r in t||(t={}))Z6.call(t,r)&&fm(e,r,t[r]);if(pm)for(var r of pm(t))K6.call(t,r)&&fm(e,r,t[r]);return e};const J6=Y6({},xc),X6=P({name:"status/remote/applicationForm",initialState:J6,reducers:{},extraReducers:e=>{e.addMatcher($(...Li),t=>{t.form=y.NotLoaded;}).addMatcher($(...rf,vo.pending),t=>{t.form=y.Loading;}).addMatcher($(...be,vo.fulfilled),t=>{t.form=y.Loaded;}).addMatcher($(...Ni,vo.rejected),t=>{t.form=y.Error;});}}),eH={},tH=P({name:"status/remote/bindQuotes",initialState:eH,reducers:{},extraReducers:e=>{e.addCase(sn.pending,(t,{meta:r})=>{t[r.arg.quoteId]=y.Loading;}).addCase(sn.fulfilled,(t,{meta:r})=>{t[r.arg.quoteId]=y.Loaded;}).addCase(sn.rejected,(t,{meta:r})=>{t[r.arg.quoteId]=y.Error;}).addCase(Lo.pending,(t,{meta:r})=>{t[r.arg.quote.id]=y.Loading;}).addCase(Lo.fulfilled,(t,{meta:r})=>{t[r.arg.quote.id]=y.Loaded;}).addCase(Lo.rejected,(t,{meta:r})=>{t[r.arg.quote.id]=y.Error;}),e.addMatcher($(...k),t=>({}));}});var rH=Object.defineProperty,vm=Object.getOwnPropertySymbols,nH=Object.prototype.hasOwnProperty,oH=Object.prototype.propertyIsEnumerable,gm=(e,t,r)=>t in e?rH(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,hm=(e,t)=>{for(var r in t||(t={}))nH.call(t,r)&&gm(e,r,t[r]);if(vm)for(var r of vm(t))oH.call(t,r)&&gm(e,r,t[r]);return e};const mm=hm({},$n),aH=P({name:"status/remote/carriersProducts",initialState:mm,reducers:{},extraReducers:e=>{e.addCase(pr.pending,t=>{t.state=y.Loading;}).addCase(pr.fulfilled,t=>{t.state=y.Loaded;}).addCase(pr.rejected,t=>{t.state=y.Error;}),e.addMatcher($(...k),t=>hm({},mm));}}),iH={},sH=P({name:"status/remote/choices",initialState:iH,reducers:{},extraReducers:e=>{e.addCase(ke.pending,(t,{meta:r})=>{t[r.arg.questionId]=y.Loading;}).addCase(ke.fulfilled,(t,{meta:r})=>{t[r.arg.questionId]=y.Loaded;}).addCase(ke.rejected,(t,{meta:r})=>{t[r.arg.questionId]=y.Error;}),e.addMatcher($(...k),t=>({}));}});var cH=Object.defineProperty,bm=Object.getOwnPropertySymbols,uH=Object.prototype.hasOwnProperty,lH=Object.prototype.propertyIsEnumerable,ym=(e,t,r)=>t in e?cH(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,_m=(e,t)=>{for(var r in t||(t={}))uH.call(t,r)&&ym(e,r,t[r]);if(bm)for(var r of bm(t))lH.call(t,r)&&ym(e,r,t[r]);return e};const $m=_m({},Dt),dH=P({name:"status/remote/questionPool",initialState:$m,reducers:{},extraReducers:e=>{e.addCase(ko.pending,(t,{meta:r})=>le(t,r.arg.question_pool_id,y.Loading)).addCase(ko.fulfilled,(t,{meta:r})=>le(t,r.arg.question_pool_id,y.Loaded)).addCase(ko.rejected,(t,{meta:r})=>le(t,r.arg.question_pool_id,y.Error)).addCase(Do.pending,(t,{meta:r})=>le(t,r.arg.answer_pool_id,y.Loading)).addCase(Do.fulfilled,(t,{meta:r})=>le(t,r.arg.answer_pool_id,y.Loaded)).addCase(Do.rejected,(t,{meta:r})=>le(t,r.arg.answer_pool_id,y.Error)),e.addMatcher($(...k),t=>_m({},$m));}});var pH=Object.defineProperty,wm=Object.getOwnPropertySymbols,fH=Object.prototype.hasOwnProperty,vH=Object.prototype.propertyIsEnumerable,Am=(e,t,r)=>t in e?pH(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Om=(e,t)=>{for(var r in t||(t={}))fH.call(t,r)&&Am(e,r,t[r]);if(wm)for(var r of wm(t))vH.call(t,r)&&Am(e,r,t[r]);return e};const Sm=Om({},Dt),gH=P({name:"status/remote/questionSets",initialState:Sm,reducers:{},extraReducers:e=>{e.addCase(on.pending,(t,{meta:r})=>le(t,r.arg.id,y.Loading)).addCase(on.fulfilled,(t,{meta:r})=>le(t,r.arg.id,y.Loaded)).addCase(on.rejected,(t,{meta:r})=>le(t,r.arg.id,y.Error)),e.addCase(an.pending,(t,{meta:r})=>le(t,r.arg.appFormQuestionSet.id,y.Loading)).addCase(an.fulfilled,(t,{meta:r})=>le(t,r.arg.appFormQuestionSet.id,y.Loaded)).addCase(an.rejected,(t,{meta:r})=>le(t,r.arg.appFormQuestionSet.id,y.Error)),e.addMatcher($(...k),t=>Om({},Sm));}});var hH=Object.defineProperty,jm=Object.getOwnPropertySymbols,mH=Object.prototype.hasOwnProperty,bH=Object.prototype.propertyIsEnumerable,Cm=(e,t,r)=>t in e?hH(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Pm=(e,t)=>{for(var r in t||(t={}))mH.call(t,r)&&Cm(e,r,t[r]);if(jm)for(var r of jm(t))bH.call(t,r)&&Cm(e,r,t[r]);return e};const Em=Pm({},jc),yH=P({name:"viewState/Features",initialState:Em,reducers:{},extraReducers:e=>{e.addCase(xo.pending,t=>{t.isLoaded=!1,t.status=q.Loading;}).addCase(xo.fulfilled,(t,{payload:r})=>{t.list=r,t.isLoaded=!0,t.status=q.Loaded;}).addCase(xo.rejected,t=>{t.isLoaded=!1,t.status=q.Error;}).addCase(Sr,t=>Pm({},Em));}});var _H=Object.defineProperty,Im=Object.getOwnPropertySymbols,$H=Object.prototype.hasOwnProperty,wH=Object.prototype.propertyIsEnumerable,Tm=(e,t,r)=>t in e?_H(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,qm=(e,t)=>{for(var r in t||(t={}))$H.call(t,r)&&Tm(e,r,t[r]);if(Im)for(var r of Im(t))wH.call(t,r)&&Tm(e,r,t[r]);return e};const xm=qm({},Cc),km=P({name:"viewState/Quotes",initialState:xm,reducers:{setFilteredOnlineQuotes:(e,{payload:t})=>{e.filteredOnlineQuotes=t;},setFilteredOfflineQuotes:(e,{payload:t})=>{e.filteredOfflineQuotes=t;}},extraReducers:e=>{e.addMatcher($(...k),t=>qm({},xm));}}),{setFilteredOfflineQuotes:AH,setFilteredOnlineQuotes:OH}=km.actions;var SH=Object.defineProperty,jH=Object.defineProperties,CH=Object.getOwnPropertyDescriptors,Dm=Object.getOwnPropertySymbols,PH=Object.prototype.hasOwnProperty,EH=Object.prototype.propertyIsEnumerable,Lm=(e,t,r)=>t in e?SH(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,IH=(e,t)=>{for(var r in t||(t={}))PH.call(t,r)&&Lm(e,r,t[r]);if(Dm)for(var r of Dm(t))EH.call(t,r)&&Lm(e,r,t[r]);return e},TH=(e,t)=>jH(e,CH(t));const qH={},xH=P({name:"viewState/vehicleOptions/Configurations",initialState:qH,reducers:{},extraReducers:e=>{e.addCase(De.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=Ci(r);t[o.questionGroupId]=ji(a);}).addCase(cn.fulfilled,(t,{payload:r,meta:n})=>{t[n.arg.questionGroupId]=ji(r);}).addCase(fr.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r==null?void 0:r.configurations)!=null?o:[];t[a.questionGroupId]=ji(i);}).addCase(N.fulfilled,(t,{payload:r,meta:n})=>{const o=r,a=o==null?void 0:o.parsedBody;if(!r)return t;const i=Ci(a);t[n.arg.questionGroupId]=i.reduce((s,c)=>TH(IH({},s),{[c.id]:c.name}),{});}),e.addMatcher($(cn.pending,ce.pending,H.pending,M.pending,N.pending,ce.fulfilled,H.fulfilled,M.fulfilled,ce.rejected,H.rejected,M.rejected,N.rejected),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher($(...k),t=>({}));}}),kH={},DH=P({name:"viewState/vehicleOptions/Makes",initialState:kH,reducers:{},extraReducers:e=>{e.addCase(De.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r.makes)!=null?o:[];t[a.questionGroupId]=Oi(i);}).addCase(H.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=r!=null?r:[];t[o.questionGroupId]=Oi(a);}).addCase(fr.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r==null?void 0:r.makes)!=null?o:[];t[a.questionGroupId]=Oi(i);}),e.addMatcher($(H.rejected,M.pending,N.pending,M.fulfilled,N.rejected,N.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher($(...k),t=>({}));}}),LH={},NH=P({name:"viewState/vehicleOptions/Models",initialState:LH,reducers:{},extraReducers:e=>{e.addCase(De.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r.models)!=null?o:[];t[a.questionGroupId]=Si(i);}).addCase(ce.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=r!=null?r:[];t[o.questionGroupId]=Si(a);}).addCase(fr.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r==null?void 0:r.models)!=null?o:[];t[a.questionGroupId]=Si(i);}),e.addMatcher($(H.rejected,H.fulfilled,M.rejected,M.fulfilled,N.rejected,N.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher($(...k),t=>({}));}});var RH=Object.defineProperty,MH=Object.defineProperties,VH=Object.getOwnPropertyDescriptors,Nm=Object.getOwnPropertySymbols,FH=Object.prototype.hasOwnProperty,QH=Object.prototype.propertyIsEnumerable,Rm=(e,t,r)=>t in e?RH(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ts=(e,t)=>{for(var r in t||(t={}))FH.call(t,r)&&Rm(e,r,t[r]);if(Nm)for(var r of Nm(t))QH.call(t,r)&&Rm(e,r,t[r]);return e},UH=(e,t)=>MH(e,VH(t));const BH={},Mm=P({name:"viewState/vehicleOptions/Selection",initialState:BH,reducers:{setVehicleSelection:(e,{payload:t})=>{e[t.questionGroupId]=Ts(Ts({},e[t.questionGroupId]),t.selection);}},extraReducers:e=>{e.addCase(fr.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,{yearId:a,makeId:i,modelId:s,configurationId:c}=r;t[o.questionGroupId]=UH(Ts({},t[o.questionGroupId]),{yearId:a,makeId:i,modelId:s,configurationId:c});}),e.addMatcher($(...k),t=>({}));}}),{setVehicleSelection:GH}=Mm.actions;var zH=Object.defineProperty,WH=Object.defineProperties,HH=Object.getOwnPropertyDescriptors,Vm=Object.getOwnPropertySymbols,ZH=Object.prototype.hasOwnProperty,KH=Object.prototype.propertyIsEnumerable,Fm=(e,t,r)=>t in e?zH(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,hr=(e,t)=>{for(var r in t||(t={}))ZH.call(t,r)&&Fm(e,r,t[r]);if(Vm)for(var r of Vm(t))KH.call(t,r)&&Fm(e,r,t[r]);return e},qs=(e,t)=>WH(e,HH(t));const Qm=3,YH={},JH=P({name:"viewState/vehicleOptions/Vin",initialState:YH,reducers:{},extraReducers:e=>{e.addCase(N.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=r,i=a==null?void 0:a.parsedBody,s=n.arg;if(!i)return t;const c=Ci(i),u=(o=t[s.questionGroupId])!=null?o:hr({},ma);if(c.length===0){const l=u.errorCount+1;return t[s.questionGroupId]=qs(hr({},u),{errorCount:l,isAvailable:l<Qm,lastResponse:a}),t}t[s.questionGroupId]=qs(hr({},u),{errorCount:0,isAvailable:!0,lastResponse:void 0});}).addCase(N.rejected,(t,{meta:r})=>{var n;const o=r.arg,a=(n=t[o.questionGroupId])!=null?n:hr({},ma),i=a.errorCount+1;t[o.questionGroupId]=qs(hr({},a),{errorCount:i,isAvailable:i<Qm,lastResponse:void 0});}),e.addMatcher($(De.pending,ce.pending,H.pending,M.pending,De.rejected,De.fulfilled,ce.rejected,ce.fulfilled,H.rejected,H.fulfilled,M.rejected,M.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]=hr({},ma);}).addMatcher($(...k),t=>({}));}}),XH={},e7=P({name:"viewState/vehicleOptions/Years",initialState:XH,reducers:{},extraReducers:e=>{e.addCase(De.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r.years)!=null?o:[];t[a.questionGroupId]=Ai(i);}).addCase(M.fulfilled,(t,{payload:r,meta:n})=>{const o=n.arg,a=r!=null?r:[];t[o.questionGroupId]=Ai(a);}).addCase(fr.fulfilled,(t,{payload:r,meta:n})=>{var o;const a=n.arg,i=(o=r==null?void 0:r.years)!=null?o:[];t[a.questionGroupId]=Ai(i);}),e.addMatcher($(M.pending,N.pending,N.rejected,N.fulfilled),(t,{meta:r})=>{const n=r.arg;t[n.questionGroupId]={};}).addMatcher($(...k),t=>({}));}});var t7=combineReducers({vehicleOptions:combineReducers({configurations:xH.reducer,makes:DH.reducer,models:NH.reducer,selection:Mm.reducer,vin:JH.reducer,years:e7.reducer}),features:yH.reducer,quotes:km.reducer,addressValidation:Dh.reducer}),r7=Object.defineProperty,Um=Object.getOwnPropertySymbols,n7=Object.prototype.hasOwnProperty,o7=Object.prototype.propertyIsEnumerable,Bm=(e,t,r)=>t in e?r7(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Gm=(e,t)=>{for(var r in t||(t={}))n7.call(t,r)&&Bm(e,r,t[r]);if(Um)for(var r of Um(t))o7.call(t,r)&&Bm(e,r,t[r]);return e};const zm=Gm({},$n),a7=P({name:"status/remote/quotes",initialState:zm,reducers:{},extraReducers:e=>{e.addCase(Br,(t,{payload:r})=>Hm(t,r)).addCase(Uc,(t,{payload:r})=>{t.state=y.Loaded,t.updated_at=r.updatedAt;}).addCase(Ae,(t,{payload:r})=>{r.events.forEach(n=>{if(n.event===x.ApplicationForm)return Hm(t,n.data)});}),e.addMatcher($(...k),t=>Gm({},zm)).addMatcher($(...be),(t,{payload:r})=>{t.state=Wm(r);}).addMatcher($(...Ni),t=>{t.state=y.Error;});}}),Wm=e=>{let t=y.NotLoaded;return e.state===xt.Completed&&!e.taken_over&&!e.cloned?t=e.quote_requests_count===0?y.Loaded:y.Loading:e.state===xt.Quoted&&(t=e.finished_quote_requests_count===e.quote_requests_count?y.Loaded:y.Loading),t},Hm=(e,t)=>{const r=new Date(e.updated_at||0),n=new Date(t.updated_at);return n>r&&(e.state=Wm(t),e.updated_at=n),e},i7={},s7=P({name:"status/remote/refilters",initialState:i7,reducers:{},extraReducers:e=>{e.addMatcher($(nn.pending,ke.pending),(t,{meta:r})=>{t[r.arg.questionId]=y.Loading;}).addMatcher($(nn.fulfilled,ke.fulfilled),(t,{meta:r})=>{t[r.arg.questionId]=y.Loaded;}).addMatcher($(nn.rejected,ke.rejected),(t,{meta:r})=>{t[r.arg.questionId]=y.Error;}).addMatcher($(...k),t=>({}));}});var c7=combineReducers({vehicles:U6,answers:W6.reducer,application:X6.reducer,bindQuotes:tH.reducer,carriersProducts:aH.reducer,choices:sH.reducer,questionPools:dH.reducer,questionSets:gH.reducer,quotes:a7.reducer,refilters:s7.reducer}),u7=Object.defineProperty,l7=Object.defineProperties,d7=Object.getOwnPropertyDescriptors,Zm=Object.getOwnPropertySymbols,p7=Object.prototype.hasOwnProperty,f7=Object.prototype.propertyIsEnumerable,Km=(e,t,r)=>t in e?u7(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r,Ym=(e,t)=>{for(var r in t||(t={}))p7.call(t,r)&&Km(e,r,t[r]);if(Zm)for(var r of Zm(t))f7.call(t,r)&&Km(e,r,t[r]);return e},v7=(e,t)=>l7(e,d7(t));const g7=combineReducers({form:af.reducer,enrollments:v6.reducer,choices:p6.reducer,errors:Ih.reducer,pendingProcesses:_f.reducer,progress:b6.reducer,questionSets:Ug.reducer,quotes:ah.reducer,quoteBind:j6.reducer,carriersProducts:c6.reducer,refilters:I6.reducer}),h7=combineReducers({remote:c7,local:Oh.reducer});let m7="production";const Jm=Fb({reducer:{application:g7,auth:ph.reducer,queues:w6.reducer,config:bh.reducer,notifications:bf.reducer,viewState:t7,status:h7},middleware:e=>e({thunk:!0,immutableCheck:m7!=="production",serializableCheck:!1}).concat([...i6,...U5]),devTools:{name:`Bold Penguin SDK Store - ${Date.now()}`}}),b7=()=>{var e,t,r,n;return window&&!((e=window.BpSdk)!=null&&e.store)&&(window.BpSdk=v7(Ym({},(t=window.BpSdk)!=null?t:{}),{store:Jm})),(n=(r=window==null?void 0:window.BpSdk)==null?void 0:r.store)!=null?n:Ym({},Jm)};

exports.$L = $L;
exports.AH = AH;
exports.AR = AR;
exports.BR = BR;
exports.Bp = Bp;
exports.De = De;
exports.Do = Do;
exports.FR = FR;
exports.Fp = Fp;
exports.Fr = Fr;
exports.GH = GH;
exports.GR = GR;
exports.Gc = Gc;
exports.Gp = Gp;
exports.H = H;
exports.HR = HR;
exports.HW = HW;
exports.IL = IL;
exports.IR = IR;
exports.JR = JR;
exports.KL = KL;
exports.KR = KR;
exports.Lo = Lo;
exports.M = M;
exports.MN = MN;
exports.Mh = Mh;
exports.N = N;
exports.NR = NR;
exports.Nh = Nh;
exports.No = No;
exports.OH = OH;
exports.ON = ON;
exports.Oe = Oe;
exports.Pi = Pi;
exports.QR = QR;
exports.Qp = Qp;
exports.Qr = Qr;
exports.RR = RR;
exports.Rh = Rh;
exports.S5 = S5;
exports.SL = SL;
exports.TN = TN;
exports.TR = TR;
exports.Te = Te;
exports.UN = UN;
exports.UR = UR;
exports.Uc = Uc;
exports.Up = Up;
exports.VN = VN;
exports.VR = VR;
exports.WR = WR;
exports.We = We;
exports.XR = XR;
exports.Xt = Xt;
exports.YL = YL;
exports.YR = YR;
exports.ZR = ZR;
exports._8 = _8;
exports._L = _L;
exports._e = _e;
exports.aN = aN;
exports.an = an;
exports.b7 = b7;
exports.b8 = b8;
exports.bL = bL;
exports.bM = bM;
exports.cN = cN;
exports.cR = cR;
exports.ce = ce;
exports.cn = cn;
exports.d8 = d8;
exports.dR = dR;
exports.eM = eM;
exports.eN = eN;
exports.ee = ee;
exports.fR = fR;
exports.fr = fr;
exports.gL = gL;
exports.iN = iN;
exports.jN = jN;
exports.ke = ke;
exports.ko = ko;
exports.lR = lR;
exports.m8 = m8;
exports.mL = mL;
exports.mR = mR;
exports.me = me;
exports.n6 = n6;
exports.nN = nN;
exports.nn = nn;
exports.o6 = o6;
exports.oN = oN;
exports.on = on;
exports.p5 = p5;
exports.pt = pt;
exports.r6 = r6;
exports.rn = rn;
exports.sN = sN;
exports.sn = sn;
exports.sp = sp;
exports.tM = tM;
exports.tN = tN;
exports.tf = tf;
exports.uR = uR;
exports.up = up;
exports.vo = vo;
exports.wN = wN;
exports.w_ = w_;
exports.wa = wa;
exports.ws = ws;
exports.xN = xN;
exports.xo = xo;
exports.y8 = y8;
exports.zR = zR;
exports.zp = zp;
